!/////////////////////////////////////////////////////////////////////////////
!// Name:        m_milieu_fractal.f90
!// Purpose:     Gestion de milieux inhomogènes à granulosité fractal
!      Génération du milieu fractal
!// Author:      Samuel Besse, Christophe Guiffaut
!// Modified by:  Christophe Guiffaut
!// Created  :     Avril 2003
!// Lieu     : XLIM - Faculté des sciences - 123 Av. Albert Thomas - Limoges
!// Copyright:   (c) CNRS, Université de Limoges
!// Licence  : CECILL-C - Octobre 2006
!/////////////////////////////////////////////////////////////////////////////


! =========================================================================
!              Module module_milieu_fractal               
!    AUTEURS : Samuel Besse, Christophe Guiffaut
!    VERSION 1.0, DATE : Avril 2003
!
!      Gestion de milieux inhomogènes à granulosité fractal
!      Génération du milieu fractal
!
! =========================================================================
! == Type st_milieu_fractal                               ==
! ==========================================================
module m_milieu_fractal

use m_constantes, only : LEN_MOT_CLE, LEN_CHAINE
use m_entree, only : st_fichier, &
                                  lineread, &
				  ligne_precedente, &
				  ctrl_nb_mot_chaine, &
                                  lecture_stop
use m_maillage
use m_milieu_simple, only:st_milieu_simple,&
			   milieu_simple_initdefaut,&
			   milieu_simple_copy,&
			   milieu_simple_deallocate

implicit none
integer, parameter :: FRACTAL_NIT = 3
REAL, parameter :: FRACTAL_H = 0.5
! ==========================================================
! == Type milieu_fractal                            ==
! ==========================================================
type st_param_fractal
  logical :: flag_saturation
  real :: moyen
  integer :: graine
  real :: facteur 
  integer :: nit
  real :: h 
  real, dimension(2)  :: saturation,fact_sat  !facteur de saturation, deux bornes inf et sup
  integer, dimension(3) :: nxyz,ndxyz 
  real, dimension(3) :: rnxyz,rndxyz 
 
end type st_param_fractal

type st_milieu_fractal
  character(LEN=LEN_MOT_CLE) :: nom
  logical :: flag_eps, flag_sige,flag_volume
  type(st_param_fractal) :: eps_fractal, sige_fractal  
  real :: mu_moyen, sigh_moyen  ! paramètres magnétiques 
  real :: eps_moyen, sige_moyen  ! paramètres électriques
  type(st_milieu_simple)  :: t_ms

end type st_milieu_fractal
 
contains

! ==========================================================
! == Subroutine milieu_fractal_initdefaut           ==
! == Initialisation par defaut des variables contenant    ==
! == les parametres definis par l'utilisateur             ==
! == dans le fichier d'entree                             ==
! ==========================================================	
subroutine milieu_fractal_initdefaut(m)

type(st_milieu_fractal), intent(inout) :: m
m%flag_eps = .false.
m%flag_sige = .false.
m%flag_volume = .false. 
m%eps_moyen = 1.0
m%mu_moyen = 1.0
m%sige_moyen = 0.0
m%sigh_moyen = 0.0
call param_fractal_initdefaut(m%eps_fractal)
call param_fractal_initdefaut(m%sige_fractal)
call milieu_simple_initdefaut(m%t_ms)
end subroutine milieu_fractal_initdefaut

! ==========================================================
! == Subroutine param_fractal_initdefaut           ==
! == Initialisation par defaut des variables contenant    ==
! == les parametres definis par l'utilisateur             ==
! == dans le fichier d'entree                             ==
! ==========================================================	
subroutine param_fractal_initdefaut(m)

type(st_param_fractal), intent(inout) :: m
m%flag_saturation = .false.
m%nxyz(:) = 0 ; 
m%ndxyz(:) = 0;
m%rnxyz(:) = 0.0 ; 
m%rndxyz(:) = 0.0;

m%moyen = 0.0
m%graine = 0;
m%facteur = 0.0;
m%nit = FRACTAL_NIT; 
m%h = FRACTAL_H;
m%saturation(1) = 0.0;
m%saturation(2) = 1.0e8;
m%fact_sat(1:2) = 0;
end subroutine param_fractal_initdefaut


! ==========================================================
! == Subroutine milieu_fractal_copy                       ==
! ==========================================================
subroutine milieu_fractal_copy(m_in,m_out)
type(st_milieu_fractal), intent(in)  :: m_in
type(st_milieu_fractal), intent(out) :: m_out

m_out%nom = m_in%nom ;
m_out%flag_eps = m_in%flag_eps ;
m_out%flag_sige = m_in%flag_sige ;
m_out%flag_volume = m_in%flag_volume ;
m_out%eps_fractal = m_in%eps_fractal ;
m_out%sige_fractal = m_in%sige_fractal ;
m_out%mu_moyen = m_in%mu_moyen ;
m_out%sigh_moyen = m_in%sigh_moyen ;
m_out%eps_moyen = m_in%eps_moyen ;
m_out%sige_moyen = m_in%sige_moyen ;
call milieu_simple_copy(m_in%t_ms,m_out%t_ms)
end subroutine milieu_fractal_copy

! ==========================================================
! == Subroutine milieu_fractal_deallocate                 ==
! ==========================================================
subroutine milieu_fractal_deallocate(m)
type(st_milieu_fractal), intent(inout) :: m
! rien à faire
call milieu_simple_deallocate(m%t_ms)
end subroutine milieu_fractal_deallocate

! ==========================================================
! == Subroutine milieu_fractal_lecture              ==
! ==========================================================
subroutine milieu_fractal_lecture(f,m,tm)
!=== Déclaration des arguments ===
type(st_fichier), intent(inout) :: f
type(st_milieu_fractal), intent(inout) :: m
type(st_maillage), intent(inout) :: tm

!===  Déclaration des variables  ===
real  :: valeur
real  :: r1(6)
character(LEN=LEN_MOT_CLE) :: premier_mot, ch
integer :: status
logical :: flag_reel
!===   Début des traitements   ===
flag_reel = .false.
do
  premier_mot = lineread(f)

  if (premier_mot == f%nomfinbloc .or.premier_mot(1:6) == 'milieu' .or. &
      premier_mot(1:14) == 'milieu_fractal') then
     call ligne_precedente(f)
     exit
  end if
  if (premier_mot=='volume') then ! commun à epsilon et sige
    status = ctrl_nb_mot_chaine(f,f%chaine,7)
    read(f%chaine,*) ch, r1(1:6)
    if (flag_reel) then
      m%eps_fractal%rnxyz  = r1(1:3)
      m%eps_fractal%rndxyz = r1(4:6)
      !m%eps_fractal%nxyz   = maillage_Unif_ConvToCell3(tm,r1(1:3))-1
      !m%eps_fractal%ndxyz  = maillage_Unif_ConvToCell3(tm,r1(4:6))
      m%eps_fractal%nxyz   = nint(r1(1:3)/tm%dxyz) ! nbre de cellules total
      m%eps_fractal%ndxyz  = nint(r1(4:6)/tm%dxyz)
      
      where (m%eps_fractal%rnxyz ==0.0) m%eps_fractal%nxyz=0
      where (m%eps_fractal%rndxyz ==0.0) m%eps_fractal%ndxyz=0
      if (rang==RANG_MAITRE_AFF) &
        print '(T3,"fractal : nxyz,idxyz = ",6I5)',m%eps_fractal%nxyz,m%eps_fractal%ndxyz
    else
      m%eps_fractal%nxyz  = nint(r1(1:3))
      m%eps_fractal%ndxyz = nint(r1(4:6))
      m%eps_fractal%rnxyz = r1(1:3)*tm%dxyz
      m%eps_fractal%ndxyz = r1(4:6)*tm%dxyz
      !m%eps_fractal%rnxyz   = maillage_Unif_ConvToReal3(tm,nint(r1(1:3)+1))
      !m%eps_fractal%rndxyz  = maillage_Unif_ConvToReal3(tm,m%eps_fractal%ndxyz)
      where (m%eps_fractal%nxyz ==0.0) m%eps_fractal%rnxyz=0
      where (m%eps_fractal%ndxyz ==0.0) m%eps_fractal%rndxyz=0
      if (rang==RANG_MAITRE_AFF) &
        print '(T3,"fractal : rnxyz,ridxyz = ",6ES15.6)',m%eps_fractal%rnxyz,m%eps_fractal%rndxyz
    
    end if
    m%sige_fractal%nxyz = m%eps_fractal%nxyz
    m%sige_fractal%ndxyz = m%eps_fractal%ndxyz
    m%sige_fractal%rnxyz = m%eps_fractal%rnxyz
    m%sige_fractal%rndxyz = m%eps_fractal%rndxyz
    m%flag_volume = .true.
    cycle
  else if (premier_mot =='eps_saturation')  then
    status = ctrl_nb_mot_chaine(f,f%chaine,5)
    read(f%chaine,*) ch, m%eps_fractal%saturation(1:2),m%eps_fractal%fact_sat(1:2)
    m%eps_fractal%flag_saturation = .true.
    cycle
  else if (premier_mot =='sige_saturation')  then
    status = ctrl_nb_mot_chaine(f,f%chaine,5)
    read(f%chaine,*) ch, m%sige_fractal%saturation(1:2),m%sige_fractal%fact_sat(1:2)
    m%sige_fractal%flag_saturation = .true.
    cycle
  else if (premier_mot =='coordonnee_reelle') then
      flag_reel = .true.
  end if
  
  status = ctrl_nb_mot_chaine(f,f%chaine,2)
  select case(premier_mot)
  case('mu_moyen') 
    read(f%chaine,*) ch, m%mu_moyen
  case('eps_moyen') 
    read(f%chaine,*) ch, m%eps_moyen
    read(f%chaine,*) ch, m%eps_fractal%moyen
  case('eps_graine') 
    read(f%chaine,*) ch, m%eps_fractal%graine
  case('eps_facteur') 
    read(f%chaine,*) ch, m%eps_fractal%facteur
    m%flag_eps = .true.
  case('eps_nit') ; read(f%chaine,*) ch, m%eps_fractal%nit
  case('eps_h') ; read(f%chaine,*) ch, m%eps_fractal%h
  
  case('sigh_moyen') 
    read(f%chaine,*) ch, m%sigh_moyen
  case('sige_moyen') 
    read(f%chaine,*) ch, m%sige_moyen
    read(f%chaine,*) ch, m%sige_fractal%moyen
  case('sige_graine') 
    read(f%chaine,*) ch, m%sige_fractal%graine
  case('sige_facteur') 
    read(f%chaine,*) ch, m%sige_fractal%facteur
    m%flag_sige = .true.
  case('sige_nit') ; read(f%chaine,*) ch, m%sige_fractal%nit
  case('sige_h') ; read(f%chaine,*) ch, m%sige_fractal%h

  case default
      print '("Mot clé non reconnu : ",A)', premier_mot
      call lecture_stop(f)      
  end select
enddo
if (m%eps_moyen == 1.0 .and. m%sige_moyen == 0.0) then
  PRINT '("Pas de valeur défini pour eps_moyen ou sige_moyen")'
  call lecture_stop(f)
end if

if (m%eps_fractal%facteur == 0.0 .and. m%sige_fractal%facteur == 0.0) then
  PRINT '("Pas de valeur défini pour eps_facteur ou sige_facteur")'
  call lecture_stop(f)
end if

if (m%eps_fractal%graine == 0 .and. m%sige_fractal%graine == 0) then
  PRINT '("Pas de valeur défini pour eps_graine ou sige_graine")'
  call lecture_stop(f)
end if

call milieu_fractal_init(m)
end subroutine milieu_fractal_lecture


! ==========================================================
! == Subroutine milieu_simple_ecriture                    ==
! == Ecriture dans le fichier d'entrée...                 ==
! ==========================================================	
subroutine milieu_fractal_ecriture(f,m)

type(st_fichier), intent(inout):: f
type(st_milieu_fractal), intent(inout) :: m


if (m%flag_volume) then
!  write(f%unite,'(T3,A)') 'coordonnee_reelle'
  write(f%unite,'(T5,A,3x,6I5)') 'volume', m%eps_fractal%nxyz(1:3), &
                                   m%eps_fractal%ndxyz(1:3)
!  write(f%unite,'(T5,A,3x,6ES15.6E2)') 'volume', m%eps_fractal%rnxyz(1:3), &
!                                   m%eps_fractal%rndxyz(1:3)
end if

if (m%mu_moyen/=1.0)   write(f%unite,'(T5,A,3x,F9.3)') 'mu_moyen', m%mu_moyen
if (m%eps_moyen/=1.0)  write(f%unite,'(T5,A,3x,F9.3)') 'eps_moyen', m%eps_moyen
if (m%sigh_moyen/=0.0) write(f%unite,'(T5,A,3x,ES12.3)') 'sigh_moyen', m%sigh_moyen
if (m%sige_moyen/=0.0) write(f%unite,'(T5,A,3x,ES12.3)') 'sige_moyen', m%sige_moyen

if (m%flag_eps) then
  write(f%unite,'(T5,A,3x,I7)') 'eps_graine', m%eps_fractal%graine
  write(f%unite,'(T5,A,3x,F9.3)') 'eps_facteur', m%eps_fractal%facteur
!  if (m%eps_fractal%nit /= FRACTAL_NIT) &
    write(f%unite,'(T5,A,3x,I3)') 'eps_nit', m%eps_fractal%nit
!  if (m%eps_fractal%h /= FRACTAL_H) &
    write(f%unite,'(T5,A,3x,F8.5)') 'eps_h', m%eps_fractal%h
  
  if (m%eps_fractal%flag_saturation) &
    write(f%unite,'(T5,A,3x,4ES12.3)') 'eps_saturation', &
      m%eps_fractal%saturation(1:2),m%eps_fractal%fact_sat(1:2)
  
end if

if (m%flag_sige) then
  write(f%unite,'(T5,A,3x,I7)') 'sige_graine', m%sige_fractal%graine
  write(f%unite,'(T5,A,3x,ES12.3)') 'sige_facteur', m%sige_fractal%facteur
!  if (m%sige_fractal%nit /= FRACTAL_NIT) &
    write(f%unite,'(T5,A,3x,I3)') 'sige_nit', m%sige_fractal%nit
!  if (m%sige_fractal%h /= FRACTAL_H) &
    write(f%unite,'(T5,A,3x,F8.5)') 'sige_h', m%sige_fractal%h
    
  if (m%sige_fractal%flag_saturation) &
    write(f%unite,'(T5,A,3x,4ES12.3)') 'sige_saturation', &
      m%sige_fractal%saturation(1:2),m%sige_fractal%fact_sat(1:2)
end if


end subroutine milieu_fractal_ecriture


! ==========================================================
! == Subroutine milieu_fractal_lecture                    ==
! ==========================================================
subroutine milieu_fractal_init(m)
!=== Déclaration des arguments ===
type(st_milieu_fractal), intent(inout) :: m
!===   Début des traitements   ===

m%t_ms%eps_r = m%eps_moyen
m%t_ms%mu_r  = m%mu_moyen
m%t_ms%sig_e = m%sige_moyen
m%t_ms%sig_h = m%sigh_moyen
m%t_ms%nom   = m%nom
end subroutine milieu_fractal_init

! ==========================================================
! == Subroutine milieu_fractal_precalcul                  ==
! ==========================================================
SUBROUTINE milieu_fractal_precalcul(Taille_x, Taille_y, Taille_z,m,Hcut)

!=== Déclaration des arguments ===
integer :: Taille_x, Taille_y, Taille_z
type(st_param_fractal), intent(in) :: m

real, dimension(Taille_x, Taille_y, Taille_z),intent(out) :: Hcut

!===  Déclaration des variables  ===
integer, save :: num = 0
INTEGER :: Nit
integer :: NX, NY, NZ	! Nombre de motif
integer :: L,LX,LY,LZ
integer :: iNit, idx, idy, idz
integer :: iNX, iNY, iNZ
integer :: iixNit, iiyNit,iizNit
integer :: ix, iy, iz,iix, iiy, iiz
integer :: i,j,k
integer :: p1, p2, p3, p4
integer, dimension(3) :: taille,taille_st
integer,dimension(1) :: graine
real,parameter    :: cc=1.0 ! Les points situés sur les centre des cubes sont a cc   
real,parameter    :: cf=2.0 !                           centre des faces sont a cf   
real,parameter    :: ca=3.0 !                           centre des arretes sont a ca 
real    :: rand,h_f,facteur,val_moy
real, dimension(:,:,:), allocatable :: H
real  :: u_2iNit,u_2iNit_1, h_f_iNit_2
integer ::e2iNit_1
integer, dimension(3) :: nxyz,ndxyz

!===   Début des traitements   ===
! parametres de verification (pour debugger)
graine(1) = m%graine
Nit = m%nit
h_f = m%h
val_moy = m%moyen
facteur = m%facteur
nxyz = m%nxyz
ndxyz = m%ndxyz

taille(1) =Taille_x
taille(2) =Taille_y
taille(3) =Taille_z
taille_st = taille
where(m%nxyz>0 .and.(taille+m%ndxyz<=m%nxyz))
  taille = m%nxyz
end where

Nx = ceiling((Taille(1)-1)/(2.0**m%nit));
Ny = ceiling((Taille(2)-1)/(2.0**m%nit));
Nz = ceiling((Taille(3)-1)/(2.0**m%nit));
num=num+1 ! pour info
if (rang==RANG_MAITRE_AFF) then
  print '(T15,"--- Milieu fractal",I3," ---")',num
  print '(T5,"taille x,y,z = ",3I4)',Taille_x, Taille_y, Taille_z
  print '(T5,"décalage x,y,z = ",3I4)',m%ndxyz(1:3)
  print '(T5,"flag saturation = ",L2)',m%flag_saturation
  print '(T5,"saturation et facteur = ",4F7.3)',m%saturation(1:2),m%fact_sat(1:2)
  print '(T5,"volume total Nx, Ny, Nz = ",3I4)',taille(1:3)
  print '(T5,"Val moyenne, graine,facteur,nit,h_f = ",F9.5,I8,F9.5,I8,F9.5)',&
	       m%moyen,graine(1), m%facteur, m%nit,m%h     
  print '(T5,"Nx,Ny, Nz = ",3I3)', Nx,Ny,Nz
end if

do i=1,3
  if (taille_st(i)+m%ndxyz(i)>taille(i)) then
    print '("Erreur, volume structure fractal en dehors volume fractal !!! rang :",I2)',i
    stop 
  end if
end do

L=2**m%nit+1;  !% nombre de points d'un pave elementaire (longeur=L-1)
LX=NX*(L-1)+1;        !% nombre de points totales (longeur totale=LX-1)
LY=NY*(L-1)+1;
LZ=NZ*(L-1)+1;

!initialisation de la fonction aleatoire
call random_seed(put=graine)
allocate ( H (1:LX+2*L , 1:LY+2*L , 1:LZ+2*L) )
call random_number(H)

do iz=1,LZ+2*L
do iy=1,LY+2*L
do ix=1,LX+2*L
  H(ix,iy,iz)=2*(H(ix,iy,iz)-0.5)
end do; end do; end do

do iNit=1,Nit
idx=2**(Nit-iNit); !% représentatif de la Distance entre le centre et un coté du carré
idy=2**(Nit-iNit);
idz=2**(Nit-iNit);
  
  ! variable intermédiaire
  e2iNit_1   = 2**(iNit-1) !%Nombre de point / cellule elementaire
  u_2iNit    = (L-1)/2**(iNit) 
  u_2iNit_1  = (L-1)/2**(iNit-1)
  h_f_iNit_2 = 2*h_f**iNit
  
!% Square 
  do iNX=0,NX-1
  do iNY=0,NY-1
  do iNZ=0,NZ-1
    do iixNit=1,e2iNit_1   !%Nombre de point / cellule elementaire 
    do iiyNit=1,e2iNit_1
    do iizNit=1,e2iNit_1        
      ix=L+1+u_2iNit + (iixNit-1)*u_2iNit_1 + iNX*(L-1); !%Designe le milieu d'un carré
      iy=L+1+u_2iNit + (iiyNit-1)*u_2iNit_1 + iNY*(L-1);
      iz=L+1+u_2iNit + (iizNit-1)*u_2iNit_1 + iNZ*(L-1);

      call random_number(rand)
      H(ix,iy,iz)= ( (rand-0.5)*h_f_iNit_2+ &
                  0.125*(H(ix-idx,iy-idy,iz-idz)+ &
		       H(ix-idx,iy-idy,iz+idz)+ &
		       H(ix-idx,iy+idy,iz-idz)+ &
		       H(ix-idx,iy+idy,iz+idz)+ &
		       H(ix+idx,iy-idy,iz-idz)+ &
		       H(ix+idx,iy-idy,iz+idz)+ &
		       H(ix+idx,iy+idy,iz-idz)+ &
		       H(ix+idx,iy+idy,iz+idz)));   
    enddo
    enddo
    enddo
  enddo
  enddo
  enddo

!% Diamond
!% Milieu des faces du cube
  do iNX=0,NX-1
  do iNY=0,NY-1
  do iNZ=0,NZ-1
    do iixNit=1,e2iNit_1
    do iiyNit=1,e2iNit_1
    do iizNit=1,e2iNit_1
    
      ix=L+1+u_2iNit + (iixNit-1)*u_2iNit_1 + iNX*(L-1);
      iy=L+1+u_2iNit + (iiyNit-1)*u_2iNit_1 + iNY*(L-1);
      iz=L+1+u_2iNit + (iizNit-1)*u_2iNit_1 + iNZ*(L-1);

!% X+   
      call random_number(rand)
      H(ix+idx,iy,iz)= ((rand-0.5)*h_f_iNit_2+ &
                      0.25*(H(ix+idx,iy-idy,iz-idz)+ &
		           H(ix+idx,iy-idy,iz+idz)+ &
			   H(ix+idx,iy+idy,iz-idz)+ &
			   H(ix+idx,iy+idy,iz+idz)  &
		      ));
!% X-
      call random_number(rand)
      H(ix-idx,iy,iz)= ( (rand-0.5)*h_f_iNit_2+ &
                      0.25*(H(ix-idx,iy-idy,iz-idz)+ &
		      	   H(ix-idx,iy-idy,iz+idz)+ &
		      	   H(ix-idx,iy+idy,iz-idz)+ &
		      	   H(ix-idx,iy+idy,iz+idz)  &
		      ));
!% Y+			   	
      call random_number(rand)
      H(ix,iy+idy,iz)= ( (rand-0.5)*h_f_iNit_2+ &
                      0.25*(H(ix-idx,iy+idy,iz-idz)+ &
		      	   H(ix-idx,iy+idy,iz+idz)+ &
		      	   H(ix+idx,iy+idy,iz-idz)+ &
		      	   H(ix+idx,iy+idy,iz+idz)  &
		      ));
    
!% Y-      
      call random_number(rand)
      H(ix,iy-idy,iz)= ( (rand-0.5)*h_f_iNit_2+ &
                      0.25*(H(ix-idx,iy-idy,iz-idz)+ &
		      	   H(ix-idx,iy-idy,iz+idz)+ &
		      	   H(ix+idx,iy-idy,iz-idz)+ &
		      	   H(ix+idx,iy-idy,iz+idz)  &
		      ));
      
!% Z+      
      call random_number(rand)
      H(ix,iy,iz+idz)= ( (rand-0.5)*h_f_iNit_2+ &
                      0.25*(H(ix-idx,iy-idy,iz+idz)+ &
		      	   H(ix-idx,iy+idy,iz+idz)+ &
		      	   H(ix+idx,iy-idy,iz+idz)+ &
		      	   H(ix+idx,iy+idy,iz+idz)  &
		      ));

!% Z-
      call random_number(rand)
      H(ix,iy,iz-idz)= ( (rand-0.5)*h_f_iNit_2+ &
                      0.25*(H(ix-idx,iy-idy,iz-idz)+ &
		      	   H(ix-idx,iy+idy,iz-idz)+ &
		      	   H(ix+idx,iy-idy,iz-idz)+ &
		      	   H(ix+idx,iy+idy,iz-idz)  &
		      ));
    enddo
    enddo
    enddo
  enddo
  enddo
  enddo

!% Diamond
!% Milieux des arretes

  do iNX=0,NX-1
  do iNY=0,NY-1
  do iNZ=0,NZ-1
    do iixNit=1,e2iNit_1
    do iiyNit=1,e2iNit_1
    do iizNit=1,e2iNit_1
    
      ix=L+1+u_2iNit + (iixNit-1)*u_2iNit_1 + iNX*(L-1);
      iy=L+1+u_2iNit + (iiyNit-1)*u_2iNit_1 + iNY*(L-1);
      iz=L+1+u_2iNit + (iizNit-1)*u_2iNit_1 + iNZ*(L-1);

!%(1.0/2, 0, 0) => 1
      p1=1;p2=1;p3=1;p4=1;
      if ((iy-idy)==L+1) p1=0;
      if ((iz-idz)==L+1) p3=0;
      call random_number(rand)
      H(ix, iy-idy, iz-idz)= ( (rand-0.5)*h_f_iNit_2+ &
                      1.0/(p1+p2+p3+p4+2)*( &
		         p1*H(ix    , iy-idy-idy, iz-idz    ) &
		      	+p2*H(ix    , iy-idy+idy, iz-idz    ) &
		      	+p3*H(ix    , iy-idy    , iz-idz-idz) &
		      	+p4*H(ix    , iy-idy    , iz-idz+idz) &
		      	+   H(ix-idx, iy-idy    , iz-idz    ) &
		      	+   H(ix+idx, iy-idy    , iz-idz    ) &
		      ));      

!%(1/2, 0, 1) => 2
      p1=1;p2=1;p3=1;p4=1;
      if ((iy-idy)==L+1) p1=0;
      if ((iz+idz)==L+LZ) p4=0;
      call random_number(rand)
      H(ix, iy-idy, iz+idz)= ( (rand-0.5)*h_f_iNit_2+ &
                       1.0/(p1+p2+p3+p4+2)*( &
		          p1*H(ix    , iy-idy-idy, iz+idz    ) &
  	 	         +p2*H(ix    , iy-idy+idy, iz+idz    ) &
		      	 +p3*H(ix    , iy-idy	 , iz+idz-idz) &
		      	 +p4*H(ix    , iy-idy	 , iz+idz+idz) &
		      	 +   H(ix-idx, iy-idy    , iz+idz    ) &
		      	 +   H(ix+idx, iy-idy    , iz+idz    ) &
		      ));      
 
!%(1/2, 1, 0) => 3
      p1=1;p2=1;p3=1;p4=1;
      if ((iy+idy)==L+LY) p2=0;
      if ((iz-idz)==L+1) p3=0;
      call random_number(rand)
      H(ix, iy+idy, iz-idz)= ( (rand-0.5)*h_f_iNit_2+ &
                      1.0/(p1+p2+p3+p4+2)*( &
		         p1*H(ix    , iy+idy-idy, iz-idz    ) &
		      	+p2*H(ix    , iy+idy+idy, iz-idz    ) &
		      	+p3*H(ix    , iy+idy    , iz-idz-idz) &
		      	+p4*H(ix    , iy+idy    , iz-idz+idz) &
		      	+   H(ix-idx, iy+idy    , iz-idz    ) &
		      	+   H(ix+idx, iy+idy    , iz-idz    ) &
		      ));      

!%(1.0/2, 1, 1) => 4
      p1=1;p2=1;p3=1;p4=1;
      if ((iy+idy)==L+LY) p2=0;
      if ((iz+idz)==L+LZ) p4=0;
      call random_number(rand)
      H(ix, iy+idy, iz+idz)= ( (rand-0.5)*h_f_iNit_2+ &
                      1.0/(p1+p2+p3+p4+2)*( &
		         p1*H(ix    , iy+idy-idy, iz+idz    ) &
		        +p2*H(ix    , iy+idy+idy, iz+idz    ) &
		        +p3*H(ix    , iy+idy    , iz+idz-idz) &
		        +p4*H(ix    , iy+idy    , iz+idz+idz) &
		        +   H(ix-idx, iy+idy	, iz+idz    ) &
		        +   H(ix+idx, iy+idy	, iz+idz    ) &
		      ));      

!%(0, 1.0/2, 0) => 5
      p1=1;p2=1;p3=1;p4=1;
      if ((ix-idx)==L+1) p1=0;
      if ((iz-idz)==L+1) p3=0;
      call random_number(rand)
      H(ix-idx, iy, iz-idz)= ( (rand-0.5)*h_f_iNit_2+ &
                      1.0/(p1+p2+p3+p4+2)*( &
		         p1*H(ix-idx-idx, iy    , iz-idz    ) &
		        +p2*H(ix-idx+idx, iy    , iz-idz    ) &
		        +p3*H(ix-idx    , iy    , iz-idz-idz) &
		        +p4*H(ix-idx    , iy    , iz-idz+idz) &
		        +   H(ix-idx    , iy-idy, iz-idz    ) &
		        +   H(ix-idx    , iy+idy, iz-idz    ) &
		      ));      

!%(0, 1/2, 1) => 6
      p1=1;p2=1;p3=1;p4=1;
      if ((ix-idx)==L+1) p1=0;
      if ((iz+idz)==L+LZ) p4=0;
      call random_number(rand)
      H(ix-idx, iy, iz+idz)= ( (rand-0.5)*h_f_iNit_2+ &
                      1.0/(p1+p2+p3+p4+2)*( &
		         p1*H(ix-idx-idx, iy    , iz+idz    ) &
		        +p2*H(ix-idx+idx, iy    , iz+idz    ) &
		        +p3*H(ix-idx    , iy    , iz+idz-idz) &
		        +p4*H(ix-idx    , iy    , iz+idz+idz) &
		        +   H(ix-idx    , iy-idy, iz+idz    ) &
		        +   H(ix-idx    , iy+idy, iz+idz    ) &
		      ));      

!%(1, 1.0/2, 0) => 7 
      p1=1;p2=1;p3=1;p4=1;
      if ((ix+idx)==L+LX) p2=0;
      if ((iz-idz)==L+1) p3=0;
      call random_number(rand)
      H(ix+idx, iy, iz-idz)= ( (rand-0.5)*h_f_iNit_2+ &
                      1.0/(p1+p2+p3+p4+2)*( &
		         p1*H(ix+idx-idx, iy	, iz-idz    ) &
		        +p2*H(ix+idx+idx, iy	, iz-idz    ) &
		        +p3*H(ix+idx	, iy	, iz-idz-idz) &
		        +p4*H(ix+idx	, iy	, iz-idz+idz) &
		        +   H(ix+idx	, iy-idy, iz-idz    ) &
		        +   H(ix+idx	, iy+idy, iz-idz    ) &
		      ));      

!%(1, 1.0/2, 1) => 8 
      p1=1;p2=1;p3=1;p4=1;
      if ((ix+idx)==L+LX) p2=0;
      if ((iz+idz)==L+LZ) p4=0;
      call random_number(rand)
      H(ix+idx, iy, iz+idz)= ( (rand-0.5)*h_f_iNit_2+ &
                      1.0/(p1+p2+p3+p4+2)*( &
		         p1*H(ix+idx-idx, iy	, iz+idz    ) &
		        +p2*H(ix+idx+idx, iy	, iz+idz    ) &
		        +p3*H(ix+idx	, iy	, iz+idz-idz) &
		        +p4*H(ix+idx	, iy	, iz+idz+idz) &
		        +   H(ix+idx	, iy-idy, iz+idz    ) &
		        +   H(ix+idx	, iy+idy, iz+idz    ) &
		      ));      

!%(0, 0, 1.0/2) => 9 
      p1=1;p2=1;p3=1;p4=1;
      if ((ix-idx)==L+1) p1=0;
      if ((iy-idy)==L+1) p3=0;
      call random_number(rand)
      H(ix-idx,iy-idy,iz)= ( (rand-0.5)*h_f_iNit_2+ &
                      1.0/(p1+p2+p3+p4+2)*( &
		         p1*H(ix-idx-idx, iy-idy    , iz    ) &
		        +p2*H(ix-idx+idx, iy-idy    , iz    ) &
		        +p3*H(ix-idx	, iy-idy-idy, iz    ) &
		        +p4*H(ix-idx	, iy-idy+idy, iz    ) &
		        +   H(ix-idx	, iy-idy    , iz-idz) &
		        +   H(ix-idx	, iy-idy    , iz+idz) &
		      ));      

!%(0, 1, 1.0/2) =>10 
      p1=1;p2=1;p3=1;p4=1;
      if ((ix-idx)==L+1) p1=0;
      if ((iy+idy)==L+LY) p4=0;
      call random_number(rand)
      H(ix-idx,iy+idy,iz)= ( (rand-0.5)*h_f_iNit_2+ &
                      1.0/(p1+p2+p3+p4+2)*( &
		         p1*H(ix-idx-idx, iy+idy    , iz    ) &
		        +p2*H(ix-idx+idx, iy+idy    , iz    ) &
		        +p3*H(ix-idx	, iy+idy-idy, iz    ) &
		        +p4*H(ix-idx	, iy+idy+idy, iz    ) &
		        +   H(ix-idx	, iy+idy    , iz-idz) &
		        +   H(ix-idx	, iy+idy    , iz+idz) &
		      ));

!%(1, 0, 1.0/2) =>11 
      p1=1;p2=1;p3=1;p4=1;
      if ((ix+idx)==L+LX) p2=0;
      if ((iy-idy)==L+1) p3=0;
      call random_number(rand)
      H(ix+idx,iy-idy,iz)= ( (rand-0.5)*h_f_iNit_2+ &
                      1.0/(p1+p2+p3+p4+2)*( &
		         p1*H(ix+idx-idx, iy-idy    , iz    ) &
		        +p2*H(ix+idx+idx, iy-idy    , iz    ) &
		        +p3*H(ix+idx	, iy-idy-idy, iz    ) &
		        +p4*H(ix+idx	, iy-idy+idy, iz    ) &
		        +   H(ix+idx	, iy-idy    , iz-idz) &
		        +   H(ix+idx	, iy-idy    , iz+idz) &
		      ));      

!%(1, 1, 1.0/2) =>12 
      p1=1;p2=1;p3=1;p4=1;
      if ((ix+idx)==L+LX) p2=0;
      if ((iy+idy)==L+LY) p4=0;
      call random_number(rand)
      H(ix+idx,iy+idy,iz)= ( (rand-0.5)*h_f_iNit_2+ &
                      1.0/(p1+p2+p3+p4+2)*( &
		         p1*H(ix+idx-idx, iy+idy    , iz    ) &
		        +p2*H(ix+idx+idx, iy+idy    , iz    ) &
		        +p3*H(ix+idx	, iy+idy-idy, iz    ) &
		        +p4*H(ix+idx	, iy+idy+idy, iz    ) &
		        +   H(ix+idx	, iy+idy    , iz-idz) &
		        +   H(ix+idx	, iy+idy    , iz+idz) &
		      ));      



    end do
    end do
    end do
  end do
  end do
  end do
end do  !Nit


!Hcut(:,:,:) = H(L+1:L+Taille_x,L+1:L+Taille_y,L+1:L+Taille_z);
!Hcut(:,:,:) = facteur*Hcut(:,:,:) + val_moy

! Cas où le milieu doit etre saturé
if (m%flag_saturation) then
  do iz=1,taille_z
  do iy=1,taille_y
  do ix=1,taille_x
    Hcut(ix,iy,iz) = H(ix+L+ndxyz(1),iy+L+ndxyz(2),iz+L+ndxyz(3));

    if (Hcut(ix,iy,iz)>m%saturation(2)) then
      Hcut(ix,iy,iz) = m%saturation(2)+ (Hcut(ix,iy,iz)-m%saturation(2))*m%fact_sat(2)
    else if (Hcut(ix,iy,iz)<m%saturation(1)) then
      Hcut(ix,iy,iz) = m%saturation(1)+ (Hcut(ix,iy,iz)-m%saturation(1))*m%fact_sat(1)
    end if
  end do; end do; end do
  
else
  do iz=1,taille_z
  do iy=1,taille_y
  do ix=1,taille_x
    Hcut(ix,iy,iz) = H(ix+L+ndxyz(1),iy+L+ndxyz(2),iz+L+ndxyz(3));
  end do; end do; end do
end if

do iz=1,taille_z
do iy=1,taille_y
do ix=1,taille_x
  Hcut(ix,iy,iz) = facteur*Hcut(ix,iy,iz) + val_moy
end do; end do; end do

!----------- Affichage Statistiques ------------------
if (rang==RANG_MAITRE_AFF) then
  print '(T5,"Valeur moyenne finale = ",F10.7)',SUM(Hcut)/(Taille_x*Taille_y*Taille_z)
  print '(T5,"Valeur min = ",F8.6)',minval(Hcut)
  print '(T5,"Valeur max = ",F8.6)',MAXVAL(Hcut)
end if

deallocate (H)
end SUBROUTINE milieu_fractal_precalcul

! ==========================================================
! == Subroutine milieu_fractal_precalcul                    ==
! == ancienne version, on la préserve comme référence
! ==========================================================
function milieu_fractal_precalcul_old2(Taille_x, Taille_y, Taille_z,m) result(Hcut)

implicit none

!=== Déclaration des arguments ===
integer, intent(in) :: Taille_x, Taille_y, Taille_z
type(st_param_fractal), intent(in) :: m
real, dimension(Taille_x, Taille_y, Taille_z) :: Hcut

!===  Déclaration des variables  ===
integer, save :: num = 0
integer :: NX, NY, NZ	! Nombre de motif
integer :: L,LX,LY,LZ
integer :: Nit,iNit, idx, idy, idz
integer :: iNX, iNY, iNZ
integer :: iixNit, iiyNit,iizNit
integer :: i,ix, iy, iz
integer :: p1, p2, p3, p4
integer, dimension(3) :: taille,taille_st
integer,parameter :: v=0, va=0 ! verification active a l'état 1
integer,dimension(1) :: graine
real,parameter    :: cc=1.0 ! Les points situés sur les centre des cubes sont a cc   
real,parameter    :: cf=2.0 !                           centre des faces sont a cf   
real,parameter    :: ca=3.0 !                           centre des arretes sont a ca 
real    :: rand
real    :: h_f,facteur,val_moy
integer, dimension(3) :: nxyz,ndxyz
real, dimension(:,:,:), allocatable :: H

!===   Début des traitements   ===
! parametres de verification (pour debugger)

graine(1) = m%graine
Nit = m%nit
h_f = m%h
val_moy = m%moyen
facteur = m%facteur
nxyz = m%nxyz
ndxyz = m%ndxyz



taille(1) =Taille_x
taille(2) =Taille_y
taille(3) =Taille_z
taille_st = taille
where(nxyz>0 .and.(taille+ndxyz<=nxyz))
  taille = nxyz
end where

Nx = ceiling((Taille(1)-1)/(2.0**Nit));
Ny = ceiling((Taille(2)-1)/(2.0**Nit));
Nz = ceiling((Taille(3)-1)/(2.0**Nit));
num=num+1 ! pour info
if (rang==RANG_MAITRE_AFF) then
  print '(T15,"--- Milieu fractal",I3," ---")',num
  print '(T5,"taille x,y,z = ",3I4)',Taille_x, Taille_y, Taille_z
  print '(T5,"décalage x,y,z = ",3I4)',ndxyz(1:3)
  print '(T5,"volume total Nx, Ny, Nz = ",3I4)',taille(1:3)
  print '(T5,"Val moyenne, graine,facteur,nit,h_f = ",F9.5,I8,F9.5,I8,F9.5)',&
	       val_moy,graine(1), facteur, Nit,h_f     
  print '(T5,"Nx,Ny, Nz = ",3I3)', Nx,Ny,Nz
end if

do i=1,3
  if (taille_st(i)+ndxyz(i)>taille(i)) then
    print '("Erreur, volume structure fractal en dehors volume fractal !!! rang :",I2)',i
    stop 
  end if
end do

L=2**Nit+1;  !% nombre de points d'un pave elementaire (longeur=L-1)
LX=NX*(L-1)+1;        !% nombre de points totales (longeur totale=LX-1)
LY=NY*(L-1)+1;
LZ=NZ*(L-1)+1;

!initialisation de la fonction aleatoire
call random_seed(put=graine)
allocate ( H (1:LX+2*L , 1:LY+2*L , 1:LZ+2*L) )
call random_number(H)

do iz=1,LZ+2*L
do iy=1,LY+2*L
do ix=1,LX+2*L
  H(ix,iy,iz)=2*(H(ix,iy,iz)-0.5)
end do; end do; end do

do iNit=1,Nit
idx=2**(Nit-iNit); !% représentatif de la Distance entre le centre et un coté du carré
idy=2**(Nit-iNit);
idz=2**(Nit-iNit);
!% Square 
  do iNX=0,NX-1
  do iNY=0,NY-1
  do iNZ=0,NZ-1
    do iixNit=1,2**(iNit-1)   !%Nombre de point / cellule elementaire 
    do iiyNit=1,2**(iNit-1)
    do iizNit=1,2**(iNit-1)        
      ix=L+1+(L-1)/2**(iNit)+(iixNit-1)*(L-1)/2**(iNit-1)+iNX*(L-1); !%Designe le milieu d'un carré
      iy=L+1+(L-1)/2**(iNit)+(iiyNit-1)*(L-1)/2**(iNit-1)+iNY*(L-1);
      iz=L+1+(L-1)/2**(iNit)+(iizNit-1)*(L-1)/2**(iNit-1)+iNZ*(L-1);

      call random_number(rand)
      H(ix,iy,iz)=cc*v+(1-v)*(2*(rand-0.5)*h_f**iNit+ &
                  (1.0/8.0)*(H(ix-idx,iy-idy,iz-idz)+ &
		       H(ix-idx,iy-idy,iz+idz)+ &
		       H(ix-idx,iy+idy,iz-idz)+ &
		       H(ix-idx,iy+idy,iz+idz)+ &
		       H(ix+idx,iy-idy,iz-idz)+ &
		       H(ix+idx,iy-idy,iz+idz)+ &
		       H(ix+idx,iy+idy,iz-idz)+ &
		       H(ix+idx,iy+idy,iz+idz)));   
    enddo
    enddo
    enddo
  enddo
  enddo
  enddo

!% Diamond
!% Milieu des faces du cube
  do iNX=0,NX-1
  do iNY=0,NY-1
  do iNZ=0,NZ-1
    do iixNit=1,2**(iNit-1)
    do iiyNit=1,2**(iNit-1)
    do iizNit=1,2**(iNit-1)
    
      ix=L+1+(L-1)/2**(iNit)+(iixNit-1)*(L-1)/2**(iNit-1)+iNX*(L-1);
      iy=L+1+(L-1)/2**(iNit)+(iiyNit-1)*(L-1)/2**(iNit-1)+iNY*(L-1);
      iz=L+1+(L-1)/2**(iNit)+(iizNit-1)*(L-1)/2**(iNit-1)+iNZ*(L-1);

!% X+   
      call random_number(rand)
      H(ix+idx,iy,iz)=cf*v+(1-v)*(2*(rand-0.5)*h_f**iNit+ &
                      (1.0/4.0)*(H(ix+idx,iy-idy,iz-idz)+ &
		           H(ix+idx,iy-idy,iz+idz)+ &
			   H(ix+idx,iy+idy,iz-idz)+ &
			   H(ix+idx,iy+idy,iz+idz)  &
		      ));
!% X-
      call random_number(rand)
      H(ix-idx,iy,iz)=cf*v+(1-v)*(2*(rand-0.5)*h_f**iNit+ &
                      (1.0/4.0)*(H(ix-idx,iy-idy,iz-idz)+ &
		      	   H(ix-idx,iy-idy,iz+idz)+ &
		      	   H(ix-idx,iy+idy,iz-idz)+ &
		      	   H(ix-idx,iy+idy,iz+idz)  &
		      ));
!% Y+			   	
      call random_number(rand)
      H(ix,iy+idy,iz)=cf*v+(1-v)*(2*(rand-0.5)*h_f**iNit+ &
                      (1.0/4.0)*(H(ix-idx,iy+idy,iz-idz)+ &
		      	   H(ix-idx,iy+idy,iz+idz)+ &
		      	   H(ix+idx,iy+idy,iz-idz)+ &
		      	   H(ix+idx,iy+idy,iz+idz)  &
		      ));
    
!% Y-      
      call random_number(rand)
      H(ix,iy-idy,iz)=cf*v+(1-v)*(2*(rand-0.5)*h_f**iNit+ &
                      (1.0/4.0)*(H(ix-idx,iy-idy,iz-idz)+ &
		      	   H(ix-idx,iy-idy,iz+idz)+ &
		      	   H(ix+idx,iy-idy,iz-idz)+ &
		      	   H(ix+idx,iy-idy,iz+idz)  &
		      ));
      
!% Z+      
      call random_number(rand)
      H(ix,iy,iz+idz)=cf*v+(1-v)*(2*(rand-0.5)*h_f**iNit+ &
                      (1.0/4.0)*(H(ix-idx,iy-idy,iz+idz)+ &
		      	   H(ix-idx,iy+idy,iz+idz)+ &
		      	   H(ix+idx,iy-idy,iz+idz)+ &
		      	   H(ix+idx,iy+idy,iz+idz)  &
		      ));

!% Z-
      call random_number(rand)
      H(ix,iy,iz-idz)=cf*v+(1-v)*(2*(rand-0.5)*h_f**iNit+ &
                      (1.0/4.0)*(H(ix-idx,iy-idy,iz-idz)+ &
		      	   H(ix-idx,iy+idy,iz-idz)+ &
		      	   H(ix+idx,iy-idy,iz-idz)+ &
		      	   H(ix+idx,iy+idy,iz-idz)  &
		      ));
    enddo
    enddo
    enddo
  enddo
  enddo
  enddo

!% Diamond
!% Milieux des arretes

  do iNX=0,NX-1
  do iNY=0,NY-1
  do iNZ=0,NZ-1
    do iixNit=1,2**(iNit-1)
    do iiyNit=1,2**(iNit-1)
    do iizNit=1,2**(iNit-1)
    
      ix=L+1+(L-1)/2**(iNit)+(iixNit-1)*(L-1)/2**(iNit-1)+iNX*(L-1);
      iy=L+1+(L-1)/2**(iNit)+(iiyNit-1)*(L-1)/2**(iNit-1)+iNY*(L-1);
      iz=L+1+(L-1)/2**(iNit)+(iizNit-1)*(L-1)/2**(iNit-1)+iNZ*(L-1);

!%(1.0/2, 0, 0) => 1
      p1=1;p2=1;p3=1;p4=1;
      if ((iy-idy)==L+1) p1=0;
      if ((iz-idz)==L+1) p3=0;
      call random_number(rand)
      H(ix, iy-idy, iz-idz)=ca*va+(1-va)*(2*(rand-0.5)*h_f**iNit+ &
                      1.0/(p1+p2+p3+p4+2)*( &
		         p1*H(ix    , iy-idy-idy, iz-idz    ) &
		      	+p2*H(ix    , iy-idy+idy, iz-idz    ) &
		      	+p3*H(ix    , iy-idy    , iz-idz-idz) &
		      	+p4*H(ix    , iy-idy    , iz-idz+idz) &
		      	+   H(ix-idx, iy-idy    , iz-idz    ) &
		      	+   H(ix+idx, iy-idy    , iz-idz    ) &
		      ));      

!%(1/2, 0, 1) => 2
      p1=1;p2=1;p3=1;p4=1;
      if ((iy-idy)==L+1) p1=0;
      if ((iz+idz)==L+LZ) p4=0;
      call random_number(rand)
      H(ix, iy-idy, iz+idz)=ca*va+(1-va)*(2*(rand-0.5)*h_f**iNit+ &
                       1.0/(p1+p2+p3+p4+2)*( &
		          p1*H(ix    , iy-idy-idy, iz+idz    ) &
  	 	         +p2*H(ix    , iy-idy+idy, iz+idz    ) &
		      	 +p3*H(ix    , iy-idy	 , iz+idz-idz) &
		      	 +p4*H(ix    , iy-idy	 , iz+idz+idz) &
		      	 +   H(ix-idx, iy-idy    , iz+idz    ) &
		      	 +   H(ix+idx, iy-idy    , iz+idz    ) &
		      ));      
 
!%(1/2, 1, 0) => 3
      p1=1;p2=1;p3=1;p4=1;
      if ((iy+idy)==L+LY) p2=0;
      if ((iz-idz)==L+1) p3=0;
      call random_number(rand)
      H(ix, iy+idy, iz-idz)=ca*va+(1-va)*(2*(rand-0.5)*h_f**iNit+ &
                      1.0/(p1+p2+p3+p4+2)*( &
		         p1*H(ix    , iy+idy-idy, iz-idz    ) &
		      	+p2*H(ix    , iy+idy+idy, iz-idz    ) &
		      	+p3*H(ix    , iy+idy    , iz-idz-idz) &
		      	+p4*H(ix    , iy+idy    , iz-idz+idz) &
		      	+   H(ix-idx, iy+idy    , iz-idz    ) &
		      	+   H(ix+idx, iy+idy    , iz-idz    ) &
		      ));      

!%(1.0/2, 1, 1) => 4
      p1=1;p2=1;p3=1;p4=1;
      if ((iy+idy)==L+LY) p2=0;
      if ((iz+idz)==L+LZ) p4=0;
      call random_number(rand)
      H(ix, iy+idy, iz+idz)=ca*va+(1-va)*(2*(rand-0.5)*h_f**iNit+ &
                      1.0/(p1+p2+p3+p4+2)*( &
		         p1*H(ix    , iy+idy-idy, iz+idz    ) &
		        +p2*H(ix    , iy+idy+idy, iz+idz    ) &
		        +p3*H(ix    , iy+idy    , iz+idz-idz) &
		        +p4*H(ix    , iy+idy    , iz+idz+idz) &
		        +   H(ix-idx, iy+idy	, iz+idz    ) &
		        +   H(ix+idx, iy+idy	, iz+idz    ) &
		      ));      

!%(0, 1.0/2, 0) => 5
      p1=1;p2=1;p3=1;p4=1;
      if ((ix-idx)==L+1) p1=0;
      if ((iz-idz)==L+1) p3=0;
      call random_number(rand)
      H(ix-idx, iy, iz-idz)=ca*va+(1-va)*(2*(rand-0.5)*h_f**iNit+ &
                      1.0/(p1+p2+p3+p4+2)*( &
		         p1*H(ix-idx-idx, iy    , iz-idz    ) &
		        +p2*H(ix-idx+idx, iy    , iz-idz    ) &
		        +p3*H(ix-idx    , iy    , iz-idz-idz) &
		        +p4*H(ix-idx    , iy    , iz-idz+idz) &
		        +   H(ix-idx    , iy-idy, iz-idz    ) &
		        +   H(ix-idx    , iy+idy, iz-idz    ) &
		      ));      

!%(0, 1/2, 1) => 6
      p1=1;p2=1;p3=1;p4=1;
      if ((ix-idx)==L+1) p1=0;
      if ((iz+idz)==L+LZ) p4=0;
      call random_number(rand)
      H(ix-idx, iy, iz+idz)=ca*va+(1-va)*(2*(rand-0.5)*h_f**iNit+ &
                      1.0/(p1+p2+p3+p4+2)*( &
		         p1*H(ix-idx-idx, iy    , iz+idz    ) &
		        +p2*H(ix-idx+idx, iy    , iz+idz    ) &
		        +p3*H(ix-idx    , iy    , iz+idz-idz) &
		        +p4*H(ix-idx    , iy    , iz+idz+idz) &
		        +   H(ix-idx    , iy-idy, iz+idz    ) &
		        +   H(ix-idx    , iy+idy, iz+idz    ) &
		      ));      

!%(1, 1.0/2, 0) => 7 
      p1=1;p2=1;p3=1;p4=1;
      if ((ix+idx)==L+LX) p2=0;
      if ((iz-idz)==L+1) p3=0;
      call random_number(rand)
      H(ix+idx, iy, iz-idz)=ca*va+(1-va)*(2*(rand-0.5)*h_f**iNit+ &
                      1.0/(p1+p2+p3+p4+2)*( &
		         p1*H(ix+idx-idx, iy	, iz-idz    ) &
		        +p2*H(ix+idx+idx, iy	, iz-idz    ) &
		        +p3*H(ix+idx	, iy	, iz-idz-idz) &
		        +p4*H(ix+idx	, iy	, iz-idz+idz) &
		        +   H(ix+idx	, iy-idy, iz-idz    ) &
		        +   H(ix+idx	, iy+idy, iz-idz    ) &
		      ));      

!%(1, 1.0/2, 1) => 8 
      p1=1;p2=1;p3=1;p4=1;
      if ((ix+idx)==L+LX) p2=0;
      if ((iz+idz)==L+LZ) p4=0;
      call random_number(rand)
      H(ix+idx, iy, iz+idz)=ca*va+(1-va)*(2*(rand-0.5)*h_f**iNit+ &
                      1.0/(p1+p2+p3+p4+2)*( &
		         p1*H(ix+idx-idx, iy	, iz+idz    ) &
		        +p2*H(ix+idx+idx, iy	, iz+idz    ) &
		        +p3*H(ix+idx	, iy	, iz+idz-idz) &
		        +p4*H(ix+idx	, iy	, iz+idz+idz) &
		        +   H(ix+idx	, iy-idy, iz+idz    ) &
		        +   H(ix+idx	, iy+idy, iz+idz    ) &
		      ));      

!%(0, 0, 1.0/2) => 9 
      p1=1;p2=1;p3=1;p4=1;
      if ((ix-idx)==L+1) p1=0;
      if ((iy-idy)==L+1) p3=0;
      call random_number(rand)
      H(ix-idx,iy-idy,iz)=ca*va+(1-va)*(2*(rand-0.5)*h_f**iNit+ &
                      1.0/(p1+p2+p3+p4+2)*( &
		         p1*H(ix-idx-idx, iy-idy    , iz    ) &
		        +p2*H(ix-idx+idx, iy-idy    , iz    ) &
		        +p3*H(ix-idx	, iy-idy-idy, iz    ) &
		        +p4*H(ix-idx	, iy-idy+idy, iz    ) &
		        +   H(ix-idx	, iy-idy    , iz-idz) &
		        +   H(ix-idx	, iy-idy    , iz+idz) &
		      ));      

!%(0, 1, 1.0/2) =>10 
      p1=1;p2=1;p3=1;p4=1;
      if ((ix-idx)==L+1) p1=0;
      if ((iy+idy)==L+LY) p4=0;
      call random_number(rand)
      H(ix-idx,iy+idy,iz)=ca*va+(1-va)*(2*(rand-0.5)*h_f**iNit+ &
                      1.0/(p1+p2+p3+p4+2)*( &
		         p1*H(ix-idx-idx, iy+idy    , iz    ) &
		        +p2*H(ix-idx+idx, iy+idy    , iz    ) &
		        +p3*H(ix-idx	, iy+idy-idy, iz    ) &
		        +p4*H(ix-idx	, iy+idy+idy, iz    ) &
		        +   H(ix-idx	, iy+idy    , iz-idz) &
		        +   H(ix-idx	, iy+idy    , iz+idz) &
		      ));

!%(1, 0, 1.0/2) =>11 
      p1=1;p2=1;p3=1;p4=1;
      if ((ix+idx)==L+LX) p2=0;
      if ((iy-idy)==L+1) p3=0;
      call random_number(rand)
      H(ix+idx,iy-idy,iz)=ca*va+(1-va)*(2*(rand-0.5)*h_f**iNit+ &
                      1.0/(p1+p2+p3+p4+2)*( &
		         p1*H(ix+idx-idx, iy-idy    , iz    ) &
		        +p2*H(ix+idx+idx, iy-idy    , iz    ) &
		        +p3*H(ix+idx	, iy-idy-idy, iz    ) &
		        +p4*H(ix+idx	, iy-idy+idy, iz    ) &
		        +   H(ix+idx	, iy-idy    , iz-idz) &
		        +   H(ix+idx	, iy-idy    , iz+idz) &
		      ));      

!%(1, 1, 1.0/2) =>12 
      p1=1;p2=1;p3=1;p4=1;
      if ((ix+idx)==L+LX) p2=0;
      if ((iy+idy)==L+LY) p4=0;
      call random_number(rand)
      H(ix+idx,iy+idy,iz)=ca*va+(1-va)*(2*(rand-0.5)*h_f**iNit+ &
                      1.0/(p1+p2+p3+p4+2)*( &
		         p1*H(ix+idx-idx, iy+idy    , iz    ) &
		        +p2*H(ix+idx+idx, iy+idy    , iz    ) &
		        +p3*H(ix+idx	, iy+idy-idy, iz    ) &
		        +p4*H(ix+idx	, iy+idy+idy, iz    ) &
		        +   H(ix+idx	, iy+idy    , iz-idz) &
		        +   H(ix+idx	, iy+idy    , iz+idz) &
		      ));      



    end do
    end do
    end do
  end do
  end do
  end do
end do  !Nit


!Hcut(:,:,:) = H(L+1:L+Taille_x,L+1:L+Taille_y,L+1:L+Taille_z);
!Hcut(:,:,:) = facteur*Hcut(:,:,:) + val_moy

do iz=1,taille_z
do iy=1,taille_y
do ix=1,taille_x
  Hcut(ix,iy,iz) = H(ix+L+ndxyz(1),iy+L+ndxyz(2),iz+L+ndxyz(3));
end do; end do; end do

do iz=1,taille_z
do iy=1,taille_y
do ix=1,taille_x
  Hcut(ix,iy,iz) = facteur*Hcut(ix,iy,iz) + val_moy
end do; end do; end do

!----------- Affichage Statistiques ------------------
if (rang==RANG_MAITRE_AFF) then
  print '(T5,"Valeur moyenne finale = ",F10.7)',SUM(Hcut)/(Taille_x*Taille_y*Taille_z)
  print '(T5,"Valeur min = ",F8.6)',minval(Hcut)
  print '(T5,"Valeur max = ",F8.6)',MAXVAL(Hcut)
end if

deallocate (H)
end function milieu_fractal_precalcul_old2

! ==========================================================
! == Fin du module                                        ==
! ==========================================================
end module m_milieu_fractal	



