!/////////////////////////////////////////////////////////////////////////////
!// Name:        mfdtd_cao.f90
!// Purpose:     Lecture des données cao de la modélisation
!// Author:      Christophe Guiffaut
!// Modified by: 
!// Lieu     :  Faculté des sciences - 123 Av. Albert Thomas - Limoges
!// Created  :     Septembre 2008
!// Copyright:   (c) Christophe Guiffaut,  CNRS UMR 6172, Institut XLIM
!//              Utilisation du source réservé à l'institut
!// Licence  :
!/////////////////////////////////////////////////////////////////////////////
module mfdtd_cao

use m_constantes
use m_constantes_scientifiques, only : PI
use m_entree, only : st_fichier, lineread, multibackspace, &
		ctrl_nb_mot_chaine, apres_fin_bloc,compteur_mot_cle,&
 		entier_chaine, nb_mot_chaine, nb_mot_chaine_long, &
		lecture_stop, &
		numbackspace_set
use m_maillage, only : st_maillage,origine_cellule6,conv_chcoord6_cell_int,&
                       conv_chcoord6_cell_reel,&
		       conv_coord6_cell_inttostr
use m_liste_milieu, only : st_liste_milieu_simple
use mfdtd_identifiant
use mfdtd_volume, only : st_volume

implicit none


integer, parameter :: MAT_METAL         =0
integer, parameter :: MAT_PLAQUE_MINCE  =1
integer, parameter :: MAT_VOLUME        =2


!----------------- Propriétés communes aux géométries  ---------------------------
type st_geom_prop_commun
real  :: origine(3) ! en mètre
real  :: Rx,Ry,Rz   ! angles de rotation en degré autour de x,y,z, angles d'Euler
real  :: ptrot(3)   ! point centre pour les rotations

integer  :: type_materiau  ! métal, plaque mince, volumique
character(LEN=LEN_MOT_CLE)  :: nom_mat
real     :: e  ! réprésente l'épaisseur du matériau

real  :: ptmin(3),ptmax(3)
end type st_geom_prop_commun


!----------------------------------------------------------
!--> Les géométries :
!----------------------------------------------------------
type st_plaque_rect
real  :: pt1(3)
real  :: lx,ly
integer :: normale !-1 ou +1
type (st_geom_prop_commun) :: com
end type st_plaque_rect


type st_boite
real  :: pt1(3)
real  :: lx,ly,lz
type (st_geom_prop_commun) :: com
end type st_boite

!----------------------------------------------------------
!----------------------------------------------------------


!----------------- Les objets  ---------------------------
type st_objetgeom
character(LEN=LEN_MOT_CLE)  :: nom

integer :: nb_plaque_rect
type(st_plaque_rect),pointer,dimension(:) :: t_pr
integer :: nb_boite
type(st_boite),pointer,dimension(:) :: t_b

!--- occupation spatiale pour chaque type de matériau ---
! dans le repère de calcul
real,dimension(3) :: ptmin_metal,ptmin_pm,ptmin_voldiel
real,dimension(3) :: ptmax_metal,ptmax_pm,ptmax_voldiel
!---------------------------------

end type st_objetgeom


!------------ Les familles d'objets géométriques ----------
type st_famille
character(LEN=LEN_MOT_CLE)  :: nom
integer :: nb_objet
type(st_objetgeom),pointer,dimension(:) :: t_og
end type st_famille

!----------------- La structure globale -------------------
type st_cao
logical  :: flag_data
integer  :: nb_famille
type(st_famille),pointer,dimension(:) :: t_fam

end type st_cao

!----------------------------------------------------------

contains

! =========================================================
! == Subroutine cao_initdefaut                           ==
! =========================================================
subroutine cao_initdefaut(c)

type(st_cao), intent(inout) :: c
c%flag_data  = .true.
c%nb_famille = 0
nullify(c%t_fam)

end subroutine cao_initdefaut

! =========================================================
! == Subroutine cao_famille_initdefaut                   ==
! =========================================================
subroutine cao_famille_initdefaut(c)

type(st_famille), intent(inout) :: c

c%nom=''
c%nb_objet = 0
nullify(c%t_og)

end subroutine cao_famille_initdefaut

! =========================================================
! == Subroutine cao_objet_initdefaut                     ==
! =========================================================
subroutine cao_objet_initdefaut(co)

type(st_objetgeom), intent(inout) :: co

co%nom=''
co%nb_plaque_rect = 0
co%nb_boite = 0
nullify(co%t_pr)
nullify(co%t_b)

end subroutine cao_objet_initdefaut




! ==========================================================
! == Subroutine cao_lecture                               ==
! == Lecture dans le fichier d'entrée...                  ==
! ==========================================================
subroutine cao_lecture(f,c,m)

type(st_fichier), intent(inout) :: f
type(st_cao), intent(inout) :: c
type(st_maillage), intent(in) :: m

integer  :: status,nbmot
character (len=LEN_MOT_CLE) :: premiermot, corbeille, nom,nom1
integer  :: i
integer  :: ifam,iobj,iboite,icyl,iplaque
integer, dimension(6) :: coord
character(LEN=LEN_MOT_CLE),dimension(6)   :: chcoord
real, dimension(6)  :: coord_reel

type(st_geom_prop_commun) :: gpc

type(st_objetgeom), pointer  :: pobj
type(st_plaque_rect),pointer :: pr
type(st_boite),pointer       :: pb

!============ On compte le nombre de familles ============
call cao_initdefaut(c)
ifam = compteur_mot_cle(f,'famille')
if (ifam==0) then
  print '("Observation : aucune famille de définie dans <cao>, le bloc est ignoré")'
  call apres_fin_bloc(f)
  c%flag_data = .false.
  return
end if

allocate(c%t_fam(ifam))
c%nb_famille =ifam

! Initialisation des variables locales
ifam=0
iobj=0
gpc%origine = m%origine
gpc%Rx = 0.0
gpc%Ry = 0.0
gpc%Rz = 0.0
gpc%ptrot =m%origine
gpc%type_materiau = -1
gpc%nom_mat = ''
gpc%e = 0.0

do
  premiermot = lineread(f)
  !------------------------------------------------------------------
  select case(premiermot)
  
  case ('boite')
    if (cao_ctrl_presence(f,iobj,gpc%type_materiau)) then
      iboite=iboite+1
      print '(T15,"iboite=",I6)',iboite
      pb=>pobj%t_b(iboite)
      status = ctrl_nb_mot_chaine(f,f%chaine,7)
      read(f%chaine,*,ERR=999) nom,pb%pt1(1:3),pb%lx,pb%ly,pb%lz
      if (pb%lx <0) pb%pt1(1) = pb%pt1(1)+pb%lx
      if (pb%ly <0) pb%pt1(2) = pb%pt1(2)+pb%ly
      if (pb%lz <0) pb%pt1(3) = pb%pt1(3)+pb%lz
      pb%lx=abs(pb%lx)
      pb%ly=abs(pb%ly)
      pb%lz=abs(pb%lz)
      pb%com = gpc
    end if
    
  case ('plaque_rect_xy')
    if (gpc%type_materiau==MAT_VOLUME) then
      print '("ERREUR, matériau volumique interdit pour les plaques !!!")'
      call lecture_stop(f)
    end if 
    if (cao_ctrl_presence(f,iobj,gpc%type_materiau)) then
      iplaque=iplaque+1
      print '(T15,"iplaque=",I6)',iplaque
      pr=>pobj%t_pr(iplaque)
      status = ctrl_nb_mot_chaine(f,f%chaine,7)
      read(f%chaine,*,ERR=999) nom,pr%pt1(1:3),pr%lx,pr%ly,pr%normale
      if (pr%lx <0) pr%pt1(1) = pr%pt1(1)+pr%lx
      if (pr%ly <0) pr%pt1(2) = pr%pt1(2)+pr%ly
      pr%lx=abs(pr%lx)
      pr%ly=abs(pr%ly)
      
      if (pr%normale/=-1 .and.pr%normale/=1) then
        print '("Erreur, la normale doit etre égale à +1 ou -1 !!!")'
	call lecture_stop(f)
      end if
      pr%com = gpc
    end if
  !------------------------------------------------------------------
  case ('metal')
    status = ctrl_nb_mot_chaine(f,f%chaine,1)
    read(f%chaine,*,ERR=999) gpc%nom_mat
    gpc%type_materiau = MAT_METAL
    gpc%e=0.0
    print '(T15,"materiau de type metal")'
    
  !------------------------------------------------------------------
  case ('plaque_mince')
    status = ctrl_nb_mot_chaine(f,f%chaine,3)
    read(f%chaine,*,ERR=999) nom,gpc%nom_mat,gpc%e
    gpc%type_materiau = MAT_PLAQUE_MINCE
    print '(T15,"materiau de type plaque mince")'

  !------------------------------------------------------------------
  case ('materiau_volume')
    status = ctrl_nb_mot_chaine(f,f%chaine,2)
    read(f%chaine,*,ERR=999) nom,gpc%nom_mat
    gpc%type_materiau = MAT_VOLUME
    gpc%e=0.0
    print '(T15,"materiau de type volume")'
  
  case ('objet')
    status = ctrl_nb_mot_chaine(f,f%chaine,2)
    read(f%chaine,*,ERR=999) nom,nom1
    if (ifam<=0) then
      print '("ERREUR, pas de famille pour l''objet ",A)',trim(nom1)      
      call lecture_stop(f)      
    end if    
    
    iobj = iobj+1
    print '(T15,"iobj=",I6)',iobj
    
    pobj=>c%t_fam(ifam)%t_og(iobj)
    call cao_objet_initdefaut(pobj)
    status = ctrl_nb_mot_chaine(f,f%chaine,2)
    pobj%nom = nom1
    
    gpc%type_materiau = -1
    gpc%nom_mat = ''
    gpc%e = 0.0
    !------------------------------------------------------
    ! Comptage des géométries
    !------------------------------------------------------
    call numbackspace_set(f,0)
    iboite=0
    icyl=0
    iplaque=0
    do
      premiermot = lineread(f,1)
      if (premiermot==f%nomfinbloc .or. premiermot=='famille'.or.premiermot=='objet') then
	exit
      else if (premiermot=='boite') then
	iboite=iboite+1
      !else if (premiermot=='cylindre') then
!	icyl=icyl+1
      else if (premiermot=='plaque_rect_xy') then
	iplaque=iplaque+1
      end if
    enddo
    !------------------------------------------------------
    
    !on remonte au début du bloc
    i = multibackspace(f)
    
    pobj%nb_plaque_rect = iplaque
    pobj%nb_boite       = iboite
    
    if (iplaque>0) THEN
      ALLOCATE(pobj%t_pr(iplaque))
    END IF
    
    if (iboite>0) THEN
      ALLOCATE(pobj%t_b(iboite))
    END IF
    iplaque = 0
    iboite = 0
    !------------------------------------------------------

  
  !------------------------------------------------------------------
  case ('famille')
    ifam = ifam+1
    print '(T15,"ifam=",I6)',ifam
    call cao_famille_initdefaut(c%t_fam(ifam))
    status = ctrl_nb_mot_chaine(f,f%chaine,2)
    read(f%chaine,*,ERR=999) nom,c%t_fam(ifam)%nom
    
    ! On compte le nombre d'objet
    iobj = compteur_mot_cle(f,'objet')
    if (iobj>0) then
      allocate(c%t_fam(ifam)%t_og(iobj))
      c%t_fam(ifam)%nb_objet = iobj
    end if
    iobj=0
    gpc%type_materiau = -1
    gpc%nom_mat = ''
    gpc%e = 0.0
  !------------------------------------------------------------------
  case ('rotation')
    status = ctrl_nb_mot_chaine(f,f%chaine,7)
    read(f%chaine,*,ERR=999) nom, gpc%ptrot(1:3),gpc%Rx,gpc%Ry,gpc%Rz
    !conversion en radian
    gpc%Rx=gpc%Rx*PI/180.0
    gpc%Ry=gpc%Ry*PI/180.0
    gpc%Rz=gpc%Rz*PI/180.0
    print '(T15,"ifam=",I6)',ifam
    print '(T15,"changement rotation")'

  !------------------------------------------------------------------
  case ('origine')
    status = ctrl_nb_mot_chaine(f,f%chaine,4)
    read(f%chaine,*,ERR=999) nom, gpc%origine(1:3)
    print '(T15,"changement origine")'
  
    
  !------------------------------------------------------------------
  case default
    if (f%nomfinbloc == premiermot) exit ! fin du bloc      
    print '(/,T10,"Mot clé suivant non reconnu : ",A)', TRIM(premiermot)
    call lecture_stop(f)

  end select
end do


return
999 print '(/,t10,"Erreur d''extraction des données de la ligne !!!")'
    call lecture_stop(f)

end subroutine cao_lecture



! ==========================================================
! == FUNCTION cao_ctrl_presence                           ==
!  Controle de la cohérence des données pour la géométrie
!  présence de la déclaration d'un objet et d'un matériau
! ==========================================================
logical FUNCTION cao_ctrl_presence(f,iobj,imat)
type(st_fichier), intent(in) :: f
integer, intent(in) :: iobj,imat

cao_ctrl_presence = .true.
if (iobj==0.or.imat==-1) then
  if (iobj==0) print '("ERREUR, manque la déclaration de l''objet!!!")'
  if (imat<0) print '("ERREUR, manque la définition du matériau !!!")'
  call lecture_stop(f)
  cao_ctrl_presence = .false.
end if

end FUNCTION cao_ctrl_presence


end module mfdtd_cao
