!/////////////////////////////////////////////////////////////////////////////
!// Name:        champ_lointain_demi_espace.f90
!// Purpose:     Calcul du champ lointain par la méthode mixte 
!                temp-fréquence à partir des courants circulant sur les 
!                conducteurs parfaits (concernent que les courants horizontaux mais 
!                prend en compte la présence d'un demi-espace (sol)
!                avec une constante diélectrique et des pertes.
!      Simulateur FDTD
!// Author:      Christophe Guiffaut
!// Modified by:  Christophe Guiffaut
!// Created  :   -  Janvier 2004 pour la seconde
!// Lieu     : XLIM - Faculté des sciences - 123 Av. Albert Thomas - Limoges
!// Copyright:   (c) CNRS, Université de Limoges
!// Licence  : CECILL-C - Octobre 2006
!/////////////////////////////////////////////////////////////////////////////

!================================================================
!  Calcul du champ lointain par la méthode frequentiel des surfaces de Huygens
!================================================================
module champ_lointain_demi_espace
use m_constantes
use m_constantes_scientifiques

use m_sortie, only : st_fichier_sortie, &
                                  impression_ascii_reel_2x, &
				  impression_ascii_complexe_2x, &
				  fichier_sortie_lect_format_freq, &
				  impression_ascii_complexe
                                  				  
use mfdtd_volume, only : st_volume
use mfdtd_structure, only : st_structure_objet

use mfdtd_milieu_volume, only : st_milieu_volume, &
                          milieu_volume_extrait_epsilon, &
			  milieu_volume_extrait_mu, &
			  milieu_volume_extrait_sige, & 
			  milieu_volume_extrait_sigh
                 
use m_zone_angulaire, only : st_zone_angulaire, &
                                  st_une_zone, &
				  C_PHI=>PHI, C_THETA=>THETA, &
				  zone_angulaire_pointe

use m_surface_huygens, only : st_surface_huygens, &
                                  st_une_surface_huygens, &
				  surface_huygens_pointe

use mfdtd_structure_fil,   only : st_structure_fil, &
                                   st_fil_objet, &
				   st_fil_segment

use m_temps_frequence, only : st_temps_frequence, &
                        st_liste_temps_freq, &
			temps_freq_identifie_freq, &
			liste_temps_freq_pointe

use mfdtd_champ, only : st_champ 

implicit none 

!=== Déclaration des types ===
! Structure pour les alias avec les composantes du champ
type st_pointeur_H_J
 integer  :: n1, n2
 integer  :: ideb, ifin, jdeb, jfin, hauteur
 real, pointer, dimension(:,:)       :: h1inf,h1sup,h2inf,h2sup
 complex, pointer, dimension(:,:,:)  :: J1, J2

! Pour les cotés de la plaque :
 REAL, pointer, dimension(:)         :: Hz_jxinf,Hz_jxsup,Hz_jyinf,Hz_jysup
 complex, pointer, dimension(:,:,:)  :: Jx,Jy  
 
end type st_pointeur_H_J


! Structure pour le calcul du champ lointain d'une surface de Huygens
type st_chl_demi_espace
  logical  :: flag_metal, flag_fil
  integer  :: nbobjmetal,nbobjfil
  ! champ lointain d'un demi-espace, sol réel, epsilon et sigma
  integer  :: plan_sol, position_sol_interface ! obligatoirement XOY !
  integer  :: demi_espace_objet ! position de l'objet: 1->demi-espace inférieur, 2->demi-espace supérieur
  real, dimension(3)  :: centre_phase
  real, dimension(2)  ::eps,mu,sige,sigh 
  ! structure décrivant les conducteurs parfaits sur laquelle
  ! sont évalués les courants servant pour la TF champ proche - champ lointain
  type(st_fil_objet),   pointer  :: objfil 
  type(st_structure_objet), pointer  :: tmetal 
  type(st_pointeur_H_J), dimension(:), pointer     :: thj 
  
end type st_chl_demi_espace

! Pas de variables globales...

contains
!===========================================================	
! Subroutine chl_demi_espace_initdefaut
! Initialisation par défaut des variables contenant les paramètres
! définis par l'utilisateur dans le fichier d'entrée
!===========================================================	
Subroutine chl_demi_espace_initdefaut(cde)
 implicit none
!=== Déclaration des arguments ===
type(st_chl_demi_espace), intent(inout)  :: cde
!===  Déclaration des variables  ===

!===   Début des traitements   ===
cde%flag_metal   = .false.
cde%flag_fil     = .false.
cde%nbobjmetal = 0
cde%nbobjfil = 0

cde%plan_sol = PLAN_XOY
cde%position_sol_interface = 0
nullify(cde%objfil)
nullify(cde%tmetal)
nullify(cde%thj)
				
End Subroutine chl_demi_espace_initdefaut

!===========================================================	
! Subroutine chl_demi_espace_init
! Initialisation des variables
!===========================================================	
Subroutine chl_demi_espace_init(cde)
implicit none
!=== Déclaration des arguments ===
type(st_chl_demi_espace), intent(inout)  :: cde

!===  Déclaration des variables  ===
integer :: i,demi_espace_objet
logical :: test1,test2
!===   Début des traitements   ===
! Validité des données ?
if (cde%position_sol_interface <=0) then
  print '("ERREUR : Calcul champ lointain à partir de l''objet ",A)', TRIM(cde%tmetal%nom)
  print '("ERREUR : Calcul champ lointain, interface mal ou pas définie. Arret ...")'
  STOP
end if
! Il faut que toutes les structures métalliques soit plan xoy
if (cde%flag_metal) then
  cde%nbobjmetal = cde%tmetal%nbstructure

  do i=1, cde%tmetal%nbstructure
    if (cde%tmetal%coord(5,i) /= cde%tmetal%coord(6,i)) then
      print '("ERREUR : Calcul champ lointain à partir de l''objet ",A)', TRIM(cde%tmetal%nom)
      print '("ERREUR : Toutes les structures métalliques doivent etre plan xoy (zmin = zmax), pour le champ lointain")'
      PRINT '("ERREUR : Structure n°",I4, "de l''objet")', i
      STOP    
    end if
  end do
  
  ! Il faut que toutes les structures soient dans le meme demi-espace
  test1 = .false.
  test2 = .false.
  do i=1, cde%tmetal%nbstructure
    if (cde%tmetal%coord(5,i) < cde%position_sol_interface) test1 = .true.
    if (cde%tmetal%coord(5,i) > cde%position_sol_interface) test2 = .true.
  end do
  if ((test1).and.(test2)) then
    print '("ERREUR : Calcul champ lointain à partir de l''objet ",A)', TRIM(cde%tmetal%nom)
    print '("ERREUR : Toutes les structures métalliques doivent etre dans le meme demi-espace, pour le champ lointain")'
    STOP
  end if
  cde%demi_espace_objet = 2
  if (test1) cde%demi_espace_objet = 1 ! demi-espace inférieur
end if
!------------------------------------------------------------------------------

!------------------------------------------------------------------------------
if (cde%flag_fil) then
  cde%nbobjfil = cde%objfil%nbfil
  do i=1, cde%objfil%nbfil
    if (cde%objfil%p(i)%fil%dir == DIRECTION_Z) then
      print '("ERREUR : Calcul champ lointain à partir de l''objet ",A)', TRIM(cde%objfil%nom)
      print '("ERREUR : Tous les fils doivent etre plan xoy (zmin = zmax) !")'
      PRINT '("ERREUR : Structure n°",I4, "de l''objet fil")', i
      STOP    
    end if
  end do
  test1 = .false.
  test2 = .false.
  do i=1, cde%objfil%nbfil
    if (cde%objfil%p(i)%fil%coord(5) < cde%position_sol_interface) test1 = .true.
    if (cde%objfil%p(i)%fil%coord(5) > cde%position_sol_interface) test2 = .true.
  end do
  if ((test1).and.(test2)) then
    print '("ERREUR : Calcul champ lointain à partir de l''objet ",A)', TRIM(cde%objfil%nom)
    print '("ERREUR : Toutes les structures fil doivent etre dans le meme demi-espace!")'
    STOP
  end if
  demi_espace_objet = 2
  if (test1) demi_espace_objet = 1 ! demi-espace inférieur
  if (cde%flag_metal) then
    if (demi_espace_objet /=cde%demi_espace_objet) then
      print '("ERREUR : Calcul champ lointain à partir de l''objet ",A)', TRIM(cde%objfil%nom)
      print '("ERREUR : Toutes les structures fil et metal doivent etre dans le meme demi-espace !")'
      STOP
    end if
  else
    cde%demi_espace_objet = demi_espace_objet
  end if
end if


End Subroutine chl_demi_espace_init


!===========================================================	
! Subroutine chl_demi_espace_boite_huygens
! Calcul une boite de Huygens entourant l'objet rayonnant
! et l'interface.
!===========================================================	
function chl_demi_espace_boite_huygens(cde) result(coord)
implicit none
!=== Déclaration des arguments ===
type(st_chl_demi_espace), intent(inout)  :: cde

integer, dimension(6) :: coord
!===  Déclaration des variables  ===
integer :: i,j

!===   Début des traitements   ===
if (cde%flag_metal) then
  coord(1:6) =cde%tmetal%coord(:,1) 
  do i=2, cde%tmetal%nbstructure
      if (cde%tmetal%coord(1,i)<coord(1)) coord(1) = cde%tmetal%coord(1,i)
      if (cde%tmetal%coord(3,i)<coord(3)) coord(3) = cde%tmetal%coord(3,i)
      if (cde%tmetal%coord(5,i)<coord(5)) coord(5) = cde%tmetal%coord(5,i)
      if (cde%tmetal%coord(2,i)>coord(2)) coord(2) = cde%tmetal%coord(2,i)
      if (cde%tmetal%coord(4,i)>coord(4)) coord(4) = cde%tmetal%coord(4,i)
      if (cde%tmetal%coord(6,i)>coord(6)) coord(6) = cde%tmetal%coord(6,i)
  end do
end if
if (cde%flag_fil) then
  if (.not.cde%flag_metal) coord(1:6) =cde%objfil%p(1)%fil%coord(1:6)
  do i=2, cde%objfil%nbfil
      if (cde%objfil%p(i)%fil%coord(1)<coord(1)) coord(1) = cde%objfil%p(i)%fil%coord(1)
      if (cde%objfil%p(i)%fil%coord(3)<coord(3)) coord(3) = cde%objfil%p(i)%fil%coord(3)
      if (cde%objfil%p(i)%fil%coord(5)<coord(5)) coord(5) = cde%objfil%p(i)%fil%coord(5)
      if (cde%objfil%p(i)%fil%coord(2)>coord(2)) coord(2) = cde%objfil%p(i)%fil%coord(2)
      if (cde%objfil%p(i)%fil%coord(4)>coord(4)) coord(4) = cde%objfil%p(i)%fil%coord(4)
      if (cde%objfil%p(i)%fil%coord(6)>coord(6)) coord(6) = cde%objfil%p(i)%fil%coord(6)
  end do
end if


! On agrandit la boite de 2 cellules dans chaque direction
coord(1:5:2) = coord(1:5:2) -2
coord(2:6:2) = coord(2:6:2) +2
! Positionnement de la face de Huygens à deux cellules de 
! l'interface dans l'autre demi-espace.
if (cde%demi_espace_objet == 1) then
  coord(6) = cde%position_sol_interface +2
else
  coord(5) = cde%position_sol_interface -2
end if

End FUNCTION chl_demi_espace_boite_huygens

!===========================================================	
! Subroutine chl_demi_espace_centre_phase
! Calcul du centre de phase de l'objet
!===========================================================	
function chl_demi_espace_centre_phase(cde) result(centre)
implicit none
!=== Déclaration des arguments ===
type(st_chl_demi_espace), intent(inout)  :: cde

real, dimension(3) :: centre
!===  Déclaration des variables  ===
integer :: i,j
integer, dimension(6) :: coord

!===   Début des traitements   ===
if (cde%flag_metal) then
  coord(1:6) =cde%tmetal%coord(:,1) 
  do i=2, cde%tmetal%nbstructure
      if (cde%tmetal%coord(1,i)<coord(1)) coord(1) = cde%tmetal%coord(1,i)
      if (cde%tmetal%coord(3,i)<coord(3)) coord(3) = cde%tmetal%coord(3,i)
      if (cde%tmetal%coord(5,i)<coord(5)) coord(5) = cde%tmetal%coord(5,i)
      if (cde%tmetal%coord(2,i)>coord(2)) coord(2) = cde%tmetal%coord(2,i)
      if (cde%tmetal%coord(4,i)>coord(4)) coord(4) = cde%tmetal%coord(4,i)
      if (cde%tmetal%coord(6,i)>coord(6)) coord(6) = cde%tmetal%coord(6,i)
  end do
end if


if (cde%flag_fil) then
  if (.not.cde%flag_metal) coord(1:6) =cde%objfil%p(1)%fil%coord(1:6)
  do i=2, cde%objfil%nbfil
      if (cde%objfil%p(i)%fil%coord(1)<coord(1)) coord(1) = cde%objfil%p(i)%fil%coord(1)
      if (cde%objfil%p(i)%fil%coord(3)<coord(3)) coord(3) = cde%objfil%p(i)%fil%coord(3)
      if (cde%objfil%p(i)%fil%coord(5)<coord(5)) coord(5) = cde%objfil%p(i)%fil%coord(5)
      if (cde%objfil%p(i)%fil%coord(2)>coord(2)) coord(2) = cde%objfil%p(i)%fil%coord(2)
      if (cde%objfil%p(i)%fil%coord(4)>coord(4)) coord(4) = cde%objfil%p(i)%fil%coord(4)
      if (cde%objfil%p(i)%fil%coord(6)>coord(6)) coord(6) = cde%objfil%p(i)%fil%coord(6)
  end do
end if

do i=1,3
  centre(i) = (coord(i*2) - coord(i*2-1)) / 2 +  coord(i*2-1)
end do


End FUNCTION chl_demi_espace_centre_phase

!===========================================================	
! Subroutine chl_demi_espace_precalcul
! Calculs avant les itérations temporelles
! 
!===========================================================	
Subroutine chl_demi_espace_precalcul(cde,tm,nb_f,origine,flag_centre_phase_auto)
implicit none
!=== Déclaration des arguments ===
type(st_chl_demi_espace), intent(inout)  :: cde
type(st_milieu_volume), intent(in) :: tm
integer, intent(in) :: nb_f
real, dimension(3),intent(inout) :: origine
logical,intent(in) :: flag_centre_phase_auto
!===  Déclaration des variables  ===
integer  :: i,j,n
integer, dimension(6) :: coord

!===   Début des traitements   ===

! Calcul automatique du centre de phase de l'objet si non défini
!  Le centre de phase calculé est le centre de l'objet
if (flag_centre_phase_auto) origine = chl_demi_espace_centre_phase(cde)
!  print '("ERREUR : Calcul champ lointain à partir de l''objet ",A)', TRIM(cde%tmetal%nom)
!  print '("ERREUR : Origine suivant z erronée ou pas défini")'
!  stop
!end if

cde%centre_phase = origine
!------------------------------------------------------------------------------
n=0
if (cde%flag_metal) n = cde%tmetal%nbstructure
if (cde%flag_fil)   n = n+cde%objfil%nbfil
allocate(cde%thj(1:n))
n=0
if (cde%flag_metal) then
  n=cde%tmetal%nbstructure
  ! boucle sur toutes les structures rectangulaires de l'objet métallique
  do i=1, n
    cde%thj(i)%ideb = cde%tmetal%coord(XMIN,i)
    cde%thj(i)%ifin = cde%tmetal%coord(XMAX,i)
    cde%thj(i)%jdeb = cde%tmetal%coord(YMIN,i)
    cde%thj(i)%jfin = cde%tmetal%coord(YMAX,i)
    cde%thj(i)%hauteur = cde%tmetal%coord(ZMIN,i)
    
    cde%thj(i)%n1 = cde%thj(i)%ifin - cde%thj(i)%ideb +1
    cde%thj(i)%n2 = cde%thj(i)%jfin - cde%thj(i)%jdeb +1
    allocate(cde%thj(i)%J1(1:cde%thj(i)%n1,1:cde%thj(i)%n2,1:nb_f))
    allocate(cde%thj(i)%J2(1:cde%thj(i)%n1,1:cde%thj(i)%n2,1:nb_f))
    allocate(cde%thj(i)%Jx(1:cde%thj(i)%n1,1:2,1:nb_f))
    allocate(cde%thj(i)%Jy(1:cde%thj(i)%n2,1:2,1:nb_f))
    cde%thj(i)%J1(:,:,:) = 0.0
    cde%thj(i)%J2(:,:,:) = 0.0
    cde%thj(i)%Jx(:,:,:) = 0.0
    cde%thj(i)%Jy(:,:,:) = 0.0
  end do
end if
j=n

if (cde%flag_fil) then
  ! boucle sur toutes les structures fil
  do i=1, cde%objfil%nbfil
    j=j+1
    cde%thj(j)%ideb = cde%objfil%p(i)%fil%coord(XMIN)
    cde%thj(j)%ifin = cde%objfil%p(i)%fil%coord(XMAX)
    cde%thj(j)%jdeb = cde%objfil%p(i)%fil%coord(YMIN)
    cde%thj(j)%jfin = cde%objfil%p(i)%fil%coord(YMAX)
    cde%thj(j)%hauteur = cde%objfil%p(i)%fil%coord(ZMIN)
    
    cde%thj(j)%n1 = cde%thj(j)%ifin - cde%thj(j)%ideb +1
    cde%thj(j)%n2 = cde%thj(j)%jfin - cde%thj(j)%jdeb +1
    allocate(cde%thj(j)%J1(1:cde%thj(j)%n1,1:cde%thj(j)%n2,1:nb_f))
    allocate(cde%thj(j)%J2(1:cde%thj(j)%n1,1:cde%thj(j)%n2,1:nb_f))
    cde%thj(j)%J1(:,:,:) = 0.0
    cde%thj(j)%J2(:,:,:) = 0.0
    Nullify(cde%thj(j)%Jx)
    Nullify(cde%thj(j)%Jy)
    print '("n°, cde%thj%ideb,ifin = ",3I4)',j,cde%thj(j)%ideb,cde%thj(j)%ifin
    print '("n°, cde%thj%jdeb,jfin = ",3I4)',j,cde%thj(j)%jdeb,cde%thj(j)%jfin
    print '("n°, cde%thj%hauteur = ",2I4)',j,cde%thj(j)%hauteur
  end do
end if



! Type de milieu du champ lointain de part et d'autre de l'interface air - sol
! on récupère l'epsilonn, mu et les conductivités
j=-1
do i=1,2  
  if (i ==2)  j= 1
  cde%eps(i) = milieu_volume_extrait_epsilon(tm,cde%thj(1)%ideb, &
					        cde%thj(1)%jdeb, &
					        cde%position_sol_interface+j, DIRECTION_X)

  cde%mu(i)  = milieu_volume_extrait_mu(tm,cde%thj(1)%ideb, &
                                           cde%thj(1)%jdeb, &
				           cde%position_sol_interface+j, DIRECTION_X)
  cde%sige(i) = milieu_volume_extrait_sige(tm,cde%thj(1)%ideb, &
					      cde%thj(1)%jdeb, &
					      cde%position_sol_interface+j, DIRECTION_X)
  cde%sigh(i) = milieu_volume_extrait_sigh(tm,cde%thj(1)%ideb, &
					      cde%thj(1)%jdeb, &
  					      cde%position_sol_interface+j, DIRECTION_X)
end do
end Subroutine chl_demi_espace_precalcul
				

!===========================================================	
! Subroutine chl_demi_espace_alias_champ
! Alias sur les composantes du champ E, H
!===========================================================	
Subroutine chl_demi_espace_alias_champ(cde, tc)
implicit none
!=== Déclaration des arguments ===
type(st_chl_demi_espace), intent(inout)  :: cde
type(st_champ), intent(in)   :: tc

!===  Déclaration des variables  ===
integer  :: i, ph
!===   Début des traitements   ===

! boucle sur toutes les structures rectangulaires de l'objet métallique
do i=1, cde%nbobjmetal
  ph = cde%thj(i)%hauteur
  cde%thj(i)%h1inf => tc%hx(cde%thj(i)%ideb:cde%thj(i)%ifin,cde%thj(i)%jdeb:cde%thj(i)%jfin,ph-1)
  cde%thj(i)%h1sup => tc%hx(cde%thj(i)%ideb:cde%thj(i)%ifin,cde%thj(i)%jdeb:cde%thj(i)%jfin,ph)
  cde%thj(i)%h2inf => tc%hy(cde%thj(i)%ideb:cde%thj(i)%ifin,cde%thj(i)%jdeb:cde%thj(i)%jfin,ph-1)
  cde%thj(i)%h2sup => tc%hy(cde%thj(i)%ideb:cde%thj(i)%ifin,cde%thj(i)%jdeb:cde%thj(i)%jfin,ph)

  cde%thj(i)%hz_jxinf => tc%hz(cde%thj(i)%ideb:cde%thj(i)%ifin,cde%thj(i)%jdeb-1,ph)
  cde%thj(i)%hz_jxsup => tc%hz(cde%thj(i)%ideb:cde%thj(i)%ifin,cde%thj(i)%jfin  ,ph)

  cde%thj(i)%hz_jyinf => tc%hz(cde%thj(i)%ideb-1,cde%thj(i)%jdeb:cde%thj(i)%jfin,ph)
  cde%thj(i)%hz_jysup => tc%hz(cde%thj(i)%ifin  ,cde%thj(i)%jdeb:cde%thj(i)%jfin,ph)
end do

End Subroutine chl_demi_espace_alias_champ

!===========================================================	
! Subroutine chl_demi_espace_affichage
!  Affichage de résultats dans le terminal
!===========================================================	
Subroutine chl_demi_espace_affichage(cde)
implicit none
!=== Déclaration des arguments ===
type(st_chl_demi_espace), intent(inout)  :: cde

!===  Déclaration des variables  ===
!===   Début des traitements   ===

PRINT '(T10,"Plan_Sol =",I2," Position = ",I3)',cde%plan_sol,cde%position_sol_interface
if (cde%demi_espace_objet == 1) then
  PRINT '(T10,"Objet rayonnant dans le demi-espace inférieur")'
else if (cde%demi_espace_objet == 2) then
  PRINT '(T10,"Objet rayonnant dans le demi-espace supérieur")'
end if

PRINT '(T10," Centre de phase = ",3F8.2)',cde%centre_phase
PRINT '(T10," eps(1:2) = ",2ES16.6E2)',cde%eps
PRINT '(T10," mu(1:2) = ",2ES16.6E2)',cde%mu
PRINT '(T10," sige(1:2) = ",2ES16.6E2)',cde%sige
PRINT '(T10," sigh(1:2) = ",2ES16.6E2)',cde%sigh
if (cde%flag_metal) PRINT '(T10," Nom objet rayonnant metal: ",A)',cde%tmetal%nom
if (cde%flag_fil) PRINT '(T10," Nom objet rayonnant fil: ",A)',cde%objfil%nom

PRINT '(T10," Nb structure de l''objet rayonnant metal = ",I4)',cde%nbobjmetal
PRINT '(T10," Nb structure de l''objet rayonnant fil = ",I4)',cde%nbobjfil

End Subroutine chl_demi_espace_affichage

!===========================================================	
! Subroutine chl_demi_espace_calcul
!  Calcul pendant les itérations temporelles
!===========================================================	
Subroutine chl_demi_espace_calcul(cde,tf)
implicit none
!=== Déclaration des arguments ===
type(st_chl_demi_espace), intent(inout)  :: cde
type(st_temps_frequence), intent(in)     :: tf

!===  Déclaration des variables  ===
integer  :: i,j,n,iseg,lf
type(st_pointeur_H_J), pointer :: phj

!===   Début des traitements   ===

do i=1, cde%nbobjmetal
  phj =>cde%thj(i)
  ! boucle sur les fréquences
  do lf = 1, tf%n_liste
    phj%J1(:,:,lf) = phj%J1(:,:,lf) - (phj%h2sup(:,:)-phj%h2inf(:,:)) * tf%expi2pitf(lf)
    phj%J2(:,:,lf) = phj%J2(:,:,lf) + (phj%h1sup(:,:)-phj%h1inf(:,:)) * tf%expi2pitf(lf)

    phj%Jx(:,1,lf) = phj%Jx(:,1,lf) - phj%hz_jxinf(:) * tf%expi2pitf(lf)
    phj%Jx(:,2,lf) = phj%Jx(:,2,lf) + phj%hz_jxsup(:) * tf%expi2pitf(lf)
    phj%Jy(:,1,lf) = phj%Jy(:,1,lf) + phj%hz_jyinf(:) * tf%expi2pitf(lf)
    phj%Jy(:,2,lf) = phj%Jy(:,2,lf) - phj%hz_jysup(:) * tf%expi2pitf(lf)
  end do
end do

j = cde%nbobjmetal

do i=1, cde%nbobjfil
  j = j+1
  phj =>cde%thj(j)
  if (cde%objfil%p(i)%fil%dir == DIRECTION_X) then
    do lf = 1, tf%n_liste
      do n=1,phj%n1-1
	phj%J1(n,1,lf) = phj%J1(n,1,lf) +  cde%objfil%p(i)%fil%l_seg(n)%cur* tf%expi2pitf(lf)
      end do
    end do
  else ! Direction Y
    do lf = 1, tf%n_liste
      do n=1,phj%n2-1
	phj%J2(1,n,lf) = phj%J2(1,n,lf) +  cde%objfil%p(i)%fil%l_seg(n)%cur* tf%expi2pitf(lf)
      end do
    end do
    
  end if
end do

End Subroutine chl_demi_espace_calcul

!===========================================================	
! Subroutine chl_demi_espace_J_M
!  Annulation dans les tableaux de sources de courant 
! des valeurs sur certains bords extérieurs.
! Intégration des courants surfaciques
!===========================================================	
Subroutine chl_demi_espace_J_M(cde,tv)
implicit none
!=== Déclaration des arguments ===
type(st_chl_demi_espace), intent(inout)  :: cde
type(st_volume), intent(in) :: tv

!===  Déclaration des variables  ===
integer  :: i
real     :: dxy,dyz,dxz
type(st_pointeur_H_J), pointer :: phj

!===   Début des traitements   ===
! Les courants sont obligatoirement sur une surface de Huygens plan xOy
! Calcul des surfaces élémentaires
dxy = tv%celx*tv%cely
dyz = tv%cely*tv%celz
dxz = tv%celx*tv%celz

do i=1, cde%nbobjmetal+cde%nbobjfil
  phj =>cde%thj(i)
  phj%J1(phj%n1,:,:) = CMPLX(0.0,0.0)
  phj%J2(:,phj%n2,:) = CMPLX(0.0,0.0)
  phj%J1 = phj%J1 * dxy
  phj%J2 = phj%J2 * dxy
end do

do i=1, cde%nbobjmetal
  phj =>cde%thj(i)
  phj%Jx(phj%n1,:,:) = CMPLX(0.0,0.0)
  phj%Jy(phj%n2,:,:) = CMPLX(0.0,0.0)
  phj%Jx = phj%Jx * dxz
  phj%Jy = phj%Jy * dyz
end do

! correction pour les courants sur les fils
do i=cde%nbobjmetal+1, cde%nbobjmetal+cde%nbobjfil
  phj =>cde%thj(i)
  phj%J1 = phj%J1 / tv%cely
  phj%J2 = phj%J2 / tv%celx
end do
!phj =>cde%thj(1) 
!do i=1,phj%n1
!  write (100,'(10ES16.6E2)') ABS(phj%J1(i,1,1:5))
!  write (101,'(10ES16.6E2)') ABS(phj%J2(i,1,1:5))
!end do
End Subroutine chl_demi_espace_J_M

!===========================================================	
! Subroutine chl_demi_espace_calcul_eth_eph
!  Calcul des champ Etheta et Ephi en fonction de la fréquence
! A partir des sources de courant de la surface de Huygens
! L'algorithme est mise en oeuvre de manière particulière
! afin que la vectorisation puisse s'appliquer.
! L'objet doit etre entièrement dans le meme demi-espace
!===========================================================	
Subroutine chl_demi_espace_calcul_eth_eph(cde,tf,tv,tza,etheta,ephi)
implicit none
!=== Déclaration des arguments ===
type(st_chl_demi_espace), intent(in)  :: cde
type(st_temps_frequence), intent(in)  :: tf
type(st_volume), intent(in) :: tv
!integer, intent(in) :: nbfreq ! nombre de fréquences
type(st_une_zone)   :: tza
complex, dimension(1:tza%nbcoupleangle,1:tf%n_liste), intent(out) :: etheta, ephi

!===  Déclaration des variables  ===
type(st_pointeur_H_J), pointer :: phj
integer   :: nbstructure
integer   :: i,j,k,l,nf,n, imr,imt,imilieu
real      :: unsur2pi,h
complex, allocatable, dimension(:,:)  :: jk,epsc,muc,Zi
complex, allocatable, dimension(:,:)  :: Aparal, Aperp
complex, allocatable, dimension(:)  :: Rparal, Rperp
complex, allocatable, dimension(:)  :: Tparal, Tperp
complex, pointer, dimension(:)  :: nk,n21,n12,cossintheta
real      :: theta, phi
real	  :: dxsintetacosfi, dysintetasinfi, dzcosteta
real	  :: cosfi,costeta,sinteta,sinfi,costetacosfi, costetasinfi
real      :: isrc, jsrc, ksrc,iref,jref
real      :: yj1, yj2
real      :: rsJ1, rsJ2,erreur
complex   :: jkth,jkth1, jkth2,coefc,n21_sintheta
real,dimension(2) :: xrj,yrj

!===   Début des traitements   ===
unsur2pi = 1 / (2 * PI)
! Calcul des surfaces élémentaires

!dxdy = tv%celx*tv%cely
!dydz = tv%cely*tv%celz
!dxdz = tv%celx*tv%celz

etheta = cmplx(0.0,0.0)
ephi   = cmplx(0.0,0.0)
nbstructure = cde%nbobjmetal+cde%nbobjfil
! Calcul de j*k sur la bande de fréquence dans les deux demi-espace
allocate(jk  (1:tf%n_liste,2))
allocate(Zi  (1:tf%n_liste,2))
allocate(epsc(1:tf%n_liste,2))
allocate(muc (1:tf%n_liste,2))
allocate(n21  (1:tf%n_liste))
allocate(n12  (1:tf%n_liste))

! i=1 , milieu en dessous de l'interface, zinf
! i=2 , milieu au dessus  de l'interface, zsup
do i=1,2
  epsc(:,i) = cmplx(cde%eps(i),-cde%sige(i)*unsur2pi/tf%liste(:))
  muc(:,i)  = cmplx(cde%mu(i) ,-cde%sigh(i)*unsur2pi/tf%liste(:))
end do  
do i=1,2
  jk(:,i) = CMPLX(0.0,1.0) * 2 * PI * tf%liste(:) *sqrt(epsc(:,i)*muc(:,i))
  Zi(:,i) = sqrt(muc(:,i)/epsc(:,i))
end do
n21(:) = jk(:,2)/jk(:,1) 
n12(:) = jk(:,1)/jk(:,2) 
do nf = 1, tf%n_liste
erreur = abs(jk(nf,1)-jk(nf,2)) /ABS(jk(nf,1))
  if (erreur <1e-5) then 
    n21(:) = CMPLX(1.0,0.0)
    n12(:) = CMPLX(1.0,0.0)
  end if
end do  
!  Composantes des vecteurs de phase
allocate(Aparal(1:tf%n_liste,1:nbstructure))
allocate(Aperp(1:tf%n_liste,1:nbstructure))
allocate(Rparal(1:tf%n_liste))
allocate(Rperp(1:tf%n_liste))
allocate(Tparal(1:tf%n_liste))
allocate(Tperp(1:tf%n_liste))
allocate(cossintheta(1:tf%n_liste))
imr = 1 !milieu  par défaut de l'objet rayonnant
imt = 2 !milieu par défaut de transmission
if (cde%demi_espace_objet==2) then
  imt = 1
  imr = 2
end if  

! Boucle sur tous les couples d'angle (theta,phi)
do n = 1, tza%nbcoupleangle
  phi   = tza%liste_couple_angle(C_PHI,n)
  theta = tza%liste_couple_angle(C_THETA,n)
  costeta = cos(theta)
  if (costeta>=0) then
    imilieu = 2
  else
    imilieu = 1
  end if
  
  cosfi = cos(phi);
  sinteta        = sin(theta)
  sinfi          = sin(phi);
  costetacosfi   = abs(costeta)* cosfi;
  costetasinfi   = abs(costeta)* sinfi;  
  dxsintetacosfi = tv%celx * sinteta* cosfi;
  dysintetasinfi = tv%cely * sinteta* sinfi;
!  dzcosteta      = tv%celz * costeta

  
  Aparal(:,:) = CMPLX(0.0,0.0) 
  Aperp(:,:)  = CMPLX(0.0,0.0)

  !---------------------- Rayonnement des surfaces de courant plan xOy ----------------------------
  do k=1, nbstructure
    phj =>cde%thj(k)
    
    ! précalcul des coordonnées    
    jref = -1 + phj%jdeb - cde%centre_phase(2)
    iref = -1 + phj%ideb - cde%centre_phase(1)
    ! boucle suivant y
    do j = 1,phj%n2
      jsrc = j +jref
      yj1 = jsrc * dysintetasinfi       !Jx = -hy
      yj2 = (jsrc+0.5) * dysintetasinfi !Jy = hx
	  
      ! boucle suivant x
      do i = 1,phj%n1
	isrc = i +iref
        rsJ1 = yj1 + (isrc+0.5) * dxsintetacosfi
        rsJ2 = yj2 + isrc * dxsintetacosfi
	  	  
	! Boucle sur toutes les fréquences
        do nf = 1, tf%n_liste
	  !Calcul des vecteurs de phase pour chaque fréquence
	  Aparal(nf,k) = Aparal(nf,k) - phj%J1(i,j,nf) * costetacosfi * CEXP(jk(nf,imilieu)*rsJ1) &
                                      - phj%J2(i,j,nf) * costetasinfi * CEXP(jk(nf,imilieu)*rsJ2)   
			     
	  Aperp(nf,k)  = Aperp(nf,k)  + phj%J1(i,j,nf) * sinfi * CEXP(jk(nf,imilieu)*rsJ1)  &
	                              - phj%J2(i,j,nf) * cosfi * CEXP(jk(nf,imilieu)*rsJ2)   

	end do
      end do
    end do
  end do
  !-------------------------------------------------------------------------------------------------
  !---------------------- Rayonnement des surfaces de courant plan yOz ----------------------------
  do k=1, cde%nbobjmetal
    phj =>cde%thj(k)
    
    ! précalcul des coordonnées    
    jref = -1 + phj%jdeb - cde%centre_phase(2)
    xrj(1) = (phj%ideb -0.5 - cde%centre_phase(1))*dxsintetacosfi
    xrj(2) = (phj%ifin +0.5 - cde%centre_phase(1))*dxsintetacosfi
    do i=1,2
      ! boucle suivant y
      do j = 1,phj%n2
        rsJ2 =  xrj(i)+ (j +jref+0.5) * dysintetasinfi
	  	  
	! Boucle sur toutes les fréquences
        do nf = 1, tf%n_liste
	  !Calcul des vecteurs de phase pour chaque fréquence
	  Aparal(nf,k) = Aparal(nf,k) - phj%Jy(j,i,nf) * costetasinfi * CEXP(jk(nf,imilieu)*rsJ2)   
	  Aperp(nf,k)  = Aperp(nf,k)  - phj%Jy(j,i,nf) * cosfi * CEXP(jk(nf,imilieu)*rsJ2)   
	end do
      end do
    end do
  end do
  !-------------------------------------------------------------------------------------------------
  !---------------------- Rayonnement des surfaces de courant plan xOz ----------------------------
  do k=1, cde%nbobjmetal
    phj =>cde%thj(k)
    
    ! précalcul des coordonnées    
    iref = -1 + phj%ideb - cde%centre_phase(1)
    yrj(1) = (phj%jdeb -0.5 - cde%centre_phase(2))*dysintetasinfi
    yrj(2) = (phj%jfin +0.5 - cde%centre_phase(2))*dysintetasinfi
    do i=1,2
      ! boucle suivant x
      do j = 1,phj%n1
        rsJ1 = yrj(i) + (j +iref+0.5) * dxsintetacosfi
	! Boucle sur toutes les fréquences
        do nf = 1, tf%n_liste
	  !Calcul des vecteurs de phase pour chaque fréquence
	  Aparal(nf,k) = Aparal(nf,k) - phj%Jx(j,i,nf) * costetacosfi * CEXP(jk(nf,imilieu)*rsJ1) 
	  Aperp(nf,k)  = Aperp(nf,k)  + phj%Jx(j,i,nf) * sinfi * CEXP(jk(nf,imilieu)*rsJ1) 

	end do
      end do
    end do
  end do
  !-------------------------------------------------------------------------------------------------
  
  
  
  ! Calcul des coefficients de réflexion et de transmission
  ! Cela dépend du milieu, de l'angle de rayonnement et de la position des éléments
  nk =>n12
  if (cde%demi_espace_objet == 1)  nk=>n21 ! demi-espace inférieur pour les objets
  
  !---- Calcul de Etheta et Ephi normalisé, ----
  ! le terme exp(-jkr) / r n'est pas inclu !
  if (((cde%demi_espace_objet == 1).and.(costeta<=0)) &
  .or.((cde%demi_espace_objet == 2).and.(costeta>=0)))then
    Rparal = chl_demi_espace_Rparal(theta,tf,nk)
    Rperp  = chl_demi_espace_Rperp (theta,tf,nk)

!    print '("theta,phi,Rparal=",2F6.1,8ES12.2E2)', theta*180/PI,phi*180/pi,Rparal(1:4)
!    print '("theta,phi,Rperp=",2F6.1,8ES12.2E2)', theta*180/PI,phi*180/pi,Rperp(1:4)
    do k=1, nbstructure
      phj =>cde%thj(k)
      h = (phj%hauteur - cde%position_sol_interface)*tv%celz
      do nf = 1, tf%n_liste
	etheta(n,nf) = etheta(n,nf) - jk(nf,imr)*unsur2pi*0.5*Zi(nf,imr) * &
	     (          -Aparal(nf,k)*exp(+jk(nf,imr)*h*costeta) + &
	      Rparal(nf)*Aparal(nf,k)*exp(-jk(nf,imr)*h*costeta))
	      
        ephi  (n,nf) = ephi  (n,nf) + jk(nf,imr)*unsur2pi*0.5*Zi(nf,imr) * &
	     (           Aperp(nf,k)*exp(+jk(nf,imr)*h*costeta) + &
	       Rperp(nf)*Aperp(nf,k)*exp(-jk(nf,imr)*h*costeta))
      end do
    end do
  else
    Tparal = chl_demi_espace_Tparal(theta,tf,nk)
    Tperp  = chl_demi_espace_Tperpmod (theta,tf,nk)
    do nf = 1, tf%n_liste
      if (nk(nf) == CMPLX(1.0,0.0)) then
	cossintheta(nf) = CMPLX(1.0,0.0)
      else
!	  cossintheta(nf) = abs(costeta)/sqrt(1-(nk(nf)*sinteta)**2)
	  cossintheta(nf) = abs(costeta)
      end if
    end do
    
    do k=1, nbstructure
      phj =>cde%thj(k)
      h = (phj%hauteur - cde%position_sol_interface)*tv%celz
!      if (n==1) print '("h=",8ES12.2E2)', h
      do nf = 1, tf%n_liste
        ! Le terme d'atténuation doit etre négatif
        jkth1 = -jk(nf,imt)*h*sqrt(1/nk(nf)**2 -sinteta**2)
	if (real(jkth1)>0) jkth1 = CMPLX(-real(jkth1),aimag(jkth1))
	
	coefc = jk(nf,imt)*unsur2pi*0.5*Zi(nf,imr)*exp(jkth1)
	etheta(n,nf) = etheta(n,nf) - coefc*Tparal(nf)*Aparal(nf,k)
        ephi  (n,nf) = ephi  (n,nf) + coefc*Tperp(nf) *Aperp(nf,k) *cossintheta(nf)
!	                 *abs(costeta)/sqrt(1-(nk(nf)*sinteta)**2)
      end do
    end do
      
  end if
	
  ! On évite les valeurs strictements nulles pour la conversion en dB  
  do nf = 1, tf%n_liste
    if (CABS(etheta(n,nf)) == 0.0) etheta(n,nf) =   1e-32
    if (CABS(ephi(n,nf))   == 0.0) ephi(n,nf)   =   1e-32
  end do

end do
!print '("jk1=",8ES12.2E2)', jk(1:4,1)
!print '("jk2=",8ES12.2E2)', jk(1:4,2)
!print '("zi1=",8ES12.2E2)', Zi(1:4,1)
!print '("zi2=",8ES12.2E2)', Zi(1:4,2)
!print '("n21=",8ES12.2E2)', n21(1:4)
!print '("n12=",8ES12.2E2)', n12(1:4)

deallocate(Aparal, Aperp)
deallocate(jk, epsc, muc,Zi)
deallocate(Rparal,Rperp,Tparal,Tperp)
deallocate(n21,n12)
nullify(nk)

End Subroutine chl_demi_espace_calcul_eth_eph

!===========================================================	
! Subroutine chl_demi_espace_calcul_dth_dph
!  Calcul de la directivité en fonction de la fréquence
!===========================================================	
Subroutine chl_demi_espace_calcul_dth_dph(cde,nf,tza,pr,etheta,ephi,dtheta,dphi)
implicit none
!=== Déclaration des arguments ===
type(st_chl_demi_espace), intent(in)  :: cde
integer, intent(in)  :: nf
type(st_une_zone)   :: tza
complex, dimension(1:nf), intent(in) :: pr
complex, dimension(1:tza%nbcoupleangle,1:nf), intent(in) :: etheta, ephi
real, dimension(1:tza%nbcoupleangle,1:nf), intent(out) :: dtheta, dphi

!===  Déclaration des variables  ===
integer   :: i,n
real, dimension(1:nf,2) :: Ri
real      :: theta, costeta
real, dimension(1:nf) :: pr_normalise

!===   Début des traitements   ===

dtheta = 0.0
dphi   = 0.0
! Calcul de l'impédance de chacun des deux demi-espace
do i=1,2
  Ri(:,i) =sqrt(cde%mu(i)/cde%eps(i))
end do  


! Boucle sur tous les couples d'angle (theta,phi)
do n = 1, tza%nbcoupleangle
  theta = tza%liste_couple_angle(C_THETA,n)
  costeta = cos(theta)
  if (pr(nf) == 0.0) then
    dtheta(:,nf) =   1e-32
    dphi(:,nf)   =   1e-32
    cycle
  end if
  if (costeta>=0) then
    pr_normalise(1:nf) = 2 * PI /(Ri(:,2) * real(pr(1:nf)))
  else
    pr_normalise(1:nf) = 2 * PI /(Ri(:,1) * real(pr(1:nf)))
  end if
  dtheta(n,:) =  pr_normalise(:)* CABS(etheta(n,:))**2
  dphi  (n,:) =  pr_normalise(:)* CABS(ephi  (n,:))**2
  dtheta(n,:) =  MAX(dtheta(n,:),1e-32)
  dphi  (n,:) =  MAX(dphi(n,:),1e-32)
  
end do
end Subroutine chl_demi_espace_calcul_dth_dph

!===========================================================	
! Subroutine chl_demi_espace_Rparal
!  Calcul le coefficient de réflexion sur une bande de fréquence
!  Polarisation parallèle
!===========================================================	
function chl_demi_espace_Rparal(theta,tf,n21) result(Rparal)
implicit none
!=== Déclaration des arguments ===
real, intent(in)  :: theta
type(st_temps_frequence), intent(in)  :: tf
complex,dimension(1:tf%n_liste) :: n21

complex,dimension(1:tf%n_liste) :: Rparal

!===  Déclaration des variables  ===
integer  :: i
complex,dimension(1:tf%n_liste)  :: c1

!===   Début des traitements   ===
c1 = n21**2 -sin(theta)**2
Rparal = CMPLX(0.0,0.0)
do i=1, tf%n_liste
  if (n21(i) /= CMPLX(1.0,0.0))  &
    Rparal(i) = (n21(i)**2*abs(cos(theta))-sqrt(c1(i)))/(n21(i)**2*abs(cos(theta))+sqrt(c1(i)))
end do

End function chl_demi_espace_Rparal

!===========================================================	
! Subroutine chl_demi_espace_Tparal
!  Calcul le coefficient de transmission sur une bande de fréquence
!  Polarisation parallèle
!===========================================================	
function chl_demi_espace_Tparal(theta,tf,n21) result(Tparal)
implicit none
!=== Déclaration des arguments ===
real, intent(in)  :: theta
type(st_temps_frequence), intent(in)  :: tf
complex,dimension(1:tf%n_liste) :: n21
complex,dimension(1:tf%n_liste) :: Tparal

!===  Déclaration des variables  ===
integer  :: i
complex,dimension(1:tf%n_liste)  :: c1

!===   Début des traitements   ===
c1 = 1-(n21*sin(theta))**2
Tparal = CMPLX(1.0,0.0)
do i=1, tf%n_liste
  if (n21(i) /= CMPLX(1.0,0.0))  &
    Tparal(i) = (2*sqrt(c1(i)))/(n21(i)*sqrt(c1(i)) + abs(cos(theta)))
end do
!print '("theta,Tparal=",1F6.1,10ES12.2E2)', theta*180/PI,Tparal(:)
End function chl_demi_espace_Tparal

!===========================================================	
! Subroutine chl_demi_espace_Rparal
!  Calcul le coefficient de réflexion sur une bande de fréquence
!  Polarisation perpendiculaire
!===========================================================	
function chl_demi_espace_Rperp(theta,tf,n21) result(Rperp)
implicit none
!=== Déclaration des arguments ===
real, intent(in)  :: theta
type(st_temps_frequence), intent(in)  :: tf
complex,dimension(1:tf%n_liste) :: n21
complex,dimension(1:tf%n_liste) :: Rperp

!===  Déclaration des variables  ===
integer  :: i
complex,dimension(1:tf%n_liste)  :: c1

!===   Début des traitements   ===
c1 = n21**2 -sin(theta)**2
Rperp = CMPLX(0.0,0.0)
do i=1, tf%n_liste
  if (n21(i) /= CMPLX(1.0,0.0))  &
    Rperp(i) = (abs(cos(theta))-sqrt(c1(i)))/(abs(cos(theta))+sqrt(c1(i)))
end do
End function chl_demi_espace_Rperp

!===========================================================	
! Subroutine chl_demi_espace_Rparal
!  Calcul le coefficient de réflexion sur une bande de fréquence
!  Polarisation perpendiculaire
!===========================================================	
function chl_demi_espace_Tperp(theta,tf,n21) result(Tperp)
implicit none
!=== Déclaration des arguments ===
real, intent(in)  :: theta
type(st_temps_frequence), intent(in)  :: tf
complex,dimension(1:tf%n_liste) :: n21
complex,dimension(1:tf%n_liste) :: Tperp

!===  Déclaration des variables  ===
integer  :: i
complex,dimension(1:tf%n_liste)  :: c1

!===   Début des traitements   ===
c1 = 1-(n21*sin(theta))**2
Tperp = CMPLX(1.0,0.0)
do i=1, tf%n_liste
 if (n21(i) /= CMPLX(1.0,0.0))  &
  Tperp(i) = (2*sqrt(c1(i)))/(sqrt(c1(i)) + n21(i)*abs(cos(theta)))
end do
!print '("theta,Tperp=",1F6.1,10ES12.2E2)', theta*180/PI,Tperp(:)
!print '("n21=",10ES12.2E2)', n21(:)
End function chl_demi_espace_Tperp

!===========================================================	
! Subroutine chl_demi_espace_Rparal
!  Calcul le coefficient de réflexion sur une bande de fréquence
!  Polarisation perpendiculaire
!===========================================================	
function chl_demi_espace_Tperpmod(theta,tf,n21) result(Tperp)
implicit none
!=== Déclaration des arguments ===
real, intent(in)  :: theta
type(st_temps_frequence), intent(in)  :: tf
complex,dimension(1:tf%n_liste) :: n21
complex,dimension(1:tf%n_liste) :: Tperp

!===  Déclaration des variables  ===
integer  :: i
complex,dimension(1:tf%n_liste)  :: c1

!===   Début des traitements   ===
c1 = 1-(n21*sin(theta))**2
Tperp = CMPLX(1.0,0.0)
do i=1, tf%n_liste
  if (n21(i) /= CMPLX(1.0,0.0))  &
    Tperp(i) = 2/(sqrt(c1(i)) + n21(i)*abs(cos(theta)))
end do
!print '("theta,Tperp=",1F6.1,10ES12.2E2)', theta*180/PI,Tperp(:)
!print '("n21=",10ES12.2E2)', n21(:)
End function chl_demi_espace_Tperpmod


!===========================================================	
! Subroutine chl_demi_espace_deallocate
!  Libération de la mémoire
!===========================================================	
Subroutine chl_demi_espace_deallocate(cde)
implicit none
!=== Déclaration des arguments ===
type(st_chl_demi_espace), intent(inout)  :: cde

!===  Déclaration des variables  ===
integer  :: i
type(st_pointeur_H_J), pointer :: phj

!===   Début des traitements   ===

do i=1, cde%nbobjmetal
  phj =>cde%thj(i)
  deallocate(phj%J1,phj%J2)
  if (associated(phj%Jx)) deallocate(phj%Jx)
  if (associated(phj%Jy)) deallocate(phj%Jy)
  nullify(phj%h1inf)
  nullify(phj%h1sup)
  nullify(phj%h2inf)
  nullify(phj%h2sup)
  nullify(phj%hz_jxinf)
  nullify(phj%hz_jxsup)
  nullify(phj%hz_jyinf)
  nullify(phj%hz_jysup)
end do
deallocate(cde%thj)
nullify(cde%tmetal)

End Subroutine chl_demi_espace_deallocate


end module champ_lointain_demi_espace



