!/////////////////////////////////////////////////////////////////////////////
!// Name:        mfdtd_milieu_volume.f90
!// Purpose:     Calcul des paramètres électromagnétique dans chaque  
!//              cellules du volume de calcul, moyennage aux interfaces    
!// Author:      Christophe Guiffaut
!// Modified by:  Christophe Guiffaut
!// Created  :     Novembre 2001
!// Lieu     : XLIM - Faculté des sciences - 123 Av. Albert Thomas - Limoges
!// Copyright:   (c) CNRS, Université de Limoges
!// Licence  : CECILL-C - Octobre 2006
!/////////////////////////////////////////////////////////////////////////////

module mfdtd_milieu_volume

use m_constantes
use m_constantes_scientifiques, only : EPSILON_0,MU_0
use mfdtd_structure, only : st_structure, &
                             st_surf_rugueuse, &
			     st_structure_plaque_mince, &
			     st_structure_fente
use mfdtd_volume, only : st_volume
use m_milieu_simple, only : st_milieu_simple
use m_milieu_fractal, only : st_milieu_fractal, &
                                  milieu_fractal_precalcul
				 
use m_liste_milieu, only : st_liste_milieu_simple, &
                          identifie_milieu_fractal, &
			  liste_milieu_nom, &
		          pointe_milieu_fractal
use mfdtd_champ, only : st_champ

implicit none

! ==========================================================
! == Type milieu_volume                            ==
! ==========================================================
type st_milieu_volume
  logical :: flag_eps, flag_mu, flag_eps_sigmae, flag_mu_sigmah
  real  :: mu_default, epsilon_default, sigmae_defaut, sigmah_defaut
  integer(kind=2), dimension(:,:,:), pointer :: milieu
  
  real, dimension(:,:,:), pointer   :: epsx,epsy,epsz
  real, dimension(:,:,:), pointer   :: mux, muy, muz
  real, dimension(:,:,:), pointer   :: sigex,sigey,sigez
  real, dimension(:,:,:), pointer   :: sighx,sighy,sighz


  real, dimension(:,:,:), pointer   :: tampon
  real, dimension(:,:,:), pointer :: surfxy, surfyz, surfxz
  real, dimension(:,:), pointer :: axex, axey, axez
  
  type(st_liste_milieu_simple), pointer :: lms
  
  !-------- Taille mémoire et Nbre d'opération par seconde ----
  real            :: taille_mem,taille_mem_extra
  integer         :: nb_oper
  
end type st_milieu_volume
 
contains

! ==========================================================
! == Subroutine milieu_volume_initdefaut                  ==
! == Initialisation par defaut des variables              ==
! ==========================================================	
subroutine milieu_volume_initdefaut(tm,lms)

type(st_milieu_volume), intent(inout) :: tm
type(st_liste_milieu_simple), pointer :: lms

tm%flag_eps     = .false.
tm%flag_eps_sigmae  = .false.
tm%flag_mu      = .false.
tm%flag_mu_sigmah  = .false.
tm%lms =>lms

tm%taille_mem     = 0.0
tm%taille_mem_extra =0.0
tm%nb_oper        = 0

end subroutine milieu_volume_initdefaut

! ==========================================================
! == Subroutine milieu_volume_init                        ==
! == Initialisation des paramètres par défaut
! ==  Recherche des inhomogénéités
! ==========================================================
subroutine milieu_volume_init(tm,tv,ts)
type(st_milieu_volume), intent(inout) :: tm
type(st_volume), intent(inout)        :: tv
type(st_structure), intent(in)        :: ts

tm%epsilon_default = tv%epsilon				
tm%mu_default      = tv%mu
tm%sigmae_defaut   = tv%sigma_e
tm%sigmah_defaut   = tv%sigma_h

! Flag pour l'allocation des tableaux epsilon,mu,sigmaE,sigmaH
if (ts%type_milieu_dielec==VOL_DIELEC_INHOMOGENE .or. &
    ts%type_milieu_dielec==VOL_DIELEC_PERTES_INHOMOGENE) then
  tm%flag_eps     = .true.
end if

if (ts%type_milieu_dielec==VOL_DIELEC_PERTES_INHOMOGENE) then
  tm%flag_eps_sigmae  = .true.
end if

if (ts%type_milieu_magnet == VOL_MAGNET_INHOMOGENE .or. &
    ts%type_milieu_magnet == VOL_MAGNET_PERTES_INHOMOGENE ) then
  tm%flag_mu      = .true.
end if

if (ts%type_milieu_magnet == VOL_MAGNET_PERTES_INHOMOGENE) then
  tm%flag_mu_sigmah  = .true.
end if
call milieu_volume_mem_nboper(tm,tv)

end subroutine milieu_volume_init

! ==========================================================
! == Subroutine milieu_volume_mem_nboper                  ==
! ==   Evaluation de la taille mémoire en octets dans la
! ==    structure et du nbre d'opération flottante si impactant
! ==     sur le temps de calcul
!===========================================================	
subroutine milieu_volume_mem_nboper(tm,v)

type(st_milieu_volume), intent(inout) :: tm
type(st_volume), intent(inout)        :: v

real :: test,mem,mem1
test=0.0
mem  = (v%ivolf-v%ivold+3)*(v%jvolf-v%jvold+3)*(v%kvolf-v%kvold+3)*kind(test)
mem1 = (v%ivolf-v%ivold+4)*(v%jvolf-v%jvold+4)*(v%kvolf-v%kvold+4)*kind(test)
tm%taille_mem = (v%ivolf-v%ivold+4)*(v%jvolf-v%jvold+4)*(v%kvolf-v%kvold+4)*2

! Calcul de la taille Max : cumulation des tableaux temporaires 
!  eps,mu,sige,sigh avec les tableaux de coefficients multiplicatif du champ
tm%taille_mem_extra = 0
if (tm%flag_eps)        tm%taille_mem_extra=tm%taille_mem_extra+(mem+mem1)*3
if (tm%flag_eps_sigmae) tm%taille_mem_extra=tm%taille_mem_extra+(mem+mem1)*3
if (tm%flag_mu)         tm%taille_mem_extra=tm%taille_mem_extra+(mem+mem1)*3
if (tm%flag_mu_sigmah)  tm%taille_mem_extra=tm%taille_mem_extra+(mem+mem1)*3

end subroutine milieu_volume_mem_nboper


! ==========================================================
! == Subroutine milieu_volume_extrait_epsilon
! == retourne la permittivité correspondant à la position
! == donnée et à la direction données                    
! ==========================================================	
function milieu_volume_extrait_epsilon(tm,i,j,k,direction) result(eps)

type(st_milieu_volume), intent(in) :: tm
integer, intent(in)  :: i,j,k,direction
real                 :: eps

eps = 0.0
if (tm%flag_eps) then
  if (direction == DIRECTION_X) then
    eps = tm%epsx(i,j,k) * EPSILON_0
  else if (direction == DIRECTION_Y) then
    eps = tm%epsy(i,j,k) * EPSILON_0
  else if (direction == DIRECTION_Z) then
    eps = tm%epsz(i,j,k) * EPSILON_0
  end if
else
  eps = tm%epsilon_default
end if

end function milieu_volume_extrait_epsilon

! ==========================================================
! == Subroutine milieu_volume_extrait_mu
! == retourne la perméabilité correspondant à la position
! == donnée et à la direction données                    
! ==========================================================	
function milieu_volume_extrait_mu(tm,i,j,k,direction) result(mu)

type(st_milieu_volume), intent(in) :: tm
integer, intent(in)  :: i,j,k,direction
real                 :: mu

mu= 0.0
if (tm%flag_mu) then
  if (direction == DIRECTION_X) then
    mu = tm%mux(i,j,k) * MU_0
  else if (direction == DIRECTION_Y) then
    mu = tm%muy(i,j,k) * MU_0
  else if (direction == DIRECTION_Z) then
    mu = tm%muz(i,j,k) * MU_0
  end if
else
  mu = tm%mu_default
end if

end function milieu_volume_extrait_mu

! ==========================================================
! == Subroutine milieu_volume_extrait_sige
! == retourne la conductivité électrique correspondant à la position
! == donnée et à la direction données                    
! ==========================================================	
function milieu_volume_extrait_sige(tm,i,j,k,direction) result(sige)
type(st_milieu_volume), intent(in) :: tm
integer, intent(in)  :: i,j,k,direction
real                 :: sige

sige = 0.0
if (tm%flag_eps_sigmae) then
  if (direction == DIRECTION_X) then
    sige = tm%sigex(i,j,k) 
  else if (direction == DIRECTION_Y) then
    sige = tm%sigey(i,j,k)
  else if (direction == DIRECTION_Z) then
    sige = tm%sigez(i,j,k) 
  end if
else
  sige = tm%sigmae_defaut
end if

end function milieu_volume_extrait_sige

! ==========================================================
! == Subroutine milieu_volume_extrait_sigh
! == retourne la conductivité magnétique correspondant à la position
! == donnée et à la direction données                    
! ==========================================================	
function milieu_volume_extrait_sigh(tm,i,j,k,direction) result(sigh)
type(st_milieu_volume), intent(in) :: tm
integer, intent(in)  :: i,j,k,direction
real                 :: sigh

sigh = 0.0
if (tm%flag_mu_sigmah) then
  if (direction == DIRECTION_X) then
    sigh = tm%sighx(i,j,k)
  else if (direction == DIRECTION_Y) then
    sigh = tm%sighy(i,j,k) 
  else if (direction == DIRECTION_Z) then
    sigh = tm%sighz(i,j,k) 
  end if
else
  sigh = tm%sigmah_defaut
end if

end function milieu_volume_extrait_sigh


! ==========================================================
! == Subroutine cherche_nu_milieu                    ==
! == récupère le n° milieu correspondant au nom de     ==
! == milieu donné                                         ==
! ==========================================================	
function cherche_nu_milieu(t_lmnd,nommilieu,nu)

implicit none

type(st_liste_milieu_simple), intent(in) :: t_lmnd
character(LEN=LEN_MOT_CLE), intent(in) :: nommilieu
integer, intent(out) :: nu
logical   ::cherche_nu_milieu

integer  :: i

nu = 0
cherche_nu_milieu = .false.
! boucle sur tous les milieux
do i= 1, t_lmnd%nb_ms_tous
  if (nommilieu == t_lmnd%t_ms_tous(i)%nom) then
    cherche_nu_milieu = .true.
    nu = i
    exit
  end if  
end do
 
end function cherche_nu_milieu

! ==========================================================
! == Subroutine cherche_epsilon_cellule                   ==
! == récupère la permittivité relative correspondant      ==
! == au n° de  milieu donné                               ==
! ==========================================================	
function cherche_epsilon_cellule(tm,i,j,k) result(epsr)

type(st_milieu_volume), intent(in) :: tm
integer, intent(in) :: i,j,k
real, dimension(3)   ::epsr

integer :: n
n = tm%milieu(i,j,k)
if (n<=0) then
  epsr = tm%epsilon_default/EPSILON_0
  return
end if
epsr = tm%lms%t_ms_tous(n)%eps_r(1:3)
 
end function cherche_epsilon_cellule

! ==========================================================
! == Subroutine cherche_mu_cellule                        ==
! == récupère la perméabilité relative correspondant      ==
! == au n° de milieu donné                                ==
! ==========================================================	
function cherche_mu_cellule(tm,i,j,k) result(mur)

type(st_milieu_volume), intent(in) :: tm
integer, intent(in) :: i,j,k
real, dimension(3)   ::mur

integer :: n
n = tm%milieu(i,j,k)
if (n<=0) then
  mur = tm%mu_default/MU_0
  return
end if
mur = tm%lms%t_ms_tous(n)%mu_r(1:3)
 
end function cherche_mu_cellule

! ==========================================================
! == Subroutine cherche_sigmaE_cellule                    ==
! == récupère la conductivité électrique correspondant    ==
! ==  au n° de milieu donné                               ==
! ==========================================================	
function cherche_sigmaE_cellule(tm,i,j,k) result(sige)
type(st_milieu_volume), intent(in) :: tm
integer, intent(in) :: i,j,k
real, dimension(3)   ::sige

integer :: n
n = tm%milieu(i,j,k)
if (n<=0) then
  sige = tm%sigmae_defaut
  return
end if

sige = tm%lms%t_ms_tous(n)%sig_e(1:3)
 
end function cherche_sigmaE_cellule

! ==========================================================
! == Subroutine cherche_sigmaH_cellule                    ==
! == récupère la conductivité magnétique correspondant    ==
! ==  au n° de milieu donné                               ==
! ==========================================================	
function cherche_sigmaH_cellule(tm,i,j,k) result(sigh)

type(st_milieu_volume), intent(in) :: tm
integer, intent(in) :: i,j,k
real, dimension(3)   ::sigh

integer :: n
n = tm%milieu(i,j,k)
if (n<=0) then
  sigh = tm%sigmah_defaut
  return
end if
sigh = tm%lms%t_ms_tous(n)%sig_h(1:3)
 
end function cherche_sigmaH_cellule


! ==========================================================
! == Subroutine cherche_epsilon_milieu                    ==
! == récupère la permittivité correspondant au nom de     ==
! == milieu donné                                         ==
! ==========================================================	
function cherche_epsilon_milieu(t_lmnd,nommilieu,epsilon)

implicit none

type(st_liste_milieu_simple), intent(in) :: t_lmnd
character(LEN=LEN_MOT_CLE), intent(in) :: nommilieu
real, dimension(1:3), intent(out) :: epsilon
logical   ::cherche_epsilon_milieu

integer  :: i

cherche_epsilon_milieu = .false.
! boucle sur tous les milieux
do i= 1, t_lmnd%nb_ms_tous
  if (nommilieu == t_lmnd%t_ms_tous(i)%nom) then
    cherche_epsilon_milieu = .true.
    epsilon(1:3) = t_lmnd%t_ms_tous(i)%eps_r(1:3)
    exit
  end if  
end do
 
end function cherche_epsilon_milieu

! ==========================================================
! == Subroutine cherche_mu_milieu                         ==
! == récupère la perméabilité correspondant au nom de     ==
! == milieu donné                                         ==
! ==========================================================	
function cherche_mu_milieu(t_lmnd,nommilieu,mu)

implicit none

type(st_liste_milieu_simple), intent(in) :: t_lmnd
character(LEN=LEN_MOT_CLE), intent(in) :: nommilieu
real, dimension(1:3), intent(out) :: mu
logical   ::cherche_mu_milieu

integer  :: i

cherche_mu_milieu = .false.
! boucle sur tous les milieux
do i= 1, t_lmnd%nb_ms_tous
  if (nommilieu == t_lmnd%t_ms_tous(i)%nom) then
    cherche_mu_milieu = .true.
    mu(1:3) = t_lmnd%t_ms_tous(i)%mu_r(1:3)
    exit
  end if  
end do
 
end function cherche_mu_milieu

! ==========================================================
! == Subroutine cherche_sigmae_milieu                     ==
! == récupère la conductivité électrique correspondant au ==
! == nom de milieu donné                                  ==
! ==========================================================	
function cherche_sigmae_milieu(t_lmnd,nommilieu,sigmae)

implicit none

type(st_liste_milieu_simple), intent(in) :: t_lmnd
character(LEN=LEN_MOT_CLE), intent(in) :: nommilieu
real, dimension(1:3), intent(out) :: sigmae
logical   ::cherche_sigmae_milieu

integer  :: i

cherche_sigmae_milieu = .false.
! boucle sur tous les milieux
do i= 1, t_lmnd%nb_ms_tous
  if (nommilieu == t_lmnd%t_ms_tous(i)%nom) then
    cherche_sigmae_milieu = .true.
    sigmae(1:3) = t_lmnd%t_ms_tous(i)%sig_e(1:3)
    exit
  end if  
end do
 
end function cherche_sigmae_milieu

! ==========================================================
! == Subroutine cherche_sigmah_milieu                     ==
! == récupère la conductivité magnétique correspondant au ==
! == nom de milieu donné                                  ==
! ==========================================================	
function cherche_sigmah_milieu(t_lmnd,nommilieu,sigmah)

implicit none

type(st_liste_milieu_simple), intent(in) :: t_lmnd
character(LEN=LEN_MOT_CLE), intent(in) :: nommilieu
real, dimension(1:3), intent(out) :: sigmah
logical   ::cherche_sigmah_milieu

integer  :: i

cherche_sigmah_milieu = .false.
! boucle sur tous les milieux
do i= 1, t_lmnd%nb_ms_tous
  if (nommilieu == t_lmnd%t_ms_tous(i)%nom) then
    cherche_sigmah_milieu = .true.
    sigmah(1:3) = t_lmnd%t_ms_tous(i)%sig_h(1:3)
    exit
  end if  
end do
 
end function cherche_sigmah_milieu

! ==========================================================
! == Subroutine milieu_calcul_surfaces                   ==
! == Calcul de la surface de chaque cellule pour les 3 
!==   plans (xy,yz,xz). Calcul aussi de toutes les 
!==  4 sub-surface cellulaires     
! ==========================================================
subroutine milieu_calcul_surfaces(t_m,v)

implicit none

type(st_milieu_volume), intent(inout) :: t_m
type(st_volume), intent(in)           :: v

integer  :: i,j

!***  Surfaces plan xOy ***
do j = v%jvold-1,v% jvolf+2
do i = v%ivold-1,v% ivolf+2
  t_m%surfxy(i,j,1) = 0.25*v%dxyz(1)*v%dxyz(2)
  t_m%surfxy(i,j,2) = 0.25*v%dxyz(1)*v%dxyz(2)
  t_m%surfxy(i,j,3) = 0.25*v%dxyz(1)*v%dxyz(2)
  t_m%surfxy(i,j,4) = 0.25*v%dxyz(1)*v%dxyz(2)
  t_m%surfxy(i,j,5) = v%dxyz(1)*v%dxyz(2)
end do; end do


!***  Surfaces plan yOz ***
do j = v%kvold-1, v%kvolf+2
do i = v%jvold-1, v%jvolf+2
  t_m%surfyz(i,j,3) = 0.25*v%dxyz(2)*v%dxyz(3)
  t_m%surfyz(i,j,1) = 0.25*v%dxyz(2)*v%dxyz(3)
  t_m%surfyz(i,j,2) = 0.25*v%dxyz(2)*v%dxyz(3)
  t_m%surfyz(i,j,4) = 0.25*v%dxyz(2)*v%dxyz(3)
  t_m%surfyz(i,j,5) = v%dxyz(2)*v%dxyz(3)
end do; end do

!***  Surfaces plan xOz ***
do j = v%kvold-1,v% kvolf+2
do i = v%ivold-1,v% ivolf+2
  t_m%surfxz(i,j,2) = 0.25*v%dxyz(1)*v%dxyz(3)
  t_m%surfxz(i,j,1) = 0.25*v%dxyz(1)*v%dxyz(3)
  t_m%surfxz(i,j,3) = 0.25*v%dxyz(1)*v%dxyz(3)
  t_m%surfxz(i,j,4) = 0.25*v%dxyz(1)*v%dxyz(3)
  t_m%surfxz(i,j,5) = v%dxyz(1)*v%dxyz(3)
end do; end do
				
end subroutine milieu_calcul_surfaces

! ==========================================================
! == Subroutine milieu_calcul_deltas                   ==
! == Affectation des pas spatiaux dans chaque direction
!==  Calcul aussi de tous les sous-pas spatiaux
! ==========================================================
subroutine milieu_calcul_deltas(t_m,v)

implicit none

type(st_milieu_volume), intent(inout) :: t_m
type(st_volume), intent(in)           :: v

integer  :: i,j,k

!***  Axe Ox ***
do i = v%ivold-1, v%ivolf+2
  t_m%axex(i,1) = 0.5*v%dxyz(1)
  t_m%axex(i,2) = 0.5*v%dxyz(1)
  t_m%axex(i,3) = v%dxyz(1) 
end do
 
!***  Axe Oy ***
do i = v%jvold-1, v%jvolf+2
  t_m%axey(i,1) = 0.5*v%dxyz(2)
  t_m%axey(i,2) = 0.5*v%dxyz(2)
  t_m%axey(i,3) = v%dxyz(2)  
end do

!***  Axe Oz ***
do i = v%kvold-1, v%kvolf+2
  t_m%axez(i,1) = 0.5*v%dxyz(3)
  t_m%axez(i,2) = 0.5*v%dxyz(3)
  t_m%axez(i,3) = v%dxyz(3)  
end do

end subroutine milieu_calcul_deltas

! ==========================================================
! == Subroutine milieu_prolonge_frontiere_sup             ==
! == Prolonge les frontières extérieures (ubound-1 vers ubound)
! ==========================================================
subroutine milieu_prolonge_frontiere_sup(te)

implicit none

real, dimension(1:,1:,1:), intent(inout) :: te 

integer,dimension(3)  :: p 

p = UBOUND(te)
te(p(1),:,:) = te(p(1)-1,:,:)
te(:,p(2),:) = te(:,p(2)-1,:)
te(:,:,p(3)) = te(:,:,p(3)-1)
end subroutine milieu_prolonge_frontiere_sup



! ==========================================================
! == Subroutine milieu_calcul_moy_electrique          ==
! ==========================================================
subroutine milieu_calcul_moy_electrique(te,nbx,nby,nbz,ox,oy,oz,typesurface,surf)

real, dimension(1:,1:,1:), intent(inout) :: te 
integer, intent(in)  :: nbx,nby,nbz
integer, intent(in)  :: ox, oy, oz  ! offsets qui valent 0 ou 1
integer, intent(in)  :: typesurface 
real,dimension(1:,1:,1:), intent(in)  :: surf

integer, target  :: i,j,k
integer, pointer  :: ip,jp
integer,dimension(3)  :: profil
real, dimension(:,:,:), allocatable :: temp

! tableau temporaire
profil = UBOUND(te)
allocate(temp(1:profil(1),1:profil(2),1:profil(3)))
temp(:,:,:) = te(:,:,:)

! pointeur sur indice en fonction de la surface
if (typesurface == 1) then
!  ip => i
!  jp => j
! moyenne
do k= 2, nbz
do j = 2, nby
do i = 2, nbx
!  ip = i ; jp = j
  te(i,j,k) = (temp(i,j,k)      * surf(i,j,4) + &
              temp(i-ox,j-oy,k) * surf(i,j,1) + &
	      temp(i-ox,j,k-oz) * surf(i,j,2) + &
	      temp(i,j-oy,k-oz) * surf(i,j,3)) / surf(i,j,5)
end do; end do; end do				

else if (typesurface == 2) then
!  ip => j
!  jp => k
! moyenne
do k= 2, nbz
do j = 2, nby
do i = 2, nbx
  te(i,j,k) = (temp(i,j,k)      * surf(j,k,4) + &
              temp(i-ox,j-oy,k) * surf(j,k,1) + &
	      temp(i-ox,j,k-oz) * surf(j,k,2) + &
	      temp(i,j-oy,k-oz) * surf(j,k,3)) / surf(j,k,5)
end do; end do; end do				

else if (typesurface == 3) then
!  ip => i
!  jp => k
! moyenne
do k= 2, nbz
do j = 2, nby
do i = 2, nbx
  te(i,j,k) = (temp(i,j,k)      * surf(i,k,4) + &
              temp(i-ox,j-oy,k) * surf(i,k,1) + &
	      temp(i-ox,j,k-oz) * surf(i,k,2) + &
	      temp(i,j-oy,k-oz) * surf(i,k,3)) / surf(i,k,5)
end do; end do; end do				

else
  stop 'Arret car erreur dans la subroutine milieu_calcul_moy_electrique'
end if



deallocate(temp)
! prolongement sur les frontières inférieures
te(1,:,:) = te(2,:,:)
te(:,1,:) = te(:,2,:)
te(:,:,1) = te(:,:,2)
end subroutine milieu_calcul_moy_electrique

! ==========================================================
! == Subroutine milieu_calcul_moy_magnetique                   ==
! ==========================================================
subroutine milieu_calcul_moy_magnetique(tm,nbx,nby,nbz,ox,oy,oz,typecomposante,delta)

implicit none

real, dimension(1:,1:,1:), intent(inout) :: tm
integer, intent(in)  :: nbx,nby,nbz
integer, intent(in)  :: ox, oy, oz  ! offsets qui valent 0 ou 1
integer, intent(in)  :: typecomposante 
real,dimension(1:,1:), intent(in)  :: delta

integer,target  :: i,j,k
integer, pointer  :: ip
integer,dimension(3)  :: profil
real, dimension(:,:,:), allocatable :: temp
real  :: vtemp

! tableau temporaire
profil = UBOUND(tm)
allocate(temp(1:profil(1),1:profil(2),1:profil(3)))
temp(:,:,:) = tm(:,:,:)

! pointeur indice tableau en fonction de l'orientation de la composante
if (typecomposante == 1) then
  ip => i
else if (typecomposante == 2) then
  ip => j
else if (typecomposante == 3) then
  ip => k
else
  stop 'Arret car erreur dans la subroutine milieu_calcul_moy_magnetique'
end if
do k= 2, nbz
do j = 2, nby
do i = 2, nbx
  vtemp = (temp(i-ox,j-oy,k-oz) *delta(ip,1) + temp(i,j,k) *delta(ip,2))
  if (.not.(vtemp == 0)) &
    tm(i,j,k) = temp(i-ox,j-oy,k-oz) * temp(i,j,k) * delta(ip,3) / vtemp
end do; end do; end do				

deallocate(temp)
! prolongement sur les frontières inférieures
tm(1,:,:) = tm(2,:,:)
tm(:,1,:) = tm(:,2,:)
tm(:,:,1) = tm(:,:,2)

end subroutine milieu_calcul_moy_magnetique

! ==========================================================
! == Subroutine milieu_calcul_moy_simple_mu                   ==
! ==========================================================
subroutine milieu_calcul_moy_simple_mu(tm,nbx,nby,nbz,ox,oy,oz,typecomposante,delta)

implicit none

real, dimension(1:,1:,1:), intent(inout) :: tm
integer, intent(in)  :: nbx,nby,nbz
integer, intent(in)  :: ox, oy, oz  ! offsets qui valent 0 ou 1
integer, intent(in)  :: typecomposante 
real,dimension(1:,1:), intent(in)  :: delta

integer,target  :: i,j,k
integer, pointer  :: ip
integer,dimension(3)  :: profil
real, dimension(:,:,:), allocatable :: temp
real  :: vtemp

! tableau temporaire
profil = UBOUND(tm)
allocate(temp(1:profil(1),1:profil(2),1:profil(3)))
temp(:,:,:) = tm(:,:,:)

! pointeur indice tableau en fonction de l'orientation de la composante
if (typecomposante == 1) then
  ip => i
else if (typecomposante == 2) then
  ip => j
else if (typecomposante == 3) then
  ip => k
else
  stop 'Arret car erreur dans la subroutine milieu_calcul_moy_magnetique'
end if
do k= 2, nbz
do j = 2, nby
do i = 2, nbx
  vtemp = (temp(i-ox,j-oy,k-oz) + temp(i,j,k))*0.5
end do; end do; end do				

deallocate(temp)
! prolongement sur les frontières inférieures
tm(1,:,:) = tm(2,:,:)
tm(:,1,:) = tm(:,2,:)
tm(:,:,1) = tm(:,:,2)

end subroutine milieu_calcul_moy_simple_mu

! ==========================================================
! == Subroutine milieu_epsilon_surf_rugueuse              ==
! == Remplissage du tableau epsilon à partir d'une surface ==
! == rugueuse. Le moyennage doit impérativement etre appliqué
! == par la suite.
! ==========================================================	
subroutine milieu_epsilon_surf_rugueuse(imilieu,ideb,t_m,t_lmnd,t_s)

!--- DÉCLARATION DES ARGUMENTS ---
integer, intent(in)  :: imilieu
integer, intent(inout) :: ideb
type(st_milieu_volume), intent(inout) :: t_m
type(st_liste_milieu_simple), intent(in) :: t_lmnd
type(st_structure), intent(in)           :: t_s

!--- DÉCLARATION DES VARIABLES LOCALES
integer   :: i,j,k,l,h
logical   :: flagnommilieu1, flagnommilieu2
real,dimension(1:3)  :: epsr1,epsr2
type(st_surf_rugueuse), pointer  :: t_sr

!--- DEBUT DES TRAITEMENTS ---
do l =ideb,t_s%nbsurf_rugueuse
  t_sr => t_s%t_surf_rugueuse(l)
!  print '("l = ",I2)', l
!  print '("ideb = ",I2)', ideb
!  print '("imilieu = ",I2)', imilieu
!  print '("ind_milieu_avant = ",I2)', t_sr%ind_milieu_avant
!  print '("coord = ",6I4)', t_sr%coord
  if (imilieu-1 ==t_sr%ind_milieu_avant) then
    !--- On récupère la valeur des deux milieux ---
    flagnommilieu1 = cherche_epsilon_milieu(t_lmnd,t_sr%nommilieu1,epsr1)
    flagnommilieu2 = cherche_epsilon_milieu(t_lmnd,t_sr%nommilieu2,epsr2)
    if ((.not.flagnommilieu1).or.(.not.flagnommilieu2)) then 
      print *,'Nom de milieu (eps) non reconnu dans le bloc structure :'
      stop 'Chomage technique ...'
    end if
    
    !---  SURFACE RUGUEUSE PLAN XOY OBLIGATOIRE---
    do j = t_sr%coord(3),t_sr%coord(4)-1
    do i = t_sr%coord(1),t_sr%coord(2)-1
      h = t_sr%hauteur(i-t_sr%coord(1)+1,j-t_sr%coord(3)+1)
!      print '("i,j,h = ",3I4)', i,j,h
      ! milieu inférieur
      t_m%epsx(i,j,t_sr%coord(5):t_sr%coord(5)+h-1) = epsr1(1)
      t_m%epsy(i,j,t_sr%coord(5):t_sr%coord(5)+h-1) = epsr1(2)
      t_m%epsz(i,j,t_sr%coord(5):t_sr%coord(5)+h-1) = epsr1(3)
      
      ! milieu supérieur
      t_m%epsx(i,j,t_sr%coord(5)+h:t_sr%coord(6)-1) = epsr2(1)
      t_m%epsy(i,j,t_sr%coord(5)+h:t_sr%coord(6)-1) = epsr2(2)
      t_m%epsz(i,j,t_sr%coord(5)+h:t_sr%coord(6)-1) = epsr2(3)
    end do;end do
  else
    exit
  end if
end do
ideb = l
end subroutine milieu_epsilon_surf_rugueuse

! ==========================================================
! == Subroutine milieu_sige_surf_rugueuse                 ==
! == Remplissage du tableau sigmae à partir d'une surface ==
! == rugueuse. Le moyennage doit impérativement etre appliqué
! == par la suite.
! ==========================================================	
subroutine milieu_sige_surf_rugueuse(imilieu,ideb,t_m,t_lmnd,t_s)

!--- DÉCLARATION DES ARGUMENTS ---
integer, intent(in)  :: imilieu
integer, intent(inout) :: ideb
type(st_milieu_volume), intent(inout) :: t_m
type(st_liste_milieu_simple), intent(in) :: t_lmnd
type(st_structure), intent(in)           :: t_s

!--- DÉCLARATION DES VARIABLES LOCALES
integer   :: i,j,k,l,h
logical   :: flagnommilieu1, flagnommilieu2
real,dimension(1:3)  :: sige1,sige2
type(st_surf_rugueuse), pointer  :: t_sr

!--- DEBUT DES TRAITEMENTS ---
do l =ideb,t_s%nbsurf_rugueuse
  t_sr => t_s%t_surf_rugueuse(l)
  if (imilieu-1 ==t_sr%ind_milieu_avant) then
    !--- On récupère la valeur des deux milieux ---
    flagnommilieu1 = cherche_sigmae_milieu(t_lmnd,t_sr%nommilieu1,sige1)
    flagnommilieu2 = cherche_sigmae_milieu(t_lmnd,t_sr%nommilieu2,sige2)
    if ((.not.flagnommilieu1).or.(.not.flagnommilieu2)) then 
      print *,'Nom de milieu (sigmae) non reconnu dans le bloc structure :'
      stop 'Chomage technique ...'
    end if
    
    !---  SURFACE RUGUEUSE PLAN XOY OBLIGATOIRE---
    do j = t_sr%coord(3),t_sr%coord(4)-1
    do i = t_sr%coord(1),t_sr%coord(2)-1
      h = t_sr%hauteur(i-t_sr%coord(1)+1,j-t_sr%coord(3)+1)
!      h = t_sr%hauteur(i,j)
      ! milieu inférieur
      t_m%sigex(i,j,t_sr%coord(5):t_sr%coord(5)+h-1) = sige1(1)
      t_m%sigey(i,j,t_sr%coord(5):t_sr%coord(5)+h-1) = sige1(2)
      t_m%sigez(i,j,t_sr%coord(5):t_sr%coord(5)+h-1) = sige1(3)
      
      ! milieu supérieur
      t_m%sigex(i,j,t_sr%coord(5)+h:t_sr%coord(6)-1) = sige2(1)
      t_m%sigey(i,j,t_sr%coord(5)+h:t_sr%coord(6)-1) = sige2(2)
      t_m%sigez(i,j,t_sr%coord(5)+h:t_sr%coord(6)-1) = sige2(3)
    end do;end do
  else
    exit
  end if
end do
ideb = l
end subroutine milieu_sige_surf_rugueuse

! ==========================================================
! == Subroutine milieu_sigh_surf_rugueuse                 ==
! == Remplissage du tableau sigmah à partir d'une surface ==
! == rugueuse. Le moyennage doit impérativement etre appliqué
! == par la suite.
! ==========================================================	
subroutine milieu_sigh_surf_rugueuse(imilieu,ideb,t_m,t_lmnd,t_s)

!--- DÉCLARATION DES ARGUMENTS ---
integer, intent(in)  :: imilieu
integer, intent(inout) :: ideb
type(st_milieu_volume), intent(inout) :: t_m
type(st_liste_milieu_simple), intent(in) :: t_lmnd
type(st_structure), intent(in)           :: t_s

!--- DÉCLARATION DES VARIABLES LOCALES
integer   :: i,j,k,l,h
logical   :: flagnommilieu1, flagnommilieu2
real,dimension(1:3)  :: sigh1,sigh2
type(st_surf_rugueuse), pointer  :: t_sr

!--- DEBUT DES TRAITEMENTS ---
do l =ideb,t_s%nbsurf_rugueuse
  t_sr => t_s%t_surf_rugueuse(l)
  if (imilieu-1 ==t_sr%ind_milieu_avant) then
    !--- On récupère la valeur des deux milieux ---
    flagnommilieu1 = cherche_sigmah_milieu(t_lmnd,t_sr%nommilieu1,sigh1)
    flagnommilieu2 = cherche_sigmah_milieu(t_lmnd,t_sr%nommilieu2,sigh2)
    if ((.not.flagnommilieu1).or.(.not.flagnommilieu2)) then 
      print *,'Nom de milieu (sigmah) non reconnu dans le bloc structure :'
      stop 'Chomage technique ...'
    end if
    
    !---  SURFACE RUGUEUSE PLAN XOY OBLIGATOIRE---
    do j = t_sr%coord(3),t_sr%coord(4)-1
    do i = t_sr%coord(1),t_sr%coord(2)-1
      h = t_sr%hauteur(i-t_sr%coord(1)+1,j-t_sr%coord(3)+1)
!      h = t_sr%hauteur(i,j)
      ! milieu inférieur
      t_m%sighx(i,j,t_sr%coord(5):t_sr%coord(5)+h-1) = sigh1(1)
      t_m%sighy(i,j,t_sr%coord(5):t_sr%coord(5)+h-1) = sigh1(2)
      t_m%sighz(i,j,t_sr%coord(5):t_sr%coord(5)+h-1) = sigh1(3)
      
      ! milieu supérieur
      t_m%sighx(i,j,t_sr%coord(5)+h:t_sr%coord(6)-1) = sigh2(1)
      t_m%sighy(i,j,t_sr%coord(5)+h:t_sr%coord(6)-1) = sigh2(2)
      t_m%sighz(i,j,t_sr%coord(5)+h:t_sr%coord(6)-1) = sigh2(3)
    end do;end do
  else
    exit
  end if
end do
ideb = l
end subroutine milieu_sigh_surf_rugueuse

! ==========================================================
! == Subroutine milieu_mu_surf_rugueuse              ==
! == Remplissage du tableau mu à partir d'une surface ==
! == rugueuse. Le moyennage doit impérativement etre appliqué
! == par la suite.
! ==========================================================	
subroutine milieu_mu_surf_rugueuse(imilieu,ideb,t_m,t_lmnd,t_s)

!--- DÉCLARATION DES ARGUMENTS ---
integer, intent(in)  :: imilieu
integer, intent(inout) :: ideb
type(st_milieu_volume), intent(inout) :: t_m
type(st_liste_milieu_simple), intent(in) :: t_lmnd
type(st_structure), intent(in)           :: t_s

!--- DÉCLARATION DES VARIABLES LOCALES
integer   :: i,j,k,l,h
logical   :: flagnommilieu1, flagnommilieu2
real,dimension(1:3)  :: mur1,mur2
type(st_surf_rugueuse), pointer  :: t_sr

!--- DEBUT DES TRAITEMENTS ---
do l =ideb,t_s%nbsurf_rugueuse
  t_sr => t_s%t_surf_rugueuse(l)
  if (imilieu-1 ==t_sr%ind_milieu_avant) then
    !--- On récupère la valeur des deux milieux ---
    flagnommilieu1 = cherche_mu_milieu(t_lmnd,t_sr%nommilieu1,mur1)
    flagnommilieu2 = cherche_mu_milieu(t_lmnd,t_sr%nommilieu2,mur2)
    if ((.not.flagnommilieu1).or.(.not.flagnommilieu2)) then 
      print *,'Nom de milieu (mu) non reconnu dans le bloc structure :'
      stop 'Chomage technique ...'
    end if
    
    !---  SURFACE RUGUEUSE PLAN XOY OBLIGATOIRE---
    do j = t_sr%coord(3),t_sr%coord(4)-1
    do i = t_sr%coord(1),t_sr%coord(2)-1
      h = t_sr%hauteur(i-t_sr%coord(1)+1,j-t_sr%coord(3)+1)
!      h = t_sr%hauteur(i,j)
      ! milieu inférieur
      t_m%mux(i,j,t_sr%coord(5):t_sr%coord(5)+h-1) = mur1(1)
      t_m%muy(i,j,t_sr%coord(5):t_sr%coord(5)+h-1) = mur1(2)
      t_m%muz(i,j,t_sr%coord(5):t_sr%coord(5)+h-1) = mur1(3)
      
      ! milieu supérieur
      t_m%mux(i,j,t_sr%coord(5)+h:t_sr%coord(6)-1) = mur2(1)
      t_m%muy(i,j,t_sr%coord(5)+h:t_sr%coord(6)-1) = mur2(2)
      t_m%muz(i,j,t_sr%coord(5)+h:t_sr%coord(6)-1) = mur2(3)
    end do;end do
  else
    exit
  end if
end do
ideb = l
end subroutine milieu_mu_surf_rugueuse

! ==========================================================
! == Subroutine milieu_calcul_param_em                    ==
! == Calcul des permittivité, perméabilité, conductivité  ==
! == dans chaque cellule élémentaire                      ==
! ==========================================================	
subroutine milieu_calcul_param_em(t_m,t_lmnd,v,t_s)

type(st_milieu_volume), intent(inout) :: t_m
type(st_liste_milieu_simple), intent(inout) :: t_lmnd
type(st_volume), intent(in)           :: v
type(st_structure), intent(in)        :: t_s

integer   :: i,j,k,l,offset,nu
integer   :: ideb_surf_rugueuse
integer   :: nbx,nby,nbz,Nx,Ny,Nz
integer, dimension(6) :: b
real,dimension(1:3)  :: epsr,mur,sigmae,sigmah
type(st_milieu_simple), pointer :: t_mnd
logical   :: flagnommilieu
character(LEN=LEN_MOT_CLE) :: nom
real, dimension(:,:,:), allocatable  :: tampon,tampon1
type(st_milieu_fractal), pointer :: mf

! nombre de cellule total des tableaux allouées ci-dessous 
nbx = v%ivolf - v%ivold +3
nby = v%jvolf - v%jvold +3
nbz = v%kvolf - v%kvold +3

!******************************************************************************
!* N° d'identification pour le milieu de chaque cellule
!******************************************************************************
allocate(t_m%milieu(v%ivold-1:v%ivolf+2,v%jvold-1:v%jvolf+2,v%kvold-1:v%kvolf+2)); 
t_m%milieu = 0
do l = 1,t_s%nbmilieu
 !print '("milieu_calcul_param_em 1")'
  flagnommilieu = cherche_nu_milieu(t_lmnd,t_s%nommilieu(l),nu)
  if (flagnommilieu) then
    do k = t_s%t_milieu(5,l),t_s%t_milieu(6,l)-1
    do j = t_s%t_milieu(3,l),t_s%t_milieu(4,l)-1
    do i = t_s%t_milieu(1,l),t_s%t_milieu(2,l)-1
      if (t_lmnd%nb_ms<t_m%milieu(i,j,k)) then
        nom = liste_milieu_nom(t_lmnd,int(t_m%milieu(i,j,k)))
        print '("ERREUR dans le module ''milieu_volume'', une cellule contenant un matériau")'
        print '("dispersif ou fractal est recouvert par un second type de matériau !!!")'
	print '("N°de la structure =",I8, " ; nom du nouveau milieu=",A," ; nom ancien milieu=",A)',&
	                   l,trim(t_s%nommilieu(l)),nom
	print '("Coordonnées de la cellule (repère de calcul) =",3I6)',i,j,k
	stop
      end if
      t_m%milieu(i,j,k) = nu
    end do;end do;end do  
  else
    print *,'ERREUR, Nom de milieu non reconnu dans le bloc structure :',t_s%nommilieu(l)
    stop 'A mon commandement, REPOS !!!'  
  end if
end do
!*********************************************************
!* Calcul des permittivités dans chaque cellule du volume*
!*********************************************************

offset = -1 ! retrait de fin de boucle
if (v%flag_nonmoy_eps) offset = 0

if (t_m%flag_eps) then

  ! la taille d'allocation est la meme que les tableaux de champ
  allocate(t_m%epsx(v%ivold-1:v%ivolf+2,v%jvold-1:v%jvolf+2,v%kvold-1:v%kvolf+2)); 
  allocate(t_m%epsy(v%ivold-1:v%ivolf+2,v%jvold-1:v%jvolf+2,v%kvold-1:v%kvolf+2)); 
  allocate(t_m%epsz(v%ivold-1:v%ivolf+2,v%jvold-1:v%jvolf+2,v%kvold-1:v%kvolf+2)); 
  allocate(t_m%tampon(v%ivold-1:v%ivolf+2,v%jvold-1:v%jvolf+2,v%kvold-1:v%kvolf+2)); 
  t_m%epsx = v%epsilon_r ; t_m%epsy = v%epsilon_r ; t_m%epsz = v%epsilon_r
  t_m%tampon = 0.0
  
  ! boucle sur toute les structures puis affectation du milieu
  ideb_surf_rugueuse = 1 ! début de la boucle sur les surfaces rugueuses
  do l = 1,t_s%nbmilieu
    !--------------------- Surface rugueuse ------------------------------------
    if (t_s%nbsurf_rugueuse>=1) then 
      call milieu_epsilon_surf_rugueuse(l,ideb_surf_rugueuse,t_m,t_lmnd,t_s)
    end if
    !---------------------------------------------------------------------------
  
    epsr = 0.0
    flagnommilieu = cherche_epsilon_milieu(t_lmnd,t_s%nommilieu(l),epsr)
    if (.not.flagnommilieu) then 
      !----------- on regarde si c'est un milieu fractal ----------------------!
      if (identifie_milieu_fractal(t_lmnd,t_s%nommilieu(l))) then
        mf => pointe_milieu_fractal(t_lmnd,t_s%nommilieu(l))
	if (mf%flag_eps) then
	  b =  t_s%t_milieu(1:6,l);
	  Nx = b(2)-b(1)+1
	  Ny = b(4)-b(3)+1
	  Nz = b(6)-b(5)+1
	  allocate(tampon1(Nx, Ny, Nz))
	  tampon1 = 0.0
	  call milieu_fractal_precalcul(Nx, Ny, Nz, mf%eps_fractal,tampon1)
	  
	  t_m%epsx(b(1):b(2)-1,b(3):b(4)+OFFSET,b(5):b(6)+OFFSET) =  &
                      tampon1(1:Nx-1,1:Ny+OFFSET,1:Nz+OFFSET)	  
	  t_m%epsy(b(1):b(2)+OFFSET,b(3):b(4)-1,b(5):b(6)+OFFSET) =  &
                      tampon1(1:Nx+OFFSET,1:Ny-1,1:Nz+OFFSET)	  
	  t_m%epsz(b(1):b(2)+OFFSET,b(3):b(4)+OFFSET,b(5):b(6)-1) =  &
                      tampon1(1:Nx+OFFSET,1:Ny+OFFSET,1:Nz-1)	  
!	  print '("epsxx =",80F5.2)',t_m%epsx(:,20,20)
	  deallocate(tampon1)
	  cycle
	else
	  epsr = mf%eps_moyen
	end if
      !------------------------------------------------------------------------!
      else
        print *,'Nom de milieu non reconnu dans le bloc structure :',t_s%nommilieu(l)
        stop 'Chomage technique ...'
      end if
    end if  
    
    ! affectation du milieu à chaque cellule du volume
    do k = t_s%t_milieu(5,l),t_s%t_milieu(6,l)+offset
    do j = t_s%t_milieu(3,l),t_s%t_milieu(4,l)+offset
    do i = t_s%t_milieu(1,l),t_s%t_milieu(2,l)-1
      t_m%epsx(i,j,k) = epsr(1)
    end do;end do;end do  
    
    do k = t_s%t_milieu(5,l),t_s%t_milieu(6,l)+offset
    do j = t_s%t_milieu(3,l),t_s%t_milieu(4,l)-1
    do i = t_s%t_milieu(1,l),t_s%t_milieu(2,l)+offset
      t_m%epsy(i,j,k) = epsr(2)
    end do;end do;end do  
    
    do k = t_s%t_milieu(5,l),t_s%t_milieu(6,l)-1
    do j = t_s%t_milieu(3,l),t_s%t_milieu(4,l)+offset
    do i = t_s%t_milieu(1,l),t_s%t_milieu(2,l)+offset
      t_m%epsz(i,j,k) = epsr(3)
    end do;end do;end do  
  end do 
  !--------------------- Surface rugueuse ------------------------------------
  if (t_s%nbsurf_rugueuse>=1) then 
    call milieu_epsilon_surf_rugueuse(l,ideb_surf_rugueuse,t_m,t_lmnd,t_s)
  end if
    !---------------------------------------------------------------------------
 
  ! dupplication sur les frontières supérieures 
  call milieu_prolonge_frontiere_sup(t_m%epsx)
  call milieu_prolonge_frontiere_sup(t_m%epsy)
  call milieu_prolonge_frontiere_sup(t_m%epsz)


  !-------------- TRAITEMENT DES PLAQUES ET MOYENNAGE ------------------------
  if (t_s%nbplaque>0) then
    do k=v%kvold-1,v%kvolf+1  
      t_m%tampon(:,:,k) = t_m%epsx(:,:,k) ! évite l
    end DO
  end if
  if (.not.v%flag_nonmoy_eps) call milieu_calcul_moy_electrique(t_m%epsx,nbx,nby,nbz,0,1,1,2,t_m%surfyz)
  if (t_s%nbplaque>0) call milieu_EpsxSigex_PlaqueMince(t_m,t_s,v%dxyz,t_lmnd,0,v%borne_volume)

  if (t_s%nbplaque>0) then
    do k=v%kvold-1,v%kvolf+1  
      t_m%tampon(:,:,k) = t_m%epsy(:,:,k) ! évite l
    end DO
  end if
  if (.not.v%flag_nonmoy_eps) call milieu_calcul_moy_electrique(t_m%epsy,nbx,nby,nbz,1,0,1,3,t_m%surfxz)
  if (t_s%nbplaque>0) call milieu_EpsySigey_PlaqueMince(t_m,t_s,v%dxyz,t_lmnd,0,v%borne_volume)
  
  if (t_s%nbplaque>0) then
    do k=v%kvold-1,v%kvolf+1  
      t_m%tampon(:,:,k) = t_m%epsz(:,:,k) ! évite l
    end DO
  end if
  if (.not.v%flag_nonmoy_eps) call milieu_calcul_moy_electrique(t_m%epsz,nbx,nby,nbz,1,1,0,1,t_m%surfxy)    
  if (t_s%nbplaque>0) call milieu_EpszSigez_PlaqueMince(t_m,t_s,v%dxyz,t_lmnd,0,v%borne_volume)
  
  !----------------------------------------------------------------------------
  ! rectification pour les plaques et les fils, pas de moyennage
  do l = 1,t_s%nbmilieu
    flagnommilieu = cherche_epsilon_milieu(t_lmnd,t_s%nommilieu(l),epsr)
    ! plaque ou fil normale à oz 
    if (t_s%t_milieu(5,l) == t_s%t_milieu(6,l)) then 
      do j = t_s%t_milieu(3,l),t_s%t_milieu(4,l)
      do i = t_s%t_milieu(1,l),t_s%t_milieu(2,l)-1
        t_m%epsx(i,j,t_s%t_milieu(5,l)) = epsr(1)
      end do;end do  
      do j = t_s%t_milieu(3,l),t_s%t_milieu(4,l)-1
      do i = t_s%t_milieu(1,l),t_s%t_milieu(2,l)
        t_m%epsy(i,j,t_s%t_milieu(5,l)) = epsr(2)
      end do;end do  
    end if
    
    ! plaque ou fil normale à oy 
    if (t_s%t_milieu(3,l) == t_s%t_milieu(4,l)) then 
      do k = t_s%t_milieu(5,l),t_s%t_milieu(6,l)
      do i = t_s%t_milieu(1,l),t_s%t_milieu(2,l)-1
        t_m%epsx(i,t_s%t_milieu(3,l),k) = epsr(1)
      end do;end do  
      do k = t_s%t_milieu(5,l),t_s%t_milieu(6,l)-1
      do i = t_s%t_milieu(1,l),t_s%t_milieu(2,l)
        t_m%epsz(i,t_s%t_milieu(3,l),k) = epsr(3)
      end do;end do  
    end if

    ! plaque ou fil normale à ox
    if (t_s%t_milieu(1,l) == t_s%t_milieu(2,l)) then 
      do k = t_s%t_milieu(5,l),t_s%t_milieu(6,l)
      do j = t_s%t_milieu(3,l),t_s%t_milieu(4,l)-1
        t_m%epsy(t_s%t_milieu(1,l),j,k) = epsr(2)
      end do;end do  
      do k = t_s%t_milieu(5,l),t_s%t_milieu(6,l)-1
      do j = t_s%t_milieu(3,l),t_s%t_milieu(4,l)
        t_m%epsz(t_s%t_milieu(1,l),j,k) = epsr(3)
      end do;end do  
    end if
  end do  


  !----------------------------------------------------------------------------

  !---Modification temporaire : milieu sub-cellulaire suivant z ---
  t_m%tampon = 0.0
  do l = 1,t_s%nbmilieu
    if (ABS(t_s%decal(ZMIN,l)) > 0.01) then
      flagnommilieu = cherche_epsilon_milieu(t_lmnd,t_s%nommilieu(l),epsr)
      k = t_s%t_milieu(5,l)
      do j = t_s%t_milieu(3,l),t_s%t_milieu(4,l)
      do i = t_s%t_milieu(1,l),t_s%t_milieu(2,l)-1
	t_m%epsx(i,j,k) = t_m%epsx(i,j,k) - epsr(1) * t_s%decal(ZMIN,l) + t_s%decal(ZMIN,l)
      end do;end do
      do j = t_s%t_milieu(3,l),t_s%t_milieu(4,l)-1
      do i = t_s%t_milieu(1,l),t_s%t_milieu(2,l)
	t_m%epsy(i,j,k) = t_m%epsy(i,j,k) - epsr(2) * t_s%decal(ZMIN,l) + t_s%decal(ZMIN,l)
      end do;end do
      
      ! Traitement de epsilon normale à l'interface
      if (t_s%decal(ZMIN,l) < -0.01) then
	do j = t_s%t_milieu(3,l),t_s%t_milieu(4,l)
	do i = t_s%t_milieu(1,l),t_s%t_milieu(2,l)
	  t_m%tampon(i,j,k-1) = (t_m%epsz(i,j,k) * t_m%epsz(i,j,k-1)) / &
	                    (t_m%epsz(i,j,k-1) * ABS(t_s%decal(ZMIN,l)) + &
			     t_m%epsz(i,j,k) * (1-ABS(t_s%decal(ZMIN,l))))
	end do;end do        
      end if
      if (t_s%decal(ZMIN,l) > 0.01) then
	do j = t_s%t_milieu(3,l),t_s%t_milieu(4,l)
	do i = t_s%t_milieu(1,l),t_s%t_milieu(2,l)
	  t_m%tampon(i,j,k) = (t_m%epsz(i,j,k) * t_m%epsz(i,j,k-1)) / &
	                    (t_m%epsz(i,j,k) * t_s%decal(ZMIN,l) + &
			     t_m%epsz(i,j,k-1) * (1-t_s%decal(ZMIN,l)))
	end do;end do        
      end if
    end if
    
    if (ABS(t_s%decal(ZMAX,l)) > 0.01) then
      flagnommilieu = cherche_epsilon_milieu(t_lmnd,t_s%nommilieu(l),epsr)
      k = t_s%t_milieu(6,l)
      do j = t_s%t_milieu(3,l),t_s%t_milieu(4,l)
      do i = t_s%t_milieu(1,l),t_s%t_milieu(2,l)-1
	t_m%epsx(i,j,k) = t_m%epsx(i,j,k) + epsr(1) * t_s%decal(ZMAX,l) - t_s%decal(ZMAX,l)
      end do;end do
      do j = t_s%t_milieu(3,l),t_s%t_milieu(4,l)-1
      do i = t_s%t_milieu(1,l),t_s%t_milieu(2,l)
	t_m%epsy(i,j,k) = t_m%epsy(i,j,k) + epsr(2) * t_s%decal(ZMAX,l) - t_s%decal(ZMAX,l)
      end do;end do
      ! Traitement de epsilon normale à l'interface
      if (t_s%decal(ZMAX,l) < -0.01) then
	do j = t_s%t_milieu(3,l),t_s%t_milieu(4,l)
	do i = t_s%t_milieu(1,l),t_s%t_milieu(2,l)
	  t_m%tampon(i,j,k-1) = (t_m%epsz(i,j,k) * t_m%epsz(i,j,k-1)) / &
	                    (t_m%epsz(i,j,k-1) * ABS(t_s%decal(ZMAX,l)) + &
			     t_m%epsz(i,j,k) * (1-ABS(t_s%decal(ZMAX,l))))
	end do;end do        
      end if
      if (t_s%decal(ZMAX,l) > 0.01) then
	do j = t_s%t_milieu(3,l),t_s%t_milieu(4,l)
	do i = t_s%t_milieu(1,l),t_s%t_milieu(2,l)
	  t_m%tampon(i,j,k) = (t_m%epsz(i,j,k) * t_m%epsz(i,j,k-1)) / &
	                    (t_m%epsz(i,j,k) * t_s%decal(ZMAX,l) + &
			     t_m%epsz(i,j,k-1) * (1-t_s%decal(ZMAX,l)))
	end do;end do  
      end if     
    end if

  end do
  
  do k=v%kvold-1,v%kvolf+1
  do j=v%jvold-1,v%jvolf+1
  do i=v%ivold-1,v%ivolf+1
    if (t_m%tampon(i,j,k) > 0.01) t_m%epsz(i,j,k) = t_m%tampon(i,j,k)
  end do; end do; end do
  
  deallocate(t_m%tampon)
end if

!******************************************************************************
!*         Calcul des perméabilités dans chaque cellule du volume             *
!******************************************************************************
offset = -1 ! retrait de fin de boucle
if (v%flag_nonmoy_mu) offset = 0

if (t_m%flag_mu) then
  ! la taille d'allocation est la meme que les tableaux de champ
  allocate(t_m%mux(v%ivold-1:v%ivolf+2,v%jvold-1:v%jvolf+2,v%kvold-1:v%kvolf+2)); 
  allocate(t_m%muy(v%ivold-1:v%ivolf+2,v%jvold-1:v%jvolf+2,v%kvold-1:v%kvolf+2)); 
  allocate(t_m%muz(v%ivold-1:v%ivolf+2,v%jvold-1:v%jvolf+2,v%kvold-1:v%kvolf+2)); 
  t_m%mux = v%mu_r ; t_m%muy = v%mu_r ; t_m%muz = v%mu_r

  ! boucle sur toute les structures puis affectation du milieu
  ideb_surf_rugueuse = 1 ! début de la boucle sur les surfaces rugueuses
  do l = 1,t_s%nbmilieu
    !--------------------- Surface rugueuse ------------------------------------
    if (t_s%nbsurf_rugueuse>=1) then 
      call milieu_mu_surf_rugueuse(l,ideb_surf_rugueuse,t_m,t_lmnd,t_s)
    end if
    !---------------------------------------------------------------------------
    
    flagnommilieu = cherche_mu_milieu(t_lmnd,t_s%nommilieu(l),mur)
    if (.not.flagnommilieu) then 
      !----------- on regarde si c'est un milieu fractal ----------------------!
      if (identifie_milieu_fractal(t_lmnd,t_s%nommilieu(l))) then
        mf => pointe_milieu_fractal(t_lmnd,t_s%nommilieu(l))
	mur = mf%mu_moyen
      !------------------------------------------------------------------------!
      else
        print *,'Nom de milieu (mu) non reconnu dans le bloc structure :',t_s%nommilieu(l)
        stop 'Pas question de continuer ...'
      end if  
    end if
    ! affectation du milieu à chaque cellule du volume
    do k = t_s%t_milieu(5,l),t_s%t_milieu(6,l)-1
    do j = t_s%t_milieu(3,l),t_s%t_milieu(4,l)-1
    do i = t_s%t_milieu(1,l),t_s%t_milieu(2,l)+offset
      t_m%mux(i,j,k) = mur(1)
    end do;end do;end do
    
    do k = t_s%t_milieu(5,l),t_s%t_milieu(6,l)-1
    do j = t_s%t_milieu(3,l),t_s%t_milieu(4,l)+offset
    do i = t_s%t_milieu(1,l),t_s%t_milieu(2,l)-1
      t_m%muy(i,j,k) = mur(2)
    end do;end do;end do
    
    do k = t_s%t_milieu(5,l),t_s%t_milieu(6,l)+offset
    do j = t_s%t_milieu(3,l),t_s%t_milieu(4,l)-1
    do i = t_s%t_milieu(1,l),t_s%t_milieu(2,l)-1
      t_m%muz(i,j,k) = mur(3)
    end do;end do;end do
  end do  
    !--------------------- Surface rugueuse ------------------------------------
  if (t_s%nbsurf_rugueuse>=1) then 
    call milieu_mu_surf_rugueuse(l,ideb_surf_rugueuse,t_m,t_lmnd,t_s)
  end if
  !---------------------------------------------------------------------------

  call milieu_prolonge_frontiere_sup(t_m%mux)
  call milieu_prolonge_frontiere_sup(t_m%muy)
  call milieu_prolonge_frontiere_sup(t_m%muz)

  ! moyennage si autorisé
  if (.not.v%flag_nonmoy_mu) then
    !call milieu_calcul_moy_magnetique(t_m%mux,nbx,nby,nbz,1,0,0,1,t_m%axex)
    !call milieu_calcul_moy_magnetique(t_m%muy,nbx,nby,nbz,0,1,0,2,t_m%axey)
    !call milieu_calcul_moy_magnetique(t_m%muz,nbx,nby,nbz,0,0,1,3,t_m%axez)
    call milieu_calcul_moy_simple_mu(t_m%mux,nbx,nby,nbz,1,0,0,1,t_m%axex)
    call milieu_calcul_moy_simple_mu(t_m%muy,nbx,nby,nbz,0,1,0,2,t_m%axey)
    call milieu_calcul_moy_simple_mu(t_m%muz,nbx,nby,nbz,0,0,1,3,t_m%axez)
  end if

end if

!*********************************************************
!* Calcul des conductivités électriques dans chaque      *
!* cellule du volume                                     * 
!*********************************************************
offset = -1 ! retrait de fin de boucle
if (v%flag_nonmoy_sigmae) offset = 0

if (t_m%flag_eps_sigmae) then

  ! la taille d'allocation est la meme que les tableaux de champ
  allocate(t_m%sigex (v%ivold-1:v%ivolf+2,v%jvold-1:v%jvolf+2,v%kvold-1:v%kvolf+2)); 
  allocate(t_m%sigey (v%ivold-1:v%ivolf+2,v%jvold-1:v%jvolf+2,v%kvold-1:v%kvolf+2)); 
  allocate(t_m%sigez (v%ivold-1:v%ivolf+2,v%jvold-1:v%jvolf+2,v%kvold-1:v%kvolf+2)); 
  allocate(t_m%tampon(v%ivold-1:v%ivolf+2,v%jvold-1:v%jvolf+2,v%kvold-1:v%kvolf+2)); 
  t_m%sigex = v%sigma_e ; t_m%sigey = v%sigma_e ; t_m%sigez = v%sigma_e

  ! boucle sur toute les structures puis affectation du milieu
  ideb_surf_rugueuse = 1 ! début de la boucle sur les surfaces rugueuses
  do l = 1,t_s%nbmilieu
    !--------------------- Surface rugueuse ------------------------------------
    if (t_s%nbsurf_rugueuse>=1) then 
      call milieu_sige_surf_rugueuse(l,ideb_surf_rugueuse,t_m,t_lmnd,t_s)
    end if
    !---------------------------------------------------------------------------
  
    flagnommilieu = cherche_sigmae_milieu(t_lmnd,t_s%nommilieu(l),sigmae)
    if (.not.flagnommilieu) then 
      !----------- on regarde si c'est un milieu fractal ----------------------!
      if (identifie_milieu_fractal(t_lmnd,t_s%nommilieu(l))) then
        mf => pointe_milieu_fractal(t_lmnd,t_s%nommilieu(l))
	if (mf%flag_sige) then
	  b =  t_s%t_milieu(1:6,l);
	  Nx = b(2)-b(1)+1
	  Ny = b(4)-b(3)+1
	  Nz = b(6)-b(5)+1
	  allocate(tampon1(Nx, Ny, Nz))
	  call milieu_fractal_precalcul(Nx, Ny, Nz, mf%sige_fractal,tampon1)

	  t_m%sigex(b(1):b(2)-1,b(3):b(4)+OFFSET,b(5):b(6)+OFFSET) =  &
                      tampon1(1:Nx-1,1:Ny+OFFSET,1:Nz+OFFSET)	  
	  t_m%sigey(b(1):b(2)+OFFSET,b(3):b(4)-1,b(5):b(6)+OFFSET) =  &
                      tampon1(1:Nx+OFFSET,1:Ny-1,1:Nz+OFFSET)	  
	  t_m%sigez(b(1):b(2)+OFFSET,b(3):b(4)+OFFSET,b(5):b(6)-1) =  &
                      tampon1(1:Nx+OFFSET,1:Ny+OFFSET,1:Nz-1)	  
	  deallocate(tampon1)
	  cycle
	else
	  sigmae = mf%sige_moyen
	end if
      !------------------------------------------------------------------------!
      else
        print *,'Nom de milieu (sigma E) non reconnu dans le bloc structure :',t_s%nommilieu(l)
        stop 'Scrachhhhh ...'
      end if
    end if  
    
    ! affectation du milieu à chaque cellule du volume
    do k = t_s%t_milieu(5,l),t_s%t_milieu(6,l)+offset
    do j = t_s%t_milieu(3,l),t_s%t_milieu(4,l)+offset
    do i = t_s%t_milieu(1,l),t_s%t_milieu(2,l)-1
      t_m%sigex(i,j,k) = sigmae(1)
    end do;end do;end do

    do k = t_s%t_milieu(5,l),t_s%t_milieu(6,l)+offset
    do j = t_s%t_milieu(3,l),t_s%t_milieu(4,l)-1
    do i = t_s%t_milieu(1,l),t_s%t_milieu(2,l)+offset
      t_m%sigey(i,j,k) = sigmae(2)
    end do;end do;end do
    
    do k = t_s%t_milieu(5,l),t_s%t_milieu(6,l)-1
    do j = t_s%t_milieu(3,l),t_s%t_milieu(4,l)+offset
    do i = t_s%t_milieu(1,l),t_s%t_milieu(2,l)+offset
      t_m%sigez(i,j,k) = sigmae(3)
    end do;end do;end do

  end do  
  !--------------------- Surface rugueuse ------------------------------------
  if (t_s%nbsurf_rugueuse>=1) then 
    call milieu_sige_surf_rugueuse(l,ideb_surf_rugueuse,t_m,t_lmnd,t_s)
  end if
  !---------------------------------------------------------------------------
  
  ! dupplication sur les frontières supérieures 
  call milieu_prolonge_frontiere_sup(t_m%sigex)
  call milieu_prolonge_frontiere_sup(t_m%sigey)
  call milieu_prolonge_frontiere_sup(t_m%sigez)

  !-------------- TRAITEMENT DES PLAQUES ET MOYENNAGE ------------------------  
  if (t_s%nbplaque>0) then
    do k=v%kvold-1,v%kvolf+1  
      t_m%tampon(:,:,k) = t_m%sigex(:,:,k) ! évite l
    end DO
  end if 
  if (.not.v%flag_nonmoy_sigmae) CALL milieu_calcul_moy_electrique(t_m%sigex,nbx,nby,nbz,0,1,1,2,t_m%surfyz)
  if (t_s%nbplaque>0) call milieu_EpsxSigex_PlaqueMince(t_m,t_s,v%dxyz,t_lmnd,1,v%borne_volume)

  if (t_s%nbplaque>0) then
    do k=v%kvold-1,v%kvolf+1  
      t_m%tampon(:,:,k) = t_m%sigey(:,:,k) ! évite l
    end DO
  end if
  if (.not.v%flag_nonmoy_sigmae)   call milieu_calcul_moy_electrique(t_m%sigey,nbx,nby,nbz,1,0,1,3,t_m%surfxz)
  if (t_s%nbplaque>0) call milieu_EpsySigey_PlaqueMince(t_m,t_s,v%dxyz,t_lmnd,1,v%borne_volume)
  
  if (t_s%nbplaque>0) then
    do k=v%kvold-1,v%kvolf+1  
      t_m%tampon(:,:,k) = t_m%sigez(:,:,k) ! évite l
    end DO
  end if
  if (.not.v%flag_nonmoy_sigmae)   call milieu_calcul_moy_electrique(t_m%sigez,nbx,nby,nbz,1,1,0,1,t_m%surfxy)    
  if (t_s%nbplaque>0) call milieu_EpszSigez_PlaqueMince(t_m,t_s,v%dxyz,t_lmnd,1,v%borne_volume)
  deallocate(t_m%tampon); 

  !----------------------------------------------------------------------------
  ! rectification pour les plaques et les fils, pas de moyennage
  do l = 1,t_s%nbmilieu
    flagnommilieu = cherche_sigmae_milieu(t_lmnd,t_s%nommilieu(l),sigmae)
    ! plaque ou fil normale à oz 
    if (t_s%t_milieu(5,l) == t_s%t_milieu(6,l)) then 
      do j = t_s%t_milieu(3,l),t_s%t_milieu(4,l)
      do i = t_s%t_milieu(1,l),t_s%t_milieu(2,l)-1
        t_m%sigex(i,j,t_s%t_milieu(5,l)) = sigmae(1)
      end do;end do  
      do j = t_s%t_milieu(3,l),t_s%t_milieu(4,l)-1
      do i = t_s%t_milieu(1,l),t_s%t_milieu(2,l)
        t_m%sigey(i,j,t_s%t_milieu(5,l)) = sigmae(2)
      end do;end do  
    end if
    
    ! plaque ou fil normale à oy 
    if (t_s%t_milieu(3,l) == t_s%t_milieu(4,l)) then 
      do k = t_s%t_milieu(5,l),t_s%t_milieu(6,l)
      do i = t_s%t_milieu(1,l),t_s%t_milieu(2,l)-1
        t_m%sigex(i,t_s%t_milieu(3,l),k) = sigmae(1)
      end do;end do  
      do k = t_s%t_milieu(5,l),t_s%t_milieu(6,l)-1
      do i = t_s%t_milieu(1,l),t_s%t_milieu(2,l)
        t_m%sigez(i,t_s%t_milieu(3,l),k) = sigmae(3)
      end do;end do  
    end if

    ! plaque ou fil normale à ox
    if (t_s%t_milieu(1,l) == t_s%t_milieu(2,l)) then 
      do k = t_s%t_milieu(5,l),t_s%t_milieu(6,l)
      do j = t_s%t_milieu(3,l),t_s%t_milieu(4,l)-1
        t_m%sigey(t_s%t_milieu(1,l),j,k) = sigmae(2)
      end do;end do  
      do k = t_s%t_milieu(5,l),t_s%t_milieu(6,l)-1
      do j = t_s%t_milieu(3,l),t_s%t_milieu(4,l)
        t_m%sigez(t_s%t_milieu(1,l),j,k) = sigmae(3)
      end do;end do  
    end if
  end do  

end if

!*********************************************************
!* Calcul des conductivités magnétiques dans chaque      *
!* cellule du volume                                     * 
!*********************************************************
offset = -1 ! retrait de fin de boucle
if (v%flag_nonmoy_sigmah) offset = 0

if (t_m%flag_mu_sigmah) then

  ! la taille d'allocation est la meme que les tableaux de champ
  allocate(t_m%sighx(v%ivold-1:v%ivolf+2,v%jvold-1:v%jvolf+2,v%kvold-1:v%kvolf+2)); 
  allocate(t_m%sighy(v%ivold-1:v%ivolf+2,v%jvold-1:v%jvolf+2,v%kvold-1:v%kvolf+2)); 
  allocate(t_m%sighz(v%ivold-1:v%ivolf+2,v%jvold-1:v%jvolf+2,v%kvold-1:v%kvolf+2)); 
  t_m%sighx = v%sigma_h ; t_m%sighy = v%sigma_h ; t_m%sighz = v%sigma_h

  ! boucle sur toute les structures puis affectation du milieu
  ideb_surf_rugueuse = 1 ! début de la boucle sur les surfaces rugueuses
  do l = 1,t_s%nbmilieu
    !--------------------- Surface rugueuse ------------------------------------
    if (t_s%nbsurf_rugueuse>=1) then 
      call milieu_sigh_surf_rugueuse(l,ideb_surf_rugueuse,t_m,t_lmnd,t_s)
    end if
    !---------------------------------------------------------------------------
  
    flagnommilieu = cherche_sigmah_milieu(t_lmnd,t_s%nommilieu(l),sigmah)
    if (.not.flagnommilieu) then 
      !----------- on regarde si c'est un milieu fractal ----------------------!
      if (identifie_milieu_fractal(t_lmnd,t_s%nommilieu(l))) then
        mf => pointe_milieu_fractal(t_lmnd,t_s%nommilieu(l))
	sigmah = mf%sigh_moyen
      !------------------------------------------------------------------------!
      else
        print *,'Nom de milieu (sigma H) non reconnu dans le bloc structure :',t_s%nommilieu(l)
        stop 'Scrachhhhh ...'
      end if
    end if  
    
    ! affectation du milieu à chaque cellule du volume
    do k = t_s%t_milieu(5,l),t_s%t_milieu(6,l)-1
    do j = t_s%t_milieu(3,l),t_s%t_milieu(4,l)-1
    do i = t_s%t_milieu(1,l),t_s%t_milieu(2,l)+offset
      t_m%sighx(i,j,k) = sigmah(1)
    end do;end do;end do

    do k = t_s%t_milieu(5,l),t_s%t_milieu(6,l)-1
    do j = t_s%t_milieu(3,l),t_s%t_milieu(4,l)+offset
    do i = t_s%t_milieu(1,l),t_s%t_milieu(2,l)-1
      t_m%sighy(i,j,k) = sigmah(2)
    end do;end do;end do

    do k = t_s%t_milieu(5,l),t_s%t_milieu(6,l)+offset
    do j = t_s%t_milieu(3,l),t_s%t_milieu(4,l)-1
    do i = t_s%t_milieu(1,l),t_s%t_milieu(2,l)-1
      t_m%sighz(i,j,k) = sigmah(3)
    end do;end do;end do
  end do 
  !--------------------- Surface rugueuse ------------------------------------
  if (t_s%nbsurf_rugueuse>=1) then 
    call milieu_sigh_surf_rugueuse(l,ideb_surf_rugueuse,t_m,t_lmnd,t_s)
  end if
  !---------------------------------------------------------------------------
  
  call milieu_prolonge_frontiere_sup(t_m%sighx)
  call milieu_prolonge_frontiere_sup(t_m%sighy)
  call milieu_prolonge_frontiere_sup(t_m%sighz)

  ! moyennage si autorisé
  if (.not.v%flag_nonmoy_sigmah) then
    !call milieu_calcul_moy_magnetique(t_m%sighx,nbx,nby,nbz,1,0,0,1,t_m%axex)
    !call milieu_calcul_moy_magnetique(t_m%sighy,nbx,nby,nbz,0,1,0,2,t_m%axey)
    !call milieu_calcul_moy_magnetique(t_m%sighz,nbx,nby,nbz,0,0,1,3,t_m%axez)
    call milieu_calcul_moy_simple_mu(t_m%sighx,nbx,nby,nbz,1,0,0,1,t_m%axex)
    call milieu_calcul_moy_simple_mu(t_m%sighy,nbx,nby,nbz,0,1,0,2,t_m%axey)
    call milieu_calcul_moy_simple_mu(t_m%sighz,nbx,nby,nbz,0,0,1,3,t_m%axez)
  end if
end if

end subroutine milieu_calcul_param_em

! ==========================================================
! == Subroutine milieu_EpsxSigex_PlaqueMince              ==
! == Modification de la permittivité et de la perméabilité  ==
! == pour les plaques minces                              ==
! ==========================================================	
subroutine milieu_EpsxSigex_PlaqueMince(tm,ts,dxyz,t_lmnd,typemilieu,bv)

implicit none

type(st_milieu_volume), intent(inout) :: tm
type(st_structure), intent(in)        :: ts
real,dimension(1:3), intent(in)       :: dxyz
type(st_liste_milieu_simple), intent(inout) :: t_lmnd
integer, intent(in)                   :: typemilieu ! (0-> eps; 1 -> sigma)
integer,dimension(6),intent(in)       :: bv

logical   :: flagnommilieu
integer   :: i,j,k,n,ipas
integer   :: num,mcell
integer, dimension(6)   :: c,c1
real, dimension(3)      :: epsr
real, dimension(4)      :: epscell, Szcell,Szpm
real                    :: epscentre,Szcentre,Scell,Scellmoins
real                    :: d1,d2 ! les deux pas transverses
type(st_structure_plaque_mince), pointer :: tp

integer, dimension(:,:,:), allocatable :: t3d
real, dimension(:,:,:), pointer  :: eps_sige
real   , dimension(:,:), allocatable :: eps_pm
real   , dimension(:,:), allocatable :: d

!=== DÉBUT DES TRAITEMENTS ===
print '(T5,"Rang=",I5," ;PLAQUE MINCE, traitement de epsilon ou sige X :",I2)',rang,typemilieu
! on évalue l'espace occupé par les plaques (ccordonnées min et max)
do n=1,ts%nbplaque
  tp => ts%t_plaque(n)
  if (n== 1)  then
    c = tp%coord
  else
    do i=1,5,2
      c(i)   = min(c(i)  ,tp%coord(i)  )
      c(i+1) = mAX(c(i+1),tp%coord(i+1))
    end do
  end if
  do i=1,5,2
      c1(i)  = MAX(c(i),bv(i))
      c1(i+1)= c(i+1)  
  end do
!print '(T15,"coord. plaque :",6I5)',tp%coord
end do
print '(T15,"Rang=",I5," ; Espace occupé par les plaques :",6I5," ; espace  de traitement=",6I5)',rang,c,c1

! Allocation du tableau qui contiendra le type de jonction
allocate(t3d(c(1):c(2),c(3):c(4),c(5):c(6)))
t3d = 0

!-------------------- TRAITEMENT POUR EPSILON OU SIGMA X ----------------------
! Recherche des cellules concernés par les modifications de epsilon x
do n=1,ts%nbplaque
  tp => ts%t_plaque(n)
  IF (tp%coord(2)==tp%coord(1)) cycle
  
    !--- plan xoz ---
  do k=tp%coord(5),tp%coord(6) 
  do i=tp%coord(1),tp%coord(2) -1
    t3d(i,tp%coord(3),k) = 1  
    t3d(i,tp%coord(4),k) = 1  
  end do; end do; 
    !--- plan xoy ---
  do j=tp%coord(3),tp%coord(4) 
  do i=tp%coord(1),tp%coord(2) -1
    t3d(i,j,tp%coord(5)) = 1  
    t3d(i,j,tp%coord(6)) = 1  
  end do; end do;   
end do  
mcell = sum(t3d);
!print '(T15,"t3d =  :",400I2)',t3d(c(1):c(2),c(3):c(4),c(5):c(6))
print '(T15,"Rang=",I5," ; Nombre de cellules concerné par les plaques :",I8)',rang,mcell
  
  
if (mcell>0) then !----- Modif à faire donc ---
  !---- numérotation seulement des cellules à modifier ----
  num = 0
  do k=c(5),c(6) 
  do j=c(3),c(4) 
  do i=c(1),c(2)
    num = num + t3d(i,j,k) ! incrémente si t3d(i,j,k) == 1
    t3d(i,j,k) = t3d(i,j,k)*num
  end do; end do; end do  
  
  !---- Allocation des tableaux ----
  allocate(eps_pm(1:8,1:mcell)) ; eps_pm = 0.0
  allocate(d(1:4,1:mcell)) ; d =0.0
  
 
  !--- Détermination des propriétés eps pour chaque cellule ----
  do n=1,ts%nbplaque
    tp => ts%t_plaque(n)
    IF (tp%coord(2)==tp%coord(1)) cycle
    if (typemilieu == 0) then ! c'est la permittivité
      flagnommilieu = cherche_epsilon_milieu(t_lmnd,tp%nommilieu,epsr)
      eps_sige =>tm%epsx
    else ! c'est la conductivité
      flagnommilieu = cherche_sigmae_milieu(t_lmnd,tp%nommilieu,epsr)  
      eps_sige =>tm%sigex
    end if
    !print '(T15," plaque n°",I5,"  Milieu =",ES12.3E2)',n,epsr(1)
  
    if (.not.flagnommilieu) then
      PRINT '("ERREUR dans la subroutine ''milieu_EpsxSigex_PlaqueMince'', nom milieu non reconnu !")' 
      PRINT '("    Nom milieu=",A," ; n° de la plaque=",I6," ; Coordonnées = ",6I6)',trim(tp%nommilieu),n,tp%coord 
      
      stop
    end if
  
      !--- plan xoz ---
    ipas = tp%coord(4)-tp%coord(3); if (ipas==0) ipas=1
    do j=tp%coord(3),tp%coord(4),ipas
    do k=tp%coord(5),tp%coord(6) 
    do i=tp%coord(1),tp%coord(2) -1
	num = t3d(i,j,k)
!	print '("(i,j,k = ", 3I5," num = ",I5)',i,j,k,num
	if (k<tp%coord(6)) then
   	  d(4,num)  = tp%d
	  eps_pm(4,num)       = epsr(1) ! epsx
	end if
	if (k>tp%coord(5)) then
   	  d(3,num)  = tp%d
	  eps_pm(3,num)       = epsr(1) ! epsx     
	end if
    end do; end do; end do

    !--- plan xoy ---
    ipas = tp%coord(6)-tp%coord(5); if (ipas==0) ipas=1
    do k=tp%coord(5),tp%coord(6),ipas
    do j=tp%coord(3),tp%coord(4) 
    do i=tp%coord(1),tp%coord(2) -1
	num = t3d(i,j,k)
!	print '("(i,j,k = ", 3I5," num = ",I5)',i,j,k,num
	if (j<tp%coord(4))  then
	  d(2,num)  = tp%d
          eps_pm(2,num) = epsr(1) ! epsx
	end if
	if (j>tp%coord(3)) then
	  d(1,num)  = tp%d
	  eps_pm(1,num) = epsr(1) ! epsx     
	end if 
    end do; end do; end do 
  end do
  
  !--- Correction de epsilon X ----
  d1 = dxyz(2); d2 = dxyz(3)
  Scell = d1*d2
  print '(T15,"Rang=",I5," ; Surface cellule        = ",ES15.5E2)',rang,Scell
  do k=c1(5),c1(6)
  do j=c1(3),c1(4)
  do i=c1(1),c1(2)
      if (t3d(i,j,k)>0) then
	num = t3d(i,j,k)
	epscell(1) = tm%tampon(i,j-1,k)
	epscell(2) = tm%tampon(i,j,k)
	epscell(3) = tm%tampon(i,j-1,k-1)
	epscell(4) = tm%tampon(i,j,k-1)
	! Calcul des surfaces élémentaires
	Szcell(1)  = (d1-d(4,num))*(d2-d(1,num))*0.25
	Szcell(2)  = (d1-d(4,num))*(d2-d(2,num))*0.25
	Szcell(3)  = (d1-d(3,num))*(d2-d(1,num))*0.25
	Szcell(4)  = (d1-d(3,num))*(d2-d(2,num))*0.25
	
	Scellmoins = abs(d(3,num)-d(4,num))*abs(d(2,num)-d(1,num))*0.25
	
	if ((d(4,num)<=d(3,num)).and.(d(1,num)<=d(2,num))) then
	  Szcell(1) = Szcell(1) - Scellmoins
	else if ((d(4,num)<=d(3,num)).and.(d(2,num)<=d(1,num))) then
	  Szcell(2) = Szcell(2) - Scellmoins
	else if ((d(3,num)<=d(4,num)).and.(d(1,num)<=d(2,num))) then	
	  Szcell(3) = Szcell(3) - Scellmoins
	else
	  Szcell(4) = Szcell(4) - Scellmoins	
	end if
	
	Szpm(1)  = d(1,num) * (d1-maxval(d(3:4,num)))*0.5
	Szpm(2)  = d(2,num) * (d1-maxval(d(3:4,num)))*0.5
	Szpm(3)  = d(3,num) * (d2-maxval(d(1:2,num)))*0.5
	Szpm(4)  = d(4,num) * (d2-maxval(d(1:2,num)))*0.5
	Szcentre = maxval(d(1:2,num))*maxval(d(3:4,num)) 
	
	epscentre = 0
	n = count(eps_pm(1:4,num)>0)
	if (n>0) epscentre = SUM(eps_pm(1:4,num))/n
	eps_sige(i,j,k) = ( SUM(epscell*Szcell)+SUM(eps_pm(1:4,num)*Szpm)+ &
			   Szcentre*epscentre ) /Scell
! 	print '(T15,"(i,j,k = ", 3I5,") num = ",I5)',i,j,k,num
! 	print '(T20,"epscell = ", 4F6.2," eps_pm = ",4F6.2)',epscell(1:4),eps_pm(1:4,num)
! 	print '(T20,"Szcell = ", 4F9.5," Szpm = ",4F9.5,"Szcentre",F9.5)',Szcell(1:4),Szpm(1:4),Szcentre
!        print '(T15,"d =",2F7.3," ; epsx = ",F6.2,"  Surface cellule sommée = ",ES15.5E2)', &
!	                  d(1:2,num),eps_sige(i,j,k),(SUM(Szcell)+SUM(Szpm)+Szcentre)

      end if
  end do; end do; end do  
  deallocate(eps_pm,d)
end if
deallocate(t3d)

end subroutine milieu_EpsxSigex_PlaqueMince

! ==========================================================
! == Subroutine milieu_EpsySigey_PlaqueMince              ==
! == Modification de la permittivité et de la perméabilité  ==
! == pour les plaques minces                              ==
! ==========================================================	
subroutine milieu_EpsySigey_PlaqueMince(tm,ts,dxyz,t_lmnd,typemilieu,bv)

implicit none

type(st_milieu_volume), intent(inout) :: tm
type(st_structure), intent(in)        :: ts
real,dimension(1:3), intent(in)       :: dxyz
type(st_liste_milieu_simple), intent(inout) :: t_lmnd
integer, intent(in)                   :: typemilieu ! (0-> eps; 1 -> sigma)
integer,dimension(6),intent(in)       :: bv

logical   :: flagnommilieu
integer   :: i,j,k,n,ipas
integer   :: num,mcell
integer, dimension(6)   :: c,c1
real, dimension(3)      :: epsr
real, dimension(4)      :: epscell, Szcell,Szpm
real                    :: epscentre,Szcentre,Scell,Scellmoins
real                    :: d1,d2 ! les deux pas transverses
type(st_structure_plaque_mince), pointer :: tp

integer, dimension(:,:,:), allocatable :: t3d
real, dimension(:,:,:), pointer  :: eps_sige
real   , dimension(:,:), allocatable :: eps_pm
real   , dimension(:,:), allocatable :: d

!=== DÉBUT DES TRAITEMENTS ===
print '(T5,"Rang=",I5," ;PLAQUE MINCE, traitement de epsilon ou sige Y :",I2)',rang,typemilieu
! on évalue l'espace occupé par les plaques (ccordonnées min et max)
do n=1,ts%nbplaque
  tp => ts%t_plaque(n)
  if (n== 1)  then
    c = tp%coord
  else
    do i=1,5,2
      c(i)   = min(c(i)  ,tp%coord(i)  )
      c(i+1) = mAX(c(i+1),tp%coord(i+1))
    end do
  end if
end do
do i=1,5,2
    c1(i)  = MAX(c(i),bv(i))
    c1(i+1)= c(i+1)  
end do

print '(T15,"Rang=",I5," ; Espace occupé par les plaques :",6I5)',rang,c

! Allocation du tableau qui contiendra le type de jonction
allocate(t3d(c(1):c(2),c(3):c(4),c(5):c(6)))
t3d = 0

!-------------------- TRAITEMENT POUR EPSILON OU SIGMA Y ----------------------
! Recherche des cellules concernés par les modifications de epsilon y
do n=1,ts%nbplaque
  tp => ts%t_plaque(n)
  IF (tp%coord(4)==tp%coord(3)) cycle
  
    !--- plan yoz ---
  do k=tp%coord(5),tp%coord(6)
  do j=tp%coord(3),tp%coord(4)-1
    t3d(tp%coord(1),j,k) = 1  
    t3d(tp%coord(2),j,k) = 1  
  end do; end do; 
    !--- plan xoy ---
  do j=tp%coord(3),tp%coord(4) -1
  do i=tp%coord(1),tp%coord(2)
    t3d(i,j,tp%coord(5)) = 1  
    t3d(i,j,tp%coord(6)) = 1  
  end do; end do;   
end do  
mcell = sum(t3d);
print '(T15,"Rang=",I5," ; Nombre de cellules concerné par les plaques :",I8)',rang,mcell
  
  
if (mcell>0) then !----- Modif à faire donc ---
  !---- numérotation seulement des cellules à modifier ----
  num = 0
  do k=c(5),c(6) 
  do j=c(3),c(4) 
  do i=c(1),c(2)
    num = num + t3d(i,j,k) ! incrémente si t3d(i,j,k) == 1
    t3d(i,j,k) = t3d(i,j,k)*num
  end do; end do; end do  
  
  !---- Allocation des tableaux ----
  allocate(eps_pm(1:8,1:mcell)) ; eps_pm = 0.0
  allocate(d(1:4,1:mcell)) ; d =0.0
  
  !---- Initialisation de eps_pm -------
  ! le tableau peut aussi etre concerné par la conductivité sigma
  !   ca dépend du contenu du tableau tampon
!  do k=c(5),c(6)
!  do j=c(3),c(4)
!  do i=c(1),c(2)
!      if (t3d(i,j,k)>0) then
!	num = t3d(i,j,k)
!	eps_pm(1,num)  =  (tm%tampon(i-1,j,k-1) + tm%tampon(i,j,k-1) )*0.5
!	eps_pm(2,num)  =  (tm%tampon(i-1,j,k)   + tm%tampon(i,j,k)   )*0.5
!	eps_pm(3,num)  =  (tm%tampon(i-1,j,k-1) + tm%tampon(i-1,j,k) )*0.5
!	eps_pm(4,num)  =  (tm%tampon(i,j,k-1)   + tm%tampon(i,j,k)   )*0.5
 !     end if
!  end do; end do; end do  
  
  
  
  !--- Détermination des propriétés eps pour chaque cellule ----
  do n=1,ts%nbplaque
    tp => ts%t_plaque(n)
    IF (tp%coord(4)==tp%coord(3)) cycle
    if (typemilieu == 0) then ! c'est la permittivité
      flagnommilieu = cherche_epsilon_milieu(t_lmnd,tp%nommilieu,epsr)
      eps_sige =>tm%epsy
    else ! c'est la conductivité
      flagnommilieu = cherche_sigmae_milieu(t_lmnd,tp%nommilieu,epsr)  
      eps_sige =>tm%sigey
    end if
    !print '(T15," plaque n°",I5,"  Milieu =",ES12.3E2)',n,epsr(2)
  
    if (.not.flagnommilieu) then
      PRINT '("ERREUR dans la subroutine ''milieu_EpsySigey_PlaqueMince'', nom milieu non reconnu !")' 
      PRINT '("    Nom milieu=",A," ; n° de la plaque=",I6," ; Coordonnées = ",6I6)',trim(tp%nommilieu),n,tp%coord 
      stop
    end if
  
      !--- plan xoy ---
    ipas = tp%coord(6)-tp%coord(5); if (ipas==0) ipas=1
    do k=tp%coord(5),tp%coord(6),ipas
    do j=tp%coord(3),tp%coord(4) -1
    do i=tp%coord(1),tp%coord(2)
	num = t3d(i,j,k)
	if (i<tp%coord(2)) then
   	  d(4,num)  = tp%d
	  eps_pm(4,num)       = epsr(2) ! epsy
	end if
	if (i>tp%coord(1)) then
   	  d(3,num)  = tp%d
	  eps_pm(3,num)       = epsr(2) ! epsy     
	end if
    end do; end do; end do
      !--- plan yoz ---
    ipas = tp%coord(2)-tp%coord(1); if (ipas==0) ipas=1
    do i=tp%coord(1),tp%coord(2),ipas
    do k=tp%coord(5),tp%coord(6) 
    do j=tp%coord(3),tp%coord(4) -1
	num = t3d(i,j,k)
	if (k<tp%coord(6)) then
	  d(2,num)  = tp%d
	  eps_pm(2,num)       = epsr(2) ! epsy
	end if
	if (k>tp%coord(5)) then
	  d(1,num)  = tp%d
	  eps_pm(1,num)       = epsr(2) ! epsy     
	end if
    end do; end do; end do
  end do
  
  !--- Correction de epsilon Y ----
  d1 = dxyz(3); d2 = dxyz(1)
  Scell = d1*d2
  print '(T15,"Rang=",I5," ; Surface cellule        = ",ES15.5E2)',rang,Scell
  do k=c1(5),c1(6)
  do j=c1(3),c1(4)
  do i=c1(1),c1(2)
      if (t3d(i,j,k)>0) then
	num = t3d(i,j,k)
	epscell(1) = tm%tampon(i,j,k-1)
	epscell(2) = tm%tampon(i,j,k)
	epscell(3) = tm%tampon(i-1,j,k-1)
	epscell(4) = tm%tampon(i-1,j,k)
	! Calcul des surfaces élémentaires
	Szcell(1)  = (d1-d(4,num))*(d2-d(1,num))*0.25
	Szcell(2)  = (d1-d(4,num))*(d2-d(2,num))*0.25
	Szcell(3)  = (d1-d(3,num))*(d2-d(1,num))*0.25
	Szcell(4)  = (d1-d(3,num))*(d2-d(2,num))*0.25
	
	Scellmoins = abs(d(3,num)-d(4,num))*abs(d(2,num)-d(1,num))*0.25
	
	if ((d(4,num)<=d(3,num)).and.(d(1,num)<=d(2,num))) then
	  Szcell(1) = Szcell(1) - Scellmoins
	else if ((d(4,num)<=d(3,num)).and.(d(2,num)<=d(1,num))) then
	  Szcell(2) = Szcell(2) - Scellmoins
	else if ((d(3,num)<=d(4,num)).and.(d(1,num)<=d(2,num))) then	
	  Szcell(3) = Szcell(3) - Scellmoins
	else
	  Szcell(4) = Szcell(4) - Scellmoins	
	end if

	Szpm(1)  = d(1,num) * (d1-maxval(d(3:4,num)))*0.5
	Szpm(2)  = d(2,num) * (d1-maxval(d(3:4,num)))*0.5
	Szpm(3)  = d(3,num) * (d2-maxval(d(1:2,num)))*0.5
	Szpm(4)  = d(4,num) * (d2-maxval(d(1:2,num)))*0.5
	Szcentre = maxval(d(1:2,num))*maxval(d(3:4,num)) 
	epscentre = 0
	n = count(eps_pm(1:4,num)>0)
	if (n>0) epscentre = SUM(eps_pm(1:4,num))/n
	eps_sige(i,j,k) = ( SUM(epscell*Szcell)+SUM(eps_pm(1:4,num)*Szpm)+ &
			   Szcentre*epscentre ) /Scell
! 	print '(T15,"(i,j,k = ", 3I5,") num = ",I5)',i,j,k,num
! 	print '(T20,"epscell = ", 4F6.2," eps_pm = ",4F6.2)',epscell(1:4),eps_pm(1:4,num)
! 	print '(T20,"Szcell = ", 4F9.5," Szpm = ",4F9.5,"Szcentre",F9.5)',Szcell(1:4),Szpm(1:4),Szcentre
!        print '(T15,"d =",2F7.3," ; epsx = ",F6.2,"  sum Surf cell = ",ES15.5E2)', &
!	                  d(1:2,num),eps_sige(i,j,k),(SUM(Szcell)+SUM(Szpm)+Szcentre)
      end if
  end do; end do; end do  
  
  deallocate(eps_pm,d)

end if

deallocate(t3d)

end subroutine milieu_EpsySigey_PlaqueMince


! ==========================================================
! == Subroutine milieu_EpszSigez_PlaqueMince              ==
! == Modification de la permittivité et de la perméabilité  ==
! == pour les plaques minces                              ==
! ==========================================================	
subroutine milieu_EpszSigez_PlaqueMince(tm,ts,dxyz,t_lmnd,typemilieu,bv)

implicit none

type(st_milieu_volume), intent(inout) :: tm
type(st_structure), intent(in)        :: ts
real,dimension(1:3), intent(in)       :: dxyz
type(st_liste_milieu_simple), intent(inout) :: t_lmnd
integer, intent(in)                   :: typemilieu ! (0-> eps; 1 -> sigma)
integer,dimension(6),intent(in)       :: bv

logical   :: flagnommilieu
integer   :: i,j,k,n,ipas
integer   :: num,mcell
integer, dimension(6)   :: c,c1
real, dimension(3)      :: epsr
real, dimension(4)      :: epscell, Szcell,Szpm
real                    :: epscentre,Szcentre,Scell,Scellmoins
real                    :: d1,d2 ! les deux pas transverses
type(st_structure_plaque_mince), pointer :: tp

integer, dimension(:,:,:), allocatable :: t3d
real, dimension(:,:,:), pointer  :: eps_sige
real   , dimension(:,:), allocatable :: eps_pm
real   , dimension(:,:), allocatable :: d

!=== DÉBUT DES TRAITEMENTS ===
print '(T5,"Rang=",I5," ; PLAQUE MINCE, traitement de epsilon ou sige Z :",I2)',rang,typemilieu
! on évalue l'espace occupé par les plaques (ccordonnées min et max)
do n=1,ts%nbplaque
  tp => ts%t_plaque(n)
  if (n== 1)  then
    c = tp%coord
  else
    do i=1,5,2
      c(i)   = min(c(i)  ,tp%coord(i)  )
      c(i+1) = mAX(c(i+1),tp%coord(i+1))
    end do
  end if
end do
do i=1,5,2
    c1(i)  = MAX(c(i),bv(i))
    c1(i+1)= c(i+1)  
end do

print '(T15,"Rang=",I5," ; Espace occupé par les plaques :",6I5)',rang,c

! Allocation du tableau qui contiendra le type de jonction
allocate(t3d(c(1):c(2),c(3):c(4),c(5):c(6)))
t3d = 0

!-------------------- TRAITEMENT POUR EPSILON OU SIGMA Z ----------------------
! Recherche des cellules concernés par les modifications de epsilon Z
do n=1,ts%nbplaque
  tp => ts%t_plaque(n)
  IF (tp%coord(6)==tp%coord(5)) cycle
  
    !--- plan yoz ---
  do k=tp%coord(5),tp%coord(6)-1
  do j=tp%coord(3),tp%coord(4)
    t3d(tp%coord(1),j,k) = 1  
    t3d(tp%coord(2),j,k) = 1  
  end do; end do; 
    !--- plan xoz ---
  do k=tp%coord(5),tp%coord(6) -1
  do i=tp%coord(1),tp%coord(2)
    t3d(i,tp%coord(3),k) = 1  
    t3d(i,tp%coord(4),k) = 1  
  end do; end do;   
end do  
mcell = sum(t3d);
print '(T15,"Rang=",I5," ; Nombre de cellules concerné par les plaques :",I8)',rang,mcell
  
  
if (mcell>0) then !----- Modif à faire donc ---
  !---- numérotation seulement des cellules à modifier ----
  num = 0
  do k=c(5),c(6) 
  do j=c(3),c(4) 
  do i=c(1),c(2)
    num = num + t3d(i,j,k) ! incrémente si t3d(i,j,k) == 1
    t3d(i,j,k) = t3d(i,j,k)*num
  end do; end do; end do  
  
  !---- Allocation des tableaux ----
  allocate(eps_pm(1:8,1:mcell)) ; eps_pm = 0.0
  allocate(d(1:4,1:mcell)) ; d =0.0
  
  !---- Initialisation de eps_pm -------
  ! le tableau peut aussi etre concerné par la conductivité sigma
  !   ca dépend du contenu du tableau tampon
!  do k=c(5),c(6)
!  do j=c(3),c(4)
!  do i=c(1),c(2)
!      if (t3d(i,j,k)>0) then
!	num = t3d(i,j,k)
!	eps_pm(1,num)  =  (tm%tampon(i-1,j-1,k) + tm%tampon(i-1,j,k) )*0.5
!	eps_pm(2,num)  =  (tm%tampon(i,j-1,k)   + tm%tampon(i,j,k)   )*0.5
!	eps_pm(3,num)  =  (tm%tampon(i-1,j-1,k) + tm%tampon(i,j-1,k) )*0.5
!	eps_pm(4,num)  =  (tm%tampon(i-1,j,k)   + tm%tampon(i,j,k)   )*0.5
 !     end if
!  end do; end do; end do  
  
  
  
  !--- Détermination des propriétés eps pour chaque cellule ----
  do n=1,ts%nbplaque
    tp => ts%t_plaque(n)
    IF (tp%coord(6)==tp%coord(5)) cycle
    if (typemilieu == 0) then ! c'est la permittivité
      flagnommilieu = cherche_epsilon_milieu(t_lmnd,tp%nommilieu,epsr)
      eps_sige =>tm%epsz
    else ! c'est la conductivité
      flagnommilieu = cherche_sigmae_milieu(t_lmnd,tp%nommilieu,epsr)  
      eps_sige =>tm%sigez
    end if
    !print '(T15," plaque n°",I5,"  Milieu =",ES12.3E2)',n,epsr(3)
  
    if (.not.flagnommilieu) then
      PRINT '("ERREUR dans la subroutine ''milieu_EpszSigez_PlaqueMince'', nom milieu non reconnu !")' 
      PRINT '("    Nom milieu=",A," ; n° de la plaque=",I6," ; Coordonnées = ",6I6)',trim(tp%nommilieu),n,tp%coord 
      stop
    end if
  
      !--- plan yoz ---
    ipas = tp%coord(2)-tp%coord(1); if (ipas==0) ipas=1
    do i=tp%coord(1),tp%coord(2),ipas
    do k=tp%coord(5),tp%coord(6) -1
    do j=tp%coord(3),tp%coord(4)
	num = t3d(i,j,k)
	if (j<tp%coord(4)) then
   	  d(4,num)  = tp%d
	  eps_pm(4,num)       = epsr(3) ! epsz
	end if
	if (j>tp%coord(3)) then
   	  d(3,num)  = tp%d
	  eps_pm(3,num)       = epsr(3) ! epsz     
	end if
    end do; end do; end do
      !--- plan xoz ---
    ipas = tp%coord(4)-tp%coord(3); if (ipas==0) ipas=1
    do j=tp%coord(3),tp%coord(4),ipas
    do k=tp%coord(5),tp%coord(6) -1
    do i=tp%coord(1),tp%coord(2)
	num = t3d(i,j,k)
	if (i<tp%coord(2)) then
	  d(2,num)  = tp%d
	  eps_pm(2,num)       = epsr(3) ! epsz
	end if
	if (i>tp%coord(1)) then
	  d(1,num)  = tp%d
	  eps_pm(1,num)       = epsr(3) ! epsz     
	end if
    end do; end do; end do  
  end do
  
  !--- Correction de epsilon Z ----
  d1 = dxyz(1); d2 = dxyz(2)
  Scell = d1*d2
  print '(T15,"Rang=",I5," ; Surface cellule        = ",ES15.5E2)',rang,Scell
  do k=c1(5),c1(6)
  do j=c1(3),c1(4)
  do i=c1(1),c1(2)
      if (t3d(i,j,k)>0) then
	num = t3d(i,j,k)
	epscell(1) = tm%tampon(i-1,j,k)
	epscell(2) = tm%tampon(i  ,j,k)
	epscell(3) = tm%tampon(i-1,j-1,k)
	epscell(4) = tm%tampon(i  ,j-1,k)
	! Calcul des surfaces élémentaires
	Szcell(1)  = (d1-d(4,num))*(d2-d(1,num))*0.25
	Szcell(2)  = (d1-d(4,num))*(d2-d(2,num))*0.25
	Szcell(3)  = (d1-d(3,num))*(d2-d(1,num))*0.25
	Szcell(4)  = (d1-d(3,num))*(d2-d(2,num))*0.25
	
	Scellmoins = abs(d(3,num)-d(4,num))*abs(d(2,num)-d(1,num))*0.25
	
	if ((d(4,num)<=d(3,num)).and.(d(1,num)<=d(2,num))) then
	  Szcell(1) = Szcell(1) - Scellmoins
	else if ((d(4,num)<=d(3,num)).and.(d(2,num)<=d(1,num))) then
	  Szcell(2) = Szcell(2) - Scellmoins
	else if ((d(3,num)<=d(4,num)).and.(d(1,num)<=d(2,num))) then	
	  Szcell(3) = Szcell(3) - Scellmoins
	else
	  Szcell(4) = Szcell(4) - Scellmoins	
	end if
	
	Szpm(1)  = d(1,num) * (d1-maxval(d(3:4,num)))*0.5
	Szpm(2)  = d(2,num) * (d1-maxval(d(3:4,num)))*0.5
	Szpm(3)  = d(3,num) * (d2-maxval(d(1:2,num)))*0.5
	Szpm(4)  = d(4,num) * (d2-maxval(d(1:2,num)))*0.5
	Szcentre = maxval(d(1:2,num))*maxval(d(3:4,num)) 
	
	epscentre = 0
	n = count(eps_pm(1:4,num)>0)
	if (n>0) epscentre = SUM(eps_pm(1:4,num))/n
	eps_sige(i,j,k) = ( SUM(epscell*Szcell)+SUM(eps_pm(1:4,num)*Szpm)+ &
			   Szcentre*epscentre ) /Scell

! 	print '(T15,"(i,j,k,n° = ", 3I5,") num = ",I5)',i,j,k,num
! 	print '(T20,"epscell = ", 4F6.2," eps_pm = ",4F6.2)',epscell(1:4),eps_pm(1:4,num)
! 	print '(T20,"Szcell = ", 4F9.5," Szpm = ",4F9.5,"Szcentre",F9.5)',Szcell(1:4),Szpm(1:4),Szcentre
!        print '(T20,"d =",2F9.5," ; eps_sigez = ",F10.6)', d(1:2,num),eps_sige(i,j,k)
    end if
  end do; end do; end do  
  deallocate(eps_pm,d)

end if

deallocate(t3d)

end subroutine milieu_EpszSigez_PlaqueMince

! ==========================================================
! == Subroutine milieu_modif_eps_mu_fente                    ==
! == Modification de la permittivité et de la perméabilité  ==
! == au niveau des fentes                                 ==
! ==========================================================	
subroutine milieu_modif_eps_mu_fente(tm,ts)

implicit none

type(st_milieu_volume), intent(inout) :: tm
type(st_structure), intent(in)        :: ts

integer   :: i,j,k,n
integer, dimension(3)   :: offs
real,dimension(:,:,:), pointer :: eps, mu
type(st_structure_fente), pointer :: tfente

!=== DÉBUT DES TRAITEMENTS ===

do n=1,ts%nbfente
  tfente => ts%t_fente(n)
  !--- modification de epsilon ---
  offs = 0
  if (tfente%dirE == DIRECTION_X) then
    eps => tm%epsx; offs(DIRECTION_X) = 1
  else if (tfente%dirE == DIRECTION_Y) then
    eps => tm%epsy; offs(DIRECTION_Y) = 1
  else
    eps => tm%epsz; offs(DIRECTION_Z) = 1
  end if
  do k=tfente%coord(5),tfente%coord(6) -offs(3)
  do j=tfente%coord(3),tfente%coord(4) -offs(2)
  do i=tfente%coord(1),tfente%coord(2) -offs(1)
    eps(i,j,k) = eps(i,j,k) * tfente%epsr
  end do;end do; end do
  
  !--- modification de mu ---
  offs = 1
  if (tfente%dirH == DIRECTION_X) then
    mu => tm%mux; offs(DIRECTION_X) = 0
  else if (tfente%dirH == DIRECTION_Y) then
    mu => tm%muy; offs(DIRECTION_Y) = 0
  else
    mu => tm%muz; offs(DIRECTION_Z) = 0
  end if
  do k=tfente%coord(5),tfente%coord(6) -offs(3)
  do j=tfente%coord(3),tfente%coord(4) -offs(2)
  do i=tfente%coord(1),tfente%coord(2) -offs(1)
    mu(i,j,k) = mu(i,j,k) / tfente%epsr
  end do;end do; end do
end do
end subroutine milieu_modif_eps_mu_fente

! ==========================================================
! == Subroutine milieu_modif_eps_mu_fente                    ==
! == Modification de la permittivité et de la perméabilité  ==
! == au niveau des fentes                                 ==
! ==========================================================	
subroutine milieu_rectif_coeff_fente(tm,ts,tc,tv)

implicit none

type(st_milieu_volume), intent(inout) :: tm
type(st_structure), intent(in)        :: ts
type(st_champ), intent(in)            :: tc
type(st_volume), intent(in)           :: tv
    
integer   :: i,j,k,n
integer, dimension(3)   :: offsfin,offsdeb
real,dimension(:,:,:), pointer :: eps, sige,ce2
type(st_structure_fente), pointer :: tfente

!=== DÉBUT DES TRAITEMENTS ===

do n=1,ts%nbfente
  tfente => ts%t_fente(n)
  
  !--- modification de epsilon ---
  offsfin = 0
  offsdeb = 0
  offsfin(tfente%dirE) = 1
  offsfin(tfente%dirfente) = 1
  offsdeb(tfente%dirfente) = 1
  if (tfente%dirE == DIRECTION_X) then
    eps => tm%epsx; 
    ce2 => tc%cex2
    if (tm%flag_eps_sigmae) sige=> tm%sigex
  else if (tfente%dirE == DIRECTION_Y) then
    eps => tm%epsy; 
    ce2 => tc%cey2
    if (tm%flag_eps_sigmae) sige=> tm%sigey
  else
    eps => tm%epsz; 
    ce2 => tc%cez2
    if (tm%flag_eps_sigmae) sige=> tm%sigez
  end if
  
  !--> Dépend du schéma : exponentielle ou linéaire
  if ( .not. tv%flag_schema_expon) then
    do k=tfente%coord(5)+offsdeb(3),tfente%coord(6) -offsfin(3)
    do j=tfente%coord(3)+offsdeb(2),tfente%coord(4) -offsfin(2)
    do i=tfente%coord(1)+offsdeb(1),tfente%coord(2) -offsfin(1)
      ce2(i,j,k) = 1 / (eps(i,j,k))
      if (tm%flag_eps_sigmae) &
	ce2(i,j,k) = EPSILON_0 / (EPSILON_0*eps(i,j,k) + 0.5*sige(i,j,k)*tv%dt)
    end do;end do; end do
  else 
    do k=tfente%coord(5)+offsdeb(3),tfente%coord(6) -offsfin(3)
    do j=tfente%coord(3)+offsdeb(2),tfente%coord(4) -offsfin(2)
    do i=tfente%coord(1)+offsdeb(1),tfente%coord(2) -offsfin(1)
      ce2(i,j,k) = 1 / (eps(i,j,k))
      if (tm%flag_eps_sigmae.and.sige(i,j,k)>0) &
        ! eps0/(dt*sige) *exp(-sige*dt/eps)
	ce2(i,j,k) = (EPSILON_0/(sige(i,j,k)*tv%dt))*&
		       (1.0-exp(-sige(i,j,k)*tv%dt/(EPSILON_0*eps(i,j,k))))
    end do;end do; end do    
  end if  
end do
end subroutine milieu_rectif_coeff_fente



! ==========================================================
! == Subroutine milieu_volume_calcul_coeff                ==
! == Calcul des coefficients multiplicatifs des champs    ==
! ==========================================================	
subroutine milieu_volume_calcul_coeff(t_m, v,t_c,t_s)

implicit none

type(st_milieu_volume), intent(inout) :: t_m
type(st_volume), intent(in)           :: v
type(st_champ), intent(inout)           :: t_c
type(st_structure), intent(in)           :: t_s

integer   :: i,j,k,l,offset


!if  ((t_s%nbmilieu >0) .or. t_s%type_milieu_dielec==VOL_DIELEC_INHOMOGENE .or. &
!      t_s%type_milieu_dielec==VOL_DIELEC_PERTES_INHOMOGENE) then  
      
  !***   calcul des coefficients pour E  ***
  if (t_m%flag_eps .and.(.not.t_m%flag_eps_sigmae)) then ! pas de pertes
    t_c%flag_dielectrique_homogene = .false.
    t_c%flag_diel_pertes_homogene = .false.
    t_c%flag_dielectrique_inhomogene = .true.
    t_c%flag_dielectrique_quelconque = .false.
    allocate(t_c%cex2(v%ivold-1:v%ivolf+1,v%jvold-1:v%jvolf+1,v%kvold-1:v%kvolf+1)); 
    allocate(t_c%cey2(v%ivold-1:v%ivolf+1,v%jvold-1:v%jvolf+1,v%kvold-1:v%kvolf+1)); 
    allocate(t_c%cez2(v%ivold-1:v%ivolf+1,v%jvold-1:v%jvolf+1,v%kvold-1:v%kvolf+1)); 
    t_c%cex2 = 1.0
    t_c%cey2 = 1.0
    t_c%cez2 = 1.0
    
    do k = v%kvold-1,v%kvolf+1
    do j = v%jvold-1,v%jvolf+1
    do i = v%ivold-1,v%ivolf+1
      t_c%cex2(i,j,k) = 1 / (t_m%epsx(i,j,k))
      t_c%cey2(i,j,k) = 1 / (t_m%epsy(i,j,k))
      t_c%cez2(i,j,k) = 1 / (t_m%epsz(i,j,k))
    end do; end do; end do 
    
  else if (t_m%flag_eps_sigmae) then ! avec pertes
    t_c%flag_dielectrique_homogene = .false.
    t_c%flag_diel_pertes_homogene = .false.
    t_c%flag_dielectrique_quelconque = .true.
    t_c%flag_dielectrique_inhomogene = .false.

    allocate(t_c%cex1(v%ivold-1:v%ivolf+1,v%jvold-1:v%jvolf+1,v%kvold-1:v%kvolf+1)); 
    allocate(t_c%cey1(v%ivold-1:v%ivolf+1,v%jvold-1:v%jvolf+1,v%kvold-1:v%kvolf+1)); 
    allocate(t_c%cez1(v%ivold-1:v%ivolf+1,v%jvold-1:v%jvolf+1,v%kvold-1:v%kvolf+1)); 
    allocate(t_c%cex2(v%ivold-1:v%ivolf+1,v%jvold-1:v%jvolf+1,v%kvold-1:v%kvolf+1)); 
    allocate(t_c%cey2(v%ivold-1:v%ivolf+1,v%jvold-1:v%jvolf+1,v%kvold-1:v%kvolf+1)); 
    allocate(t_c%cez2(v%ivold-1:v%ivolf+1,v%jvold-1:v%jvolf+1,v%kvold-1:v%kvolf+1)); 
    t_c%cex1 = 1.0;    t_c%cey1 = 1.0;    t_c%cez1 = 1.0
    t_c%cex2 = 0.0;    t_c%cey2 = 0.0;    t_c%cez2 = 0.0
    
    !--> Schéma exponentiel ou linéaire ?
    if (v%flag_schema_expon) then
      do k = v%kvold-1,v%kvolf+1
      do j = v%jvold-1,v%jvolf+1
      do i = v%ivold-1,v%ivolf+1
	t_c%cex1(i,j,k) = exp(-t_m%sigex(i,j,k)*v%dt/(EPSILON_0*t_m%epsx(i,j,k)))
	t_c%cey1(i,j,k) = exp(-t_m%sigey(i,j,k)*v%dt/(EPSILON_0*t_m%epsy(i,j,k)))
	t_c%cez1(i,j,k) = exp(-t_m%sigez(i,j,k)*v%dt/(EPSILON_0*t_m%epsz(i,j,k)))
	!--> schéma exponentiel si présence d'une conductivité associée à EX
	if (t_m%sigex(i,j,k)>0) then
	  t_c%cex2(i,j,k) = (EPSILON_0/(t_m%sigex(i,j,k)*v%dt))*&
			      (1.0-exp(-t_m%sigex(i,j,k)*v%dt/(EPSILON_0*t_m%epsx(i,j,k))))
	else
          t_c%cex2(i,j,k) = 1.0 / (t_m%epsx(i,j,k))
	end if
	!--> schéma exponentiel si présence d'une conductivité associée à EY
	if (t_m%sigey(i,j,k)>0) then
          t_c%cey2(i,j,k) = (EPSILON_0/(t_m%sigey(i,j,k)*v%dt))*&
	                      (1.0-exp(-t_m%sigey(i,j,k)*v%dt/(EPSILON_0*t_m%epsy(i,j,k))))
	else
          t_c%cey2(i,j,k) = 1.0 / (t_m%epsy(i,j,k))
	end if
	!--> schéma exponentiel si présence d'une conductivité associée à EZ
	if (t_m%sigez(i,j,k)>0) then
	  t_c%cez2(i,j,k) = (EPSILON_0/(t_m%sigez(i,j,k)*v%dt))*&
			      (1.0-exp(-t_m%sigez(i,j,k)*v%dt/(EPSILON_0*t_m%epsz(i,j,k))))
	else
          t_c%cez2(i,j,k) = 1.0 / (t_m%epsz(i,j,k))
	end if
			      
      end do; end do; end do 
    else
      do k = v%kvold-1,v%kvolf+1
      do j = v%jvold-1,v%jvolf+1
      do i = v%ivold-1,v%ivolf+1
	t_c%cex1(i,j,k) = (EPSILON_0*t_m%epsx(i,j,k) - 0.5*t_m%sigex(i,j,k)*v%dt) / &
			  (EPSILON_0*t_m%epsx(i,j,k) + 0.5*t_m%sigex(i,j,k)*v%dt)
	t_c%cey1(i,j,k) = (EPSILON_0*t_m%epsy(i,j,k) - 0.5*t_m%sigey(i,j,k)*v%dt) / &
			  (EPSILON_0*t_m%epsy(i,j,k) + 0.5*t_m%sigey(i,j,k)*v%dt)
	t_c%cez1(i,j,k) = (EPSILON_0*t_m%epsz(i,j,k) - 0.5*t_m%sigez(i,j,k)*v%dt) / &
			  (EPSILON_0*t_m%epsz(i,j,k) + 0.5*t_m%sigez(i,j,k)*v%dt)
  
	t_c%cex2(i,j,k) = EPSILON_0 / (EPSILON_0*t_m%epsx(i,j,k) + 0.5*t_m%sigex(i,j,k)*v%dt)
	t_c%cey2(i,j,k) = EPSILON_0 / (EPSILON_0*t_m%epsy(i,j,k) + 0.5*t_m%sigey(i,j,k)*v%dt)
	t_c%cez2(i,j,k) = EPSILON_0 / (EPSILON_0*t_m%epsz(i,j,k) + 0.5*t_m%sigez(i,j,k)*v%dt)
      end do; end do; end do 
    end if    
  end if

  !***   calcul des coefficients pour H  ***
  if (t_m%flag_mu .and.(.not.t_m%flag_mu_sigmah)) then
    t_c%flag_magnetique_homogene = .false.
    t_c%flag_magnet_pertes_homogene = .false.
    t_c%flag_magnetique_inhomogene = .true.
    t_c%flag_magnetique_quelconque = .false.

    allocate(t_c%chx2(v%ivold-1:v%ivolf+1,v%jvold-1:v%jvolf+1,v%kvold-1:v%kvolf+1)); 
    allocate(t_c%chy2(v%ivold-1:v%ivolf+1,v%jvold-1:v%jvolf+1,v%kvold-1:v%kvolf+1)); 
    allocate(t_c%chz2(v%ivold-1:v%ivolf+1,v%jvold-1:v%jvolf+1,v%kvold-1:v%kvolf+1)); 
    t_c%chx2 = 1.0
    t_c%chy2 = 1.0
    t_c%chz2 = 1.0
    
    do k = v%kvold-1,v%kvolf+1
    do j = v%jvold-1,v%jvolf+1
    do i = v%ivold-1,v%ivolf+1
      t_c%chx2(i,j,k) = 1 / (t_m%mux(i,j,k))
      t_c%chy2(i,j,k) = 1 / (t_m%muy(i,j,k))
      t_c%chz2(i,j,k) = 1 / (t_m%muz(i,j,k))
    end do; end do; end do 
    
  else if (t_m%flag_mu_sigmah) then ! présence des pertes magnétiques
    t_c%flag_magnetique_homogene = .false.
    t_c%flag_magnet_pertes_homogene = .false.
    t_c%flag_magnetique_inhomogene = .false.
    t_c%flag_magnetique_quelconque = .true.
    
    allocate(t_c%chx1(v%ivold-1:v%ivolf+1,v%jvold-1:v%jvolf+1,v%kvold-1:v%kvolf+1)); 
    allocate(t_c%chy1(v%ivold-1:v%ivolf+1,v%jvold-1:v%jvolf+1,v%kvold-1:v%kvolf+1)); 
    allocate(t_c%chz1(v%ivold-1:v%ivolf+1,v%jvold-1:v%jvolf+1,v%kvold-1:v%kvolf+1)); 
    allocate(t_c%chx2(v%ivold-1:v%ivolf+1,v%jvold-1:v%jvolf+1,v%kvold-1:v%kvolf+1)); 
    allocate(t_c%chy2(v%ivold-1:v%ivolf+1,v%jvold-1:v%jvolf+1,v%kvold-1:v%kvolf+1)); 
    allocate(t_c%chz2(v%ivold-1:v%ivolf+1,v%jvold-1:v%jvolf+1,v%kvold-1:v%kvolf+1)); 
    t_c%chx1 = 1.0;    t_c%chy1 = 1.0;    t_c%chz1 = 1.0
    t_c%chx2 = 0.0;    t_c%chy2 = 0.0;    t_c%chz2 = 0.0
    
    if (v%flag_schema_expon) then
      do k = v%kvold-1,v%kvolf+1
      do j = v%jvold-1,v%jvolf+1
      do i = v%ivold-1,v%ivolf+1
	t_c%chx1(i,j,k) = exp(-t_m%sighx(i,j,k)*v%dt/(MU_0*t_m%mux(i,j,k)))
	t_c%chy1(i,j,k) = exp(-t_m%sighy(i,j,k)*v%dt/(MU_0*t_m%muy(i,j,k)))
	t_c%chz1(i,j,k) = exp(-t_m%sighz(i,j,k)*v%dt/(MU_0*t_m%muz(i,j,k)))
	
	!--> schéma exponentiel si présence d'une conductivité associée à HX
	if (t_m%sighx(i,j,k)>0) then
	  t_c%chx2(i,j,k) = (MU_0/(t_m%sighx(i,j,k)*v%dt))*&
		         (1.0-exp(-t_m%sighx(i,j,k)*v%dt/(MU_0*t_m%mux(i,j,k))))
	else
          t_c%chx2(i,j,k) = 1.0 / (t_m%mux(i,j,k))
	end if

	!--> schéma exponentiel si présence d'une conductivité associée à HY
	if (t_m%sighy(i,j,k)>0) then
	  t_c%chy2(i,j,k) = (MU_0/(t_m%sighy(i,j,k)*v%dt))*&
		         (1.0-exp(-t_m%sighy(i,j,k)*v%dt/(MU_0*t_m%muy(i,j,k))))
	else
          t_c%chy2(i,j,k) = 1.0 / (t_m%muy(i,j,k))
	end if
	!--> schéma exponentiel si présence d'une conductivité associée à HZ
	if (t_m%sighz(i,j,k)>0) then
	  t_c%chz2(i,j,k) = (MU_0/(t_m%sighz(i,j,k)*v%dt))*&
		         (1.0-exp(-t_m%sighz(i,j,k)*v%dt/(MU_0*t_m%muz(i,j,k))))
	else
          t_c%chz2(i,j,k) = 1.0 / (t_m%muz(i,j,k))
	end if
      end do; end do; end do 
    else
      do k = v%kvold-1,v%kvolf+1
      do j = v%jvold-1,v%jvolf+1
      do i = v%ivold-1,v%ivolf+1
	t_c%chx1(i,j,k) = (MU_0*t_m%mux(i,j,k) - 0.5*t_m%sighx(i,j,k)*v%dt) / &
			  (MU_0*t_m%mux(i,j,k) + 0.5*t_m%sighx(i,j,k)*v%dt)
	t_c%chy1(i,j,k) = (MU_0*t_m%muy(i,j,k) - 0.5*t_m%sighy(i,j,k)*v%dt) / &
			  (MU_0*t_m%muy(i,j,k) + 0.5*t_m%sighy(i,j,k)*v%dt)
	t_c%chz1(i,j,k) = (MU_0*t_m%muz(i,j,k) - 0.5*t_m%sighz(i,j,k)*v%dt) / &
			  (MU_0*t_m%muz(i,j,k) + 0.5*t_m%sighz(i,j,k)*v%dt)
  
	t_c%chx2(i,j,k) = MU_0 / (MU_0*t_m%mux(i,j,k) + 0.5*t_m%sighx(i,j,k)*v%dt)
	t_c%chy2(i,j,k) = MU_0 / (MU_0*t_m%muy(i,j,k) + 0.5*t_m%sighy(i,j,k)*v%dt)
	t_c%chz2(i,j,k) = MU_0 / (MU_0*t_m%muz(i,j,k) + 0.5*t_m%sighz(i,j,k)*v%dt)
      end do; end do; end do 
    end if
  end if
  
!end if


end subroutine milieu_volume_calcul_coeff


! ==========================================================
! == Subroutine milieu_volume_annule_coeff          ==
! == pour l'annulation des composantes tangentielles du   ==
! == champ électrique
! ==========================================================	
subroutine milieu_volume_annule_coeff(t_c,t_s,v)
implicit none

type(st_champ), intent(inout)     :: t_c
type(st_structure), intent(in)    :: t_s
type(st_volume), intent(in)       :: v

integer   :: n,r,en
integer   :: i,j,imin,imax,jmin,jmax,kmin,kmax,ei,ej,ek
integer   :: igm
real, allocatable, dimension(:,:,:)  :: ctemp


if (t_c%flag_dielectrique_inhomogene .or. t_c%flag_dielectrique_quelconque) then
  allocate(ctemp(v%ivold-1:v%ivolf+1,v%jvold-1:v%jvolf+1,v%kvold-1:v%kvolf+1)); 
  ctemp = 0.0
  !-------------- TRAITEMENT DE LA COMPOSANTE EX -------------
  ctemp = t_c%cex2
  igm = 1
  do i= 1,t_s%nbmetal
    imin = t_s%t_metal(1,i) 
    imax = t_s%t_metal(2,i) 
    jmin = t_s%t_metal(3,i) 
    jmax = t_s%t_metal(4,i) 
    kmin = t_s%t_metal(5,i) 
    kmax = t_s%t_metal(6,i) 
    if (t_s%t_metal(7,i) == 1) then
!    print '("coord metal pour ex =",6I4)',imin,imax,jmin,jmax,kmin,kmax
      t_c%cex2(imin:imax-1,jmin:jmax,kmin) = 0.0    ! face xoyinf
      t_c%cex2(imin:imax-1,jmin,kmin:kmax) = 0.0    ! face xozinf  
      if (kmax>kmin) t_c%cex2(imin:imax-1,jmin:jmax,kmax) = 0.0  ! face xoysup    
      if (jmax>jmin) t_c%cex2(imin:imax-1,jmax,kmin:kmax) = 0.0  ! face xozsup    
    else if (t_s%t_metal(7,i) == 0) then ! Pas de métaux, on rétabli la situation d'origine
!    print '("coord non metal pour ex =",6I4)',imin,imax,jmin,jmax,kmin,kmax
      if (jmax-1>jmin) t_c%cex2(imin:imax-1,jmin+1:jmax-1,kmin) = ctemp(imin:imax-1,jmin+1:jmax-1,kmin)  ! face xoyinf
      if (kmax-1>kmin) t_c%cex2(imin:imax-1,jmin,kmin+1:kmax-1) = ctemp(imin:imax-1,jmin,kmin+1:kmax-1)  ! face xozinf
      if (kmax>kmin) t_c%cex2(imin:imax-1,jmin+1:jmax-1,kmax) = ctemp(imin:imax-1,jmin+1:jmax-1,kmax) ! face xoysup
      if (jmax>jmin) t_c%cex2(imin:imax-1,jmax,kmin+1:kmax-1) = ctemp(imin:imax-1,jmax,kmin+1:kmax-1) ! face xozsup
    else if (t_s%t_metal(7,i) == 2) then
      ei   = t_s%t_grille_metal(1,igm)
      ej   = t_s%t_grille_metal(2,igm)
      ek   = t_s%t_grille_metal(3,igm)
      igm=igm+1
      if (imax<=imin) cycle
      if (ej>0) then ! plan xoy
        if (rang==RANG_MAITRE_AFF) print '(T10," -->N°metal = ",I3," ;  ei,ej,ek = ",3I4)',i,ei,ej,ek
	n = (jmax-jmin)/ej
	if (n>0) then
	  r = (jmax-jmin) -n*ej
	  t_c%cex2(imin:imax-1,jmin:jmax:jmax-jmin,kmin) = 0.0 ! xoyinf
	  t_c%cex2(imin:imax-1,jmin:jmax:jmax-jmin,kmax) = 0.0 ! xoysup
	  en = r/2
          if (rang==RANG_MAITRE_AFF) print '(T15,"EX, PLAN XOY  n = ",I4,"  ;  r=",I3)',n,r
	  do j=1,n-1
	    en= en+ej
	    t_c%cex2(imin:imax-1,jmin+en,kmin) = 0.0 ! xoyinf
	    t_c%cex2(imin:imax-1,jmin+en,kmax) = 0.0 ! xoysup
	  end do
	end if
      end if
      if (ek>0) then ! plan xoz
        if (rang==RANG_MAITRE_AFF) print '(T10," -->N°metal = ",I3," ;  ei,ej,ek = ",3I4)',i,ei,ej,ek
	n = (kmax-kmin)/ek
	if (n>0) then
	  r = (kmax-kmin) -n*ek
	  t_c%cex2(imin:imax-1,jmin,kmin:kmax:kmax-kmin) = 0.0 ! xozinf
	  t_c%cex2(imin:imax-1,jmax,kmin:kmax:kmax-kmin) = 0.0 ! xozinf
	  en = r/2
          if (rang==RANG_MAITRE_AFF) print '(T15,"EX, PLAN XOZ  n = ",I4,"  ;  r=",I3)',n,r
	  do j=1,n-1
	    en= en+ek
	    t_c%cex2(imin:imax-1,jmin,kmin+en) = 0.0 ! xozinf
	    t_c%cex2(imin:imax-1,jmax,kmin+en) = 0.0 ! xozsup
	  end do
	end if
      end if
    end if ! type metal
  end do

  !-------------- TRAITEMENT DE LA COMPOSANTE EY -------------
  ctemp = t_c%cey2
  igm = 1
  do i= 1,t_s%nbmetal
    imin = t_s%t_metal(1,i) 
    imax = t_s%t_metal(2,i) 
    jmin = t_s%t_metal(3,i) 
    jmax = t_s%t_metal(4,i) 
    kmin = t_s%t_metal(5,i) 
    kmax = t_s%t_metal(6,i) 
    if (t_s%t_metal(7,i) == 1) then
!    print '("coord metal pour ey =",6I4)',imin,imax,jmin,jmax,kmin,kmax
      t_c%cey2(imin:imax,jmin:jmax-1,kmin) = 0.0      
      t_c%cey2(imin,jmin:jmax-1,kmin:kmax) = 0.0      
      if (kmax>kmin) t_c%cey2(imin:imax,jmin:jmax-1,kmax) = 0.0  
      if (imax>imin) t_c%cey2(imax,jmin:jmax-1,kmin:kmax) = 0.0      
    else if (t_s%t_metal(7,i) == 0) then
!    print '("coord non metal pour ey =",6I4)',imin,imax,jmin,jmax,kmin,kmax
      if (imax-1>imin) t_c%cey2(imin+1:imax-1,jmin:jmax-1,kmin) = ctemp(imin+1:imax-1,jmin:jmax-1,kmin)      
      if (kmax-1>kmin) t_c%cey2(imin,jmin:jmax-1,kmin+1:kmax-1) = ctemp(imin,jmin:jmax-1,kmin+1:kmax-1)      
      if (kmax>kmin) t_c%cey2(imin+1:imax-1,jmin:jmax-1,kmax) = t_c%cey2(imin+1:imax-1,jmin:jmax-1,kmax)  
      if (imax>imin) t_c%cey2(imax,jmin:jmax-1,kmin+1:kmax-1) = t_c%cey2(imax,jmin:jmax-1,kmin+1:kmax-1) 
    else if (t_s%t_metal(7,i) == 2) then
      ei   = t_s%t_grille_metal(1,igm)
      ej   = t_s%t_grille_metal(2,igm)
      ek   = t_s%t_grille_metal(3,igm)
      igm=igm+1
      if (jmax<=jmin) cycle
      if (ek>0) then ! plan xoz
        if (rang==RANG_MAITRE_AFF) print '(T10," -->N°metal = ",I3," ;  ei,ej,ek = ",3I4)',i,ei,ej,ek
	n = (kmax-kmin)/ek
	if (n>0) then
	  r = (kmax-kmin) -n*ek
	  t_c%cey2(imin,jmin:jmax-1,kmin:kmax:kmax-kmin) = 0.0 ! yozinf
	  t_c%cey2(imax,jmin:jmax-1,kmin:kmax:kmax-kmin) = 0.0 ! yozinf
	  en = r/2
          if (rang==RANG_MAITRE_AFF) print '(T15,"EY, PLAN YOZ  n = ",I4,"  ;  r=",I3)',n,r
	  do j=1,n-1
	    en= en+ek
	    t_c%cey2(imin,jmin:jmax-1,kmin+en) = 0.0 ! yozinf
	    t_c%cey2(imax,jmin:jmax-1,kmin+en) = 0.0 ! yozsup
	  end do
	end if
      end if
      if (ei>0) then ! plan xoy
        if (rang==RANG_MAITRE_AFF) print '(T10," -->N°metal = ",I3," ;  ei,ej,ek = ",3I4)',i,ei,ej,ek
	n = (imax-imin)/ei
	if (n>0) then
	  r = (imax-imin) -n*ei
	  t_c%cey2(imin:imax:imax-imin,jmin:jmax-1,kmin) = 0.0 ! xoyinf
	  t_c%cey2(imin:imax:imax-imin,jmin:jmax-1,kmax) = 0.0 ! xoysup
	  en = r/2
          if (rang==RANG_MAITRE_AFF) print '(T15,"EY, PLAN XOY  n = ",I4,"  ;  r=",I3)',n,r
	  do j=1,n-1
	    en= en+ei
  	    t_c%cey2(imin+en,jmin:jmax-1,kmin) = 0.0 ! xoyinf
	    t_c%cey2(imin+en,jmin:jmax-1,kmax) = 0.0 ! xoysup
	  end do
	end if
      end if
      
    end if    
  end do
  
    
  !-------------- TRAITEMENT DE LA COMPOSANTE EZ -------------
  ctemp = t_c%cez2
  igm = 1
  do i= 1,t_s%nbmetal
    imin = t_s%t_metal(1,i) 
    imax = t_s%t_metal(2,i) 
    jmin = t_s%t_metal(3,i) 
    jmax = t_s%t_metal(4,i) 
    kmin = t_s%t_metal(5,i) 
    kmax = t_s%t_metal(6,i) 
    
    if (t_s%t_metal(7,i) == 1) then
!    print '("coord metal pour ez =",6I4)',imin,imax,jmin,jmax,kmin,kmax
      t_c%cez2(imin,jmin:jmax,kmin:kmax-1) = 0.0      
      t_c%cez2(imin:imax,jmin,kmin:kmax-1) = 0.0      
      if (imax>imin) t_c%cez2(imax,jmin:jmax,kmin:kmax-1) = 0.0      
      if (jmax>jmin) t_c%cez2(imin:imax,jmax,kmin:kmax-1) = 0.0      
    else if (t_s%t_metal(7,i) == 0) then
!    print '("coord  pour ez=",6I4)',imin,imax,jmin,jmax,kmin,kmax
      if (jmax+1>jmin) t_c%cez2(imin,jmin+1:jmax-1,kmin:kmax-1) = ctemp(imin,jmin+1:jmax-1,kmin:kmax-1)      
      if (imax+1>imin) t_c%cez2(imin+1:imax-1,jmin,kmin:kmax-1) = ctemp(imin+1:imax-1,jmin,kmin:kmax-1)
      if (imax>imin) t_c%cez2(imax,jmin+1:jmax-1,kmin:kmax-1) = ctemp(imax,jmin+1:jmax-1,kmin:kmax-1)
      if (jmax>jmin) t_c%cez2(imin+1:imax-1,jmax,kmin:kmax-1) = ctemp(imin+1:imax-1,jmax,kmin:kmax-1)
    else if (t_s%t_metal(7,i) == 2) then
      ei   = t_s%t_grille_metal(1,igm)
      ej   = t_s%t_grille_metal(2,igm)
      ek   = t_s%t_grille_metal(3,igm)
      igm=igm+1
      if (kmax<=kmin) cycle
      if (ei>0) then ! plan zox
        if (rang==RANG_MAITRE_AFF) print '(T10," -->N°metal = ",I3," ;  ei,ej,ek = ",3I4)',i,ei,ej,ek
	n = (imax-imin)/ei
	if (n>0) then
	  r = (imax-imin) -n*ei
	  t_c%cez2(imin:imax:imax-imin,jmin,kmin:kmax-1) = 0.0 ! zoxinf
	  t_c%cez2(imin:imax:imax-imin,jmax,kmin:kmax-1) = 0.0 ! zoxinf
	  en = r/2
          if (rang==RANG_MAITRE_AFF) print '(T15,"EZ, PLAN ZOX  n = ",I4,"  ;  r=",I3)',n,r
	  do j=1,n-1
	    en= en+ei
	    t_c%cez2(imin+en,jmin,kmin:kmax-1) = 0.0 ! zoxinf
	    t_c%cez2(imin+en,jmax,kmin:kmax-1) = 0.0 ! zoxsup
	  end do
	end if
      end if
      if (ej>0) then ! plan yoz
        if (rang==RANG_MAITRE_AFF) print '(T10," -->N°metal = ",I3," ;  ei,ej,ek = ",3I4)',i,ei,ej,ek
	n = (jmax-jmin)/ej
	if (n>0) then
	  r = (jmax-jmin) -n*ej
	  t_c%cez2(imin,jmin:jmax:jmax-jmin,kmin:kmax-1) = 0.0 ! yozinf
	  t_c%cez2(imax,jmin:jmax:jmax-jmin,kmin:kmax-1) = 0.0 ! yozsup
	  en = r/2
          if (rang==RANG_MAITRE_AFF) print '(T15,"EZ, PLAN YOZ  n = ",I4,"  ;  r=",I3)',n,r
	  do j=1,n-1
	    en= en+ej
	    t_c%cez2(imin,jmin+en,kmin:kmax-1) = 0.0 ! yozinf
	    t_c%cez2(imax,jmin+en,kmin:kmax-1) = 0.0 ! yozsup
	  end do
	end if
      end if
    end if    
  end do
  !-------------------------------------------------------------
  deallocate(ctemp)
end if
  
    
end subroutine milieu_volume_annule_coeff


! ==========================================================
! == Subroutine milieu_volume_precalcul                   ==
! == Calcul des permittivité, perméabilité et conductivités
! == Moyennage
! == Calcul des coefficients multiplicatifs des champs
! ==========================================================	
subroutine milieu_volume_precalcul(t_m,t_lmnd,v,t_c,t_s)

type(st_milieu_volume), intent(inout) :: t_m
type(st_liste_milieu_simple), intent(inout) :: t_lmnd
type(st_volume), intent(in)           :: v
type(st_champ), intent(inout)            :: t_c
type(st_structure), intent(in)        :: t_s

integer   :: i,j,k,l,offset
integer   :: nbx,nby,nbz
integer,dimension(1:3)  :: epsr,mur,sigmae,sigmah
type(st_milieu_simple), pointer :: t_mnd
logical   :: flagnommilieu



  ! Calcul des surfaces spécifiques à chaque cellule
  allocate(t_m%surfxy(v%ivold-1:v%ivolf+2,v%jvold-1:v%jvolf+2,1:5)); 
  allocate(t_m%surfyz(v%jvold-1:v%jvolf+2,v%kvold-1:v%kvolf+2,1:5)); 
  allocate(t_m%surfxz(v%ivold-1:v%ivolf+2,v%kvold-1:v%kvolf+2,1:5)); 
  t_m%surfxy(:,:,:) = 0.0
  t_m%surfyz(:,:,:) = 0.0
  t_m%surfxz(:,:,:) = 0.0
  call milieu_calcul_surfaces(t_m,v)

  ! Calcul des pas spatiaux spécifiques à chaque cellule
  allocate(t_m%axex(v%ivold-1:v%ivolf+2,1:3)); 
  allocate(t_m%axey(v%jvold-1:v%jvolf+2,1:3)); 
  allocate(t_m%axez(v%kvold-1:v%kvolf+2,1:3)); 
  t_m%axex(:,:) = 0.0
  t_m%axey(:,:) = 0.0
  t_m%axez(:,:) = 0.0
  call milieu_calcul_deltas(t_m,v)

  ! paramètres électromagnétiques, allocation et calcul
  if (rang==RANG_MAITRE_AFF) print '(T10,"--> Calcul des paramètres epsilon, mu, sigma en tout point... ")'
  call milieu_calcul_param_em(t_m,t_lmnd,v,t_s)
  if (rang==RANG_MAITRE_AFF) print '(T10,"--> Modifications epsilon,mu pour les fentes... ")'
  call milieu_modif_eps_mu_fente(t_m,t_s)

  ! Réglage des flags et calcul des coefficients des champs
  if (rang==RANG_MAITRE_AFF) print '(T10,"--> Calcul des coefficients multiplicatifs des champs ")'
  call milieu_volume_calcul_coeff(t_m, v,t_c,t_s)
  
  ! annulation des coefficients dans le cas de la présence de métal
  ! et si le milieu est inhomogène.
  ! libération mémoire
  !call milieu_volume_deallocate(t_m)
  deallocate( t_m%surfxy); 
  deallocate( t_m%surfyz); 
  deallocate( t_m%surfxz); 
  deallocate(t_m%axex)
  deallocate(t_m%axey)
  deallocate(t_m%axez)

  if (rang==RANG_MAITRE_AFF) print '(T10,"--> Annulation des coefficients sur les conducteurs parfaits ")'
call milieu_volume_annule_coeff(t_c,t_s,v)
call milieu_rectif_coeff_fente(t_m,t_s,t_c,v)

end subroutine milieu_volume_precalcul



! ==========================================================
! == Subroutine milieu_volume_deallocate                  ==
! == libération de la mémoire allouée                     ==
! ==========================================================
subroutine milieu_volume_deallocate(t_m)

implicit none
				
type(st_milieu_volume), intent(inout) :: t_m

if (t_m%flag_eps) then
  deallocate(t_m%epsx)
  deallocate(t_m%epsy)
  deallocate(t_m%epsz)
end if
if (t_m%flag_mu) then
  deallocate(t_m%mux)
  deallocate(t_m%muy)
  deallocate(t_m%muz)
end if
if (t_m%flag_eps_sigmae) then
  deallocate(t_m%sigex)
  deallocate(t_m%sigey)
  deallocate(t_m%sigez)
end if
if (t_m%flag_mu_sigmah) then
  deallocate(t_m%sighx)
  deallocate(t_m%sighy)
  deallocate(t_m%sighz)
end if
				
end subroutine milieu_volume_deallocate


! ==========================================================
! == Subroutine milieu_volume_affichage                   ==
! == Affichage de résultats dans le terminal              ==
! ==========================================================
subroutine milieu_volume_affichage()

implicit none
				
end subroutine milieu_volume_affichage


! ==========================================================
! == Fin du module                                        ==
! ==========================================================
end module mfdtd_milieu_volume	



