!////////////////////////////////////////////////////////////////////////
!// Name:        mfdtd_structure_fil_tube.f90
!// Purpose:     Traitement du formalisme des fils minces améliorés  ==
!// Author:      Christophe Guiffaut
!// Modified by:
!// Created  :     Mars 2003
!// Lieu     : XLIM - Faculté des sciences - 123 Av. Albert Thomas - Limoges
!// Copyright:   (c) CNRS, Université de Limoges
!// Licence  : CECILL-C - Octobre 2006
!////////////////////////////////////////////////////////////////////////


module mfdtd_structure_fil_tube

use m_constantes

use m_constantes_scientifiques, only : PI, EPSILON_0, MU_0, CVIDE

use mfdtd_volume, only : st_volume
use mfdtd_champ, only : st_champ

use mfdtd_milieu_volume, only : st_milieu_volume, &
				 milieu_volume_extrait_sige, &
                                 milieu_volume_extrait_epsilon

implicit none
integer, private, parameter :: X = 1
integer, private, parameter :: Y = 2
integer, private, parameter :: Z = 3


! ==========================================================
! == Definitions des differents types                     ==
! ==========================================================

contains
! ==========================================================
! == Subroutine maillage_disque                           ==
! == Maillage d'un disque sur Nc * Nc cellules            ==
! == Retourne un tableau de taille Nc * Nc   dont         ==
! nc : diamètre disque en nombre de cellules,
! si : valeur multiplié à la surface interne d'une maille
! se : valeur multiplié à la surface externe d'une maille
! s2d: tableau 2d d'entier contenant la surface de
! rc : rayon en nombre de maille (argument optionnel)
!      si non présent, le rayon correspondra à nc/2
! chaque maille qui est à l'intérieur du disque
! ==========================================================
function maillage_disque(nc,si, s2d, rc)
implicit none

!=== Declaration des arguments ===
integer, intent(in)   :: nc ! nombre pair, correspond au diamètre du disq
real,    intent(in)   :: si
real, dimension(1:nc,1:nc), intent(out) :: s2d
real, intent(in), optional  :: rc
real  :: maillage_disque ! retourne la surface du disque discrétisé norma
!=== Declaration des variables ===
integer  :: i,j
integer  :: nc_2
real     :: r_en_cell
real     :: dm11, dm22, dm21, dm12
real     :: dx, dy,dx2, dy2
!=== debut des traitements ===

nc_2 = nc / 2 ! correspond au rayon du disque
r_en_cell = nc_2
if (PRESENT(rc)) r_en_cell = rc
if (r_en_cell > nc_2) then
  print '("ERREUR dans la subroutine maillage_disque, Arret !!!")'
  stop
end if
! Maillage du cadran positif du disque
do j=0, nc_2 -1
  do i=0, nc_2 -1
    dm11 = SQRT(real(i)**2 + real(j)**2)    ! sommet de la maille le + pr
    dm22 = SQRT(real(i+1)**2 + real(j+1)**2) ! sommet de la maille le + l

    if (dm22 <= r_en_cell) then !maille à l'intérieur du disque
      s2d(i+nc_2+1,j+nc_2+1) = si
    else if(dm11 >= r_en_cell) then !maille à l'extérieur du disque
      s2d(i+nc_2+1,j+nc_2+1) = 0.0
    else ! maille sur la frontière du disque
      ! calcul de l'aire de la partie de la maille à l'intérieur du disqu
      dm21 = SQRT(real(i+1)**2 + real(j)**2)
      dm12 = SQRT(real(i)**2 + real(j+1)**2)

      if (dm21 >=r_en_cell .and. dm12 >=r_en_cell) then
        ! l'aire intérieure au disque est un triangle
        dy = SQRT(r_en_cell**2 - real(i)**2) - j
        dx = SQRT(r_en_cell**2 - real(j)**2) - i
 s2d(i+nc_2+1,j+nc_2+1) = dx * dy * 0.5 * si

      else if (dm21 <=r_en_cell .and. dm12 <=r_en_cell) then
        ! l'aire extérieure au disque est un triangle
        dy = j+1 - SQRT(r_en_cell**2 - real(i+1)**2)
        dx = i+1 - SQRT(r_en_cell**2 - real(j+1)**2)
 s2d(i+nc_2+1,j+nc_2+1) = (1 - dx*dy*0.5) * si

      else if (dm21 > r_en_cell .and. dm12 < r_en_cell) then
        ! l'aire intérieure au disque est un trapèze
        dx  = SQRT(r_en_cell**2 - real(j)**2) - i
        dx2 = SQRT(r_en_cell**2 - real(j+1)**2) - i
 s2d(i+nc_2+1,j+nc_2+1) = (dx*0.5 + dx2*0.5)*si

      else if (dm21 < r_en_cell .and. dm12 > r_en_cell)  then
        ! l'aire intérieure au disque est un trapèze
        dy  = SQRT(r_en_cell**2 - real(i)**2) - j
        dy2 = SQRT(r_en_cell**2 - real(i+1)**2) - j
 s2d(i+nc_2+1,j+nc_2+1) = (dy*0.5 + dy2*0.5)*si

      end if
    end if
  end do
end do

! Maillage du 2ème cadran positif du disque
do i=0, nc_2 -1
  s2d(nc_2-i,nc_2+1:nc)  = s2d(i+nc_2+1,nc_2+1:nc)
end do
! Maillage du 1/2 cercle inférieur
do j=0, nc_2-1
  s2d(1:nc,nc_2-j)  = s2d(1:nc,nc_2+1+j)
end do

! vérification de l'intégrité du disque
maillage_disque = 0.0

do j=1, nc
do i=1, nc
  maillage_disque = maillage_disque + s2d(i,j)
end do; end do
maillage_disque = maillage_disque / (PI*r_en_cell**2)
end function maillage_disque

! ==========================================================
! == Subroutine cellule_du_disque                         ==
! == Evalue les cellules touchées par le disque  du fil   ==
! uc, vc : recoit les valeur min et max des coordonnées
! des cellules touchées par le disque (en nombre de cellules)
! ufil, vfil : position du fil en nombre de cellule dans le plan
! transverse.
! r : rayon du disque
! du, dv : pas spatiaux suivant u et v
! ==========================================================
subroutine cellule_du_disque(uc,vc,uvfil,r,duv)
implicit none

!=== Declaration des arguments ===
integer, dimension(2), intent(out)   :: uc, vc
real,    intent(in)   ::  r
real, dimension(2),    intent(in)    :: uvfil,duv
!=== Declaration des variables ===

!=== debut des traitements ===
! imin
uc(1) = INT(uvfil(1)-r/duv(1)+0.5)
! imax
uc(2) = INT(uvfil(1)+r/duv(1)+0.5)
! jmin
vc(1) = INT(uvfil(2)-r/duv(2)+0.5)
! jmax
vc(2) = INT(uvfil(2)+r/duv(2)+0.5)
end subroutine cellule_du_disque

! ==========================================================
! == Subroutine distribution_disque                       ==
! == Evalue les cellules touchées par le disque  du fil   ==
! uc, vc : les valeur min et max des coordonnées
! des cellules touchées par le disque (en nombre de cellules)
! ufil, vfil : position du fil en nombre de cellule dans le plan
! transverse.
! r : rayon réel du disque
! du, dv : pas spatiaux réels suivant u et v
! Nc : Nombre de pixels maillant le disque sur son diamètre
! ==========================================================
function distribution_disque(uc,vc,uvfil,r,duv, Nc,s2d) result(sc2d)
implicit none

!=== Declaration des arguments ===
integer, dimension(2), intent(in)   :: uc, vc 
real,    intent(in)   :: r 
real, dimension(2), intent(in)      :: duv,uvfil 
integer, intent(in)   :: Nc 
real, dimension(1:Nc,1:Nc)  :: s2d
real, dimension(uc(1):uc(2),vc(1):vc(2))  :: sc2d

!=== Declaration des variables ===
integer  :: i, j
real  :: uo,vo, ui, vi
real  :: diff, dum, dvm
integer, dimension(:), allocatable :: ut, vt
real, dimension(:), allocatable :: ufact, vfact

!=== debut des traitements ===

! On évalue pour chaque pixel du disque, à quelle cellule fdtd elle
! appartient

! origine suivant u et v
uo = uvfil(1) - r / duv(1)
vo = uvfil(2) - r / duv(2)
dum = 2 * r / (Nc * duv(1)) ! pas de maille du disque normalisé au pas fdtd
dvm = 2 * r / (Nc * duv(2))
allocate(ut(Nc)) ; ut=0
allocate(vt(Nc)) ; vt=0
allocate(ufact(Nc)) ; ufact=0.0
allocate(vfact(Nc)) ; vfact=0.0

!--- direction u ---
do i=1, Nc
  ui = uo + (i-1) * dum
  do j=uc(1), uc(2)
    ! position de la maille du disque par rapport aux cellules fdtd
    if (ui < (j*1.0+0.5)) then
      ut(i) = j ! coordonnée de la cellule fdtd suivant u
      ufact(i) = 1 ! proportion de la maille du disque appartenant à la cellule
      diff = j*1.0+0.5 - ui 
      if (diff<dum)  ufact(i) = diff/dum
      exit
    end if
  end do
end do

!--- direction v ---
do i=1, Nc
  vi = vo + (i-1) * dvm
  do j=vc(1), vc(2)
    ! position de la maille du disque par rapport aux cellules fdtd
    if (vi < (j*1.0+0.5)) then
      vt(i) = j ! coordonnée de la cellule fdtd suivant v
      vfact(i) = 1 ! proportion de la maille du disque appartenant à la cellule
      diff = j*1.0+0.5 - vi 
      if (diff<dvm)  vfact(i) = diff/dvm
      exit
    end if
  end do
end do

! sommation des surfaces
sc2d(:,:) = 0.0
do j=1, Nc
do i=1, Nc
 sc2d(ut(i),vt(j)) = sc2d(ut(i),vt(j)) +  s2d(i,j) * ufact(i)*vfact(j)
 
 ! cas particulier ou une maille du disque chevauchent 2 ou 4 cellules fdtd
 if ((ufact(i) < 0.9999).and.(vfact(j) < 0.9999)) &
   sc2d(ut(i)+1,vt(j)+1) = sc2d(ut(i)+1,vt(j)+1) + s2d(i,j) * (1-ufact(i))*(1-vfact(j))
 if (ufact(i) < 0.9999) &
   sc2d(ut(i)+1,vt(j)) = sc2d(ut(i)+1,vt(j)) + s2d(i,j) * (1-ufact(i))*(vfact(j))
 if (vfact(j) < 0.9999) &
   sc2d(ut(i),vt(j)+1) = sc2d(ut(i),vt(j)+1) + s2d(i,j) * (ufact(i))*(1-vfact(j)) 
 
end do
end do

deallocate(ut,vt); 
deallocate(ufact,vfact); 

end function distribution_disque

! ==========================================================
! == Subroutine permittivite_moy                          ==
! == Jonction Toron - Toron Fil à Fil                     ==
! ==========================================================
real function permittivite_cell(uc,vc,l,dir,surface,tm)
implicit none
!=== Declaration des arguments ===
integer, dimension(2), intent(in)   :: uc, vc ! coordonnées transverse
integer,               intent(in)   :: dir,l  ! direction et coordonnée longitudinale
real, dimension(uc(1):uc(2),vc(1):vc(2)) :: surface
type(st_milieu_volume), intent(in)  :: tm

!=== Declaration des variables locales ===
integer :: i,j,k
real :: epsilon_s,eps

!== Début des traitements ==
permittivite_cell = 1.0;
epsilon_s = 0.0
  k = 1
if (dir == X) then
  do j = vc(1), vc(2)  
  do i = uc(1), uc(2)  
     eps = milieu_volume_extrait_epsilon(tm,l,i,j,dir) 
     epsilon_s =   epsilon_s + eps * surface(i,j)   
     k = k+1
  end do
  end do
else if (dir == Y) then
  do j = vc(1), vc(2)  
  do i = uc(1), uc(2)  
     eps = milieu_volume_extrait_epsilon(tm,j,l,i,dir)    
     epsilon_s =   epsilon_s + eps * surface(i,j)   
     k = k+1
  end do
  end do

else
  do j = vc(1), vc(2)  
  do i = uc(1), uc(2)  
     eps = milieu_volume_extrait_epsilon(tm,i,j,l,dir)    
     epsilon_s =   epsilon_s + eps * surface(i,j)   
     k = k+1
  end do
  end do
end if

permittivite_cell = epsilon_s / SUM(surface)
end function permittivite_cell


end module mfdtd_structure_fil_tube


