!/////////////////////////////////////////////////////////////////////////////
!// Name:        m_volume.f90
!// Purpose:  contient et traite les paramètres tels que les pas spatiaux et 
!             le pas temporel, la durée de simulation, le milieu par défaut 
!             et la taille du volume de calcul...
!      Simulateur FDTD
!// Author:      Christophe Guiffaut
!// Modified by:  Christophe Guiffaut
!// Created  :     Novembre 2001
!// Lieu     : XLIM - Faculté des sciences - 123 Av. Albert Thomas - Limoges
!// Copyright:   (c) CNRS, Université de Limoges
!// Licence  : CECILL-C - Octobre 2006
!/////////////////////////////////////////////////////////////////////////////

! ==========================================================
! == Module définissant la taille des cellules            ==
! == élémentaires et du volume de calcul le pas temporel  ==
! == et les bornes des boucles de champ sont aussi        ==
! == évaluées                                             ==
! == Module module_volume                                 ==
! == Type st_maillage_variable                            ==
! == Type st_volume                                       ==
! == Subroutine volume_inidefaut                          ==
! == Subroutine volume_lecture                            ==
! == Subroutine volume_init                     ==
! == Subroutine volume_precalcul                          ==
! == Suborutine volume_calcul                             ==
! == Suborutine volume_affichage                          ==
! == Subroutine volume_postcalcul                         ==
! == Subroutine volume_maillage                           ==
! == Subroutine volume_maillage_un_axe                    ==
! == Pas de variables globales...                         ==
!===========================================================
module mfdtd_volume

use m_constantes
use m_constantes_scientifiques, only : CVIDE, MU_0, EPSILON_0
use m_entree, only : st_fichier, lineread, &
                  lecture_stop, &
		  ctrl_nb_mot_chaine,&
		  nb_mot_chaine
use m_commun_identifiant
use mfdtd_identifiant

implicit none

real, parameter :: CDTMAX_DEFAUT = 0.97
integer, parameter :: VOL1D_EZHY = 1
integer, parameter :: VOL1D_EYHZ = 2
integer, parameter :: VOL2D_TM = 1
integer, parameter :: VOL2D_TE = 2


interface volume_pasE_local
  module procedure  volume_pasE_local1,volume_pasE_local_ijk
end interface volume_pasE_local

interface volume_pasH_local
  module procedure  volume_pasH_local1,volume_pasH_local_ijk
end interface volume_pasH_local


type st_maillage_nu
  integer                    :: nb_pas
  integer, dimension(:),pointer :: liste_cell
  integer, dimension(:),pointer :: fr     ! facteur de rafinement
  real, dimension(:),pointer :: liste_pas
  
  real,dimension(:),pointer  :: pas_e,pas_h
  real,dimension(:),pointer  :: c_e,c_h
  
  real                       :: dmin
  integer                    :: ncell
end type st_maillage_nu

! ==========================================================
! == Déclaration des types                                ==
! == Type st_volume                                       ==
! ==========================================================
type st_volume
  integer                     :: type_volume,type_polar
  real                        :: fmax, vmin ! freq max de travail et vitesse min
  
  !--> schéma exponentielle pour la prise en compte des pertes
  logical                     :: flag_schema_expon 
  
  !--> flag pour le moyennage aux interfaces entre milieu
  logical                     :: flag_nonmoy_eps,flag_nonmoy_mu
  logical                     :: flag_nonmoy_sigmae,flag_nonmoy_sigmah
  
  ! Parallélisme MPI
  ! Nombre de processus dans chaque direction
  integer, dimension(3) :: Npxyz
  
  !--> Nombre de cellules,pas spatiaux et nbre d'itérations temporelles
  integer                     :: nx, ny, nz
  integer,dimension(3)        :: nxyz ! meme chose que ci-dessus
  real                        :: celx, cely, celz ! pas spatiaux par défaut
  real, dimension(3)          :: dxyz ! pas spatiaux réunis
  integer :: ntinc              ! nombre d'iterations temporelles

  ! -- Variables qui peuvent la cible de pointeurs --
  real, pointer :: dt, tlimit ! paramètres temporels
  real :: cdtmax ! paramètres temporels
  integer :: type_duree ! type d'évaluation de la durée totale
  
  integer :: temps_cpu  ! temps cpu en seconde limite fixé par l'utilisateur
  
  integer :: nxg, nyg, nzg      ! nombre de cellules totales (y compris dans les pmls)
  real, dimension(3)  :: Lxyz
  
  ! ORIGINE
  integer, dimension(3)       :: origine_cell
  real, dimension(3)          :: origine
  logical                     :: flag_coordreel
  integer :: ivold, ivolf ! debut et fin du volume de calcul suivant x
  integer :: jvold, jvolf ! debut et fin du volume de calcul suivant y
  integer :: kvold, kvolf ! debut et fin du volume de calcul suivant z

  integer, dimension(1:6) :: borne_volume
  
  real    :: mu_r, epsilon_r, sigma_e, sigma_h
  real    :: mu, epsilon
  real, dimension(:), pointer :: x, y, z, xo, yo, zo ! coordonnées axiales pour le maillage
  
  !----------------------------------------------------------------------------
  !--> Maillage non uniforme
  logical :: flag_maillage_nu
  type(st_maillage_nu),dimension(3) :: mnu
  !----------------------------------------------------------------------------
end type st_volume

contains



! ==========================================================
! == Subroutine maillage_NU_initdefaut                    ==
! ==========================================================	
subroutine maillage_NU_initdefaut(m)

type (st_maillage_nu), intent(out) :: m

m%nb_pas = 0
nullify(m%liste_cell);
nullify(m%fr);
nullify(m%liste_pas);
nullify(m%pas_e);
nullify(m%pas_h);
nullify(m%c_e);
nullify(m%c_h);
end subroutine maillage_NU_initdefaut


! ==========================================================
! == Subroutine maillage_NU_copy                          ==
! ==========================================================	
subroutine maillage_NU_copy(vin,vout)
! === Déclaration des arguments ===
type(st_maillage_nu), intent(in)       :: vin
type(st_maillage_nu), intent(out)      :: vout
! ===   Début des traitements   ===
vout%nb_pas=vin%nb_pas
if (vin%nb_pas>0) then
  allocate(vout%liste_cell(vin%nb_pas))
  allocate(vout%fr(vin%nb_pas))
  allocate(vout%liste_pas(vin%nb_pas))
  vout%liste_cell=vin%liste_cell
  vout%fr        =vin%fr
  vout%liste_pas =vin%liste_pas
end if
end subroutine maillage_NU_copy

! ==========================================================
! == Subroutine maillage_NU_copy                          ==
! ==========================================================	
subroutine maillage_NU_deallocate(m)
! === Déclaration des arguments ===
type(st_maillage_nu), intent(inout)       :: m

! ===   Début des traitements   ===
if (m%nb_pas>0) then
  deallocate(m%liste_cell)
  deallocate(m%fr)
  deallocate(m%liste_pas )
end if
end subroutine maillage_NU_deallocate

! ==========================================================
! == Subroutine volume_initdefaut                         ==
! == Initialisation par défaut des variables contenant    ==
! == les paramètres définis par l'utilisateur             ==
! == dans le fichier d'entrée                             ==
! ==========================================================	
subroutine volume_initdefaut(t_vol)

type(st_volume ), intent(inout)       :: t_vol

integer :: i

t_vol%flag_schema_expon = .false.
t_vol%flag_nonmoy_eps = .false.
t_vol%flag_nonmoy_mu = .false.
t_vol%flag_nonmoy_sigmae = .false.
t_vol%flag_nonmoy_sigmah = .false.


t_vol%type_volume = VOL_3D
t_vol%type_polar  =0
t_vol%fmax = 1e9;
t_vol%vmin = 3e8;

t_vol%nx   = 30; t_vol%ny   = 30; t_vol%nz   = 30
t_vol%Npxyz = 1; ! nbre de processus dans chaque direction
t_vol%nxyz = 30;
t_vol%celx =  1.0e-2; t_vol%cely =  1.0e-2; t_vol%celz =  1.0e-2
t_vol%origine_cell=-1;
t_vol%origine = 0.0;
t_vol%dxyz = 1.0e-2;
t_vol%ivold = 1; t_vol%ivolf = t_vol%nx
t_vol%jvold = 1; t_vol%jvolf = t_vol%ny
t_vol%kvold = 1; t_vol%kvolf = t_vol%nz

t_vol%nxg = t_vol%ivolf - t_vol%ivold + 1
t_vol%nyg = t_vol%jvolf - t_vol%jvold + 1
t_vol%nzg = t_vol%kvolf - t_vol%kvold + 1
t_vol%Lxyz(1) = t_vol%nx * t_vol%dxyz(1)
t_vol%Lxyz(2) = t_vol%ny * t_vol%dxyz(2)
t_vol%Lxyz(3) = t_vol%nz * t_vol%dxyz(3)


t_vol%borne_volume(:) = 0

t_vol%cdtmax    = CDTMAX_DEFAUT
t_vol%mu_r      = 1.0
t_vol%epsilon_r = 1.0 
t_vol%mu        = MU_0
t_vol%epsilon   = EPSILON_0

t_vol%sigma_e   = 0.0
t_vol%sigma_h   = 0.0


allocate(t_vol%tlimit)
allocate(t_vol%dt)
t_vol%dt = t_vol%cdtmax/(CVIDE*SQRT(1./t_vol%celx**2 + &
                                    1./t_vol%cely**2 + &
                                    1./t_vol%celz**2))

t_vol%tlimit    = t_vol%dt 
t_vol%type_duree= VOL_DUREE_MANUEL
t_vol%ntinc = NINT(t_vol%tlimit / t_vol%dt)
t_vol%temps_cpu = 0

nullify(t_vol%x); nullify(t_vol%y); nullify(t_vol%z);
nullify(t_vol%xo);nullify(t_vol%yo);nullify(t_vol%zo);

t_vol%flag_coordreel = .false.
!------------------------------------------------------------------------------
!--> Maillage non uniforme
t_vol%flag_maillage_nu=.false.
do i=1,3
  call maillage_NU_initdefaut(t_vol%mnu(i))
end do

!------------------------------------------------------------------------------

end subroutine volume_initdefaut

! ==========================================================
! == Subroutine volume_copy                               ==
! ==========================================================	
subroutine volume_copy(vin,vout)
! === Déclaration des arguments ===
type(st_volume ), intent(in)       :: vin
type(st_volume ), intent(out)      :: vout

integer :: i
! ===   Début des traitements   ===

vout%type_volume = vin%type_volume
vout%type_polar  = vin%type_polar
vout%fmax = vin%fmax;
vout%vmin = vin%vmin;

vout%flag_schema_expon  = vin%flag_schema_expon

vout%flag_nonmoy_eps    = vin%flag_nonmoy_eps;
vout%flag_nonmoy_mu     = vin%flag_nonmoy_mu;
vout%flag_nonmoy_sigmae = vin%flag_nonmoy_sigmae;
vout%flag_nonmoy_sigmah = vin%flag_nonmoy_sigmah;
vout%nx=vin%nx; vout%ny=vin%ny;vout%nz=vin%nz;
vout%nxyz=vin%nxyz
vout%Npxyz=vin%Npxyz
vout%origine        = vin%origine
vout%origine_cell   = vin%origine_cell
vout%flag_coordreel = vin%flag_coordreel

vout%celx=vin%celx;vout%cely=vin%cely;vout%celz=vin%celz;
vout%dxyz = vin%dxyz;vout%ntinc=vin%ntinc
vout%lxyz = vin%lxyz;

allocate(vout%tlimit)
allocate(vout%dt)
vout%dt =vin%dt; vout%tlimit = vin%tlimit
vout%cdtmax=vin%cdtmax
vout%type_duree = vin%type_duree
vout%temps_cpu  = vin%temps_cpu

vout%nxg=vin%nxg;vout%nyg=vin%nyg;vout%nzg=vin%nzg;
vout%ivold = vin%ivold;vout%ivolf=vin%ivolf
vout%jvold = vin%jvold;vout%jvolf=vin%jvolf
vout%kvold = vin%kvold;vout%kvolf=vin%kvolf

vout%borne_volume=vin%borne_volume
vout%mu_r     =vin%mu_r;
vout%epsilon_r=vin%epsilon_r;
vout%sigma_e=vin%sigma_e;
vout%sigma_h=vin%sigma_h
vout%mu=vin%mu;vout%epsilon=vin%epsilon

!------------------------------------------------------------------------------
!--> Maillage non uniforme
vout%flag_maillage_nu=vin%flag_maillage_nu
do i=1,3
  call maillage_NU_copy(vin%mnu(i),vout%mnu(i))
end do
!------------------------------------------------------------------------------


!x, y, z, xo, yo, zo ! coordonnées axiales pour le maillage

end subroutine volume_copy

! ==========================================================
! == Subroutine volume_deallocate                         ==
! ==========================================================	
subroutine volume_deallocate(v)
type(st_volume ), intent(inout)       :: v

integer :: i

deallocate(v%dt,v%tlimit)

if (associated(v%x)) deallocate(v%x,v%y, v%z, v%xo, v%yo,v%zo)

do i=1,3
  call maillage_NU_deallocate(v%mnu(i))
end do

end subroutine volume_deallocate

! ==========================================================
! == Subroutine volume_lecture                            ==
! == Lecture dans le fichier d'entrée...                  ==
!===========================================================	
subroutine volume_lecture(t_fich, t_vol)

! === Déclaration des arguments ===
type(st_fichier), intent(inout)       :: t_fich
type(st_volume ), intent(inout)       :: t_vol

! ===  Déclaration des variables  ===
character (len=LEN_MOT_CLE)  :: premiermot,corbeille,ch,ch1
character (len=100),dimension(0:2) :: chduree
integer  :: l, status,type_duree,nbmot
integer  :: i,n
integer  :: l1,l2
real     :: f1,f2
logical  :: flaglxyz,flag_duree_totale
integer,dimension(:),pointer  :: liste_cell,fr
real,dimension(:),pointer  :: liste_pas

! ===   Début des traitements   ===
call volume_initdefaut(t_vol)
type_duree = VOL_DUREE_MANUEL
flag_duree_totale=.false.

! -- Il faut ajouter case('maillage_variable') ... --
flaglxyz = .false.
do
  premiermot = lineread(t_fich)
  l = LEN_TRIM(premiermot)
  select case(premiermot(1:l))
    case('fmax') 
      status = ctrl_nb_mot_chaine(t_fich,t_fich%chaine,2)
      read(t_fich%chaine,*,ERR=999) corbeille, t_vol%fmax
    case('vmin') 
      status = ctrl_nb_mot_chaine(t_fich,t_fich%chaine,2)
      read(t_fich%chaine,*,ERR=999) corbeille, t_vol%vmin
    case('espace') 
      status = ctrl_nb_mot_chaine(t_fich,t_fich%chaine,2,3)
      read(t_fich%chaine,*,ERR=999) corbeille,ch
      if (ch=='2D') then 
        read(t_fich%chaine,*) corbeille,ch,ch1
        t_vol%type_volume = VOL_2D
	if (ch1=='TM') then; t_vol%type_polar =VOL2D_TM;
	else if (ch1=='TE') then; t_vol%type_polar =VOL2D_TE;
	else; call lecture_stop(t_fich);end if;
      else if (ch=='1D') then
        t_vol%type_volume = VOL_1D
	if (ch1=='EZHY') then; t_vol%type_polar =VOL1D_EZHY;
	else if (ch1=='EYHZ') then; t_vol%type_polar =VOL1D_EYHZ;
	else; call lecture_stop(t_fich);end if;
      else
        t_vol%type_volume = VOL_3D
      end if
    case(BV_NXNYNZ) 
      status = ctrl_nb_mot_chaine(t_fich,t_fich%chaine,4)
      read(t_fich%chaine,*,ERR=999) corbeille, t_vol%nx, t_vol%ny, t_vol%nz
      t_vol%nxyz(1) = t_vol%nx;
      t_vol%nxyz(2) = t_vol%ny;
      t_vol%nxyz(3) = t_vol%nz;
    case(BV_NP_XYZ) 
      status = ctrl_nb_mot_chaine(t_fich,t_fich%chaine,4)
      read(t_fich%chaine,*,ERR=999) corbeille, t_vol%Npxyz(1:3)
    case(BV_LXLYLZ) 
      status = ctrl_nb_mot_chaine(t_fich,t_fich%chaine,4)
      read(t_fich%chaine,*,ERR=999) corbeille, t_vol%lxyz(1:3)
      flaglxyz = .true.
      
    case(BV_ORIGINE_CELL)
      status = ctrl_nb_mot_chaine(t_fich,t_fich%chaine,4)
      read(t_fich%chaine,*,ERR=999) corbeille,t_vol%origine_cell(1:3) 
      
    case(BV_ORIGINE_REELLE)
      status = ctrl_nb_mot_chaine(t_fich,t_fich%chaine,4)
      read(t_fich%chaine,*,ERR=999) corbeille,t_vol%origine(1:3) 

    case(BV_DXDYDZ)
      status = ctrl_nb_mot_chaine(t_fich,t_fich%chaine,4)
      read(t_fich%chaine,*,ERR=999) corbeille, t_vol%celx, t_vol%cely, t_vol%celz
      t_vol%dxyz(1) = t_vol%celx;t_vol%dxyz(2) = t_vol%cely
      t_vol%dxyz(3) = t_vol%celz

    case(BV_CDTMAX)
      status = ctrl_nb_mot_chaine(t_fich,t_fich%chaine,2)
      read(t_fich%chaine,*,ERR=999) corbeille, t_vol%cdtmax
    case(BV_DUREE_TOTAL,'duree_total')
      status = ctrl_nb_mot_chaine(t_fich,t_fich%chaine,2)
      read(t_fich%chaine,*,ERR=999) corbeille, t_vol%tlimit
      flag_duree_totale = .true.
    case(BV_DUREE)
      status = ctrl_nb_mot_chaine(t_fich,t_fich%chaine,2)
      read(t_fich%chaine,*,ERR=999) corbeille,ch
      if (ch==BV_DUREE_AUTO) then
        type_duree = VOL_DUREE_AUTO
      else if (ch==BV_DUREE_SEMIAUTO) then
        type_duree = VOL_DUREE_SEMIAUTO      
      else
        call lecture_stop(t_fich)      
      end if
    case(BV_TEMPS_CPU)
      status = ctrl_nb_mot_chaine(t_fich,t_fich%chaine,2)
      read(t_fich%chaine,*,ERR=999) corbeille, ch
      l = len_trim(ch)
      
    case (BV_SCHEMA_EXPONENT)
      status = ctrl_nb_mot_chaine(t_fich,t_fich%chaine,1)
      t_vol%flag_schema_expon = .true.
    case (BV_SCHEMA_LINEAIRE)
      status = ctrl_nb_mot_chaine(t_fich,t_fich%chaine,1)
      t_vol%flag_schema_expon = .false.
    
    case (ID_COORD_REEL)
      status = ctrl_nb_mot_chaine(t_fich,t_fich%chaine,1)
      t_vol%flag_coordreel = .true.
    case (ID_COORD_CELL)
      status = ctrl_nb_mot_chaine(t_fich,t_fich%chaine,1)
      t_vol%flag_coordreel = .false.
      
    !----------------------------------------------------------------
    !--> maillage non uniforme
    case (BV_MAILLAGE_NU_X,BV_MAILLAGE_NU_Y,BV_MAILLAGE_NU_Z)
      nbmot = nb_mot_chaine(t_fich%chaine)
      n=(nbmot-1)/2
      if (n*2/=(nbmot-1)) then
        print '("ERREUR, Nbre de valeurs incorrect!!!")'
        call lecture_stop(t_fich)
      end if
      t_vol%flag_maillage_nu=.true.
      allocate(liste_pas(n))
      allocate(liste_cell(n))
      allocate(fr(n))
      read(t_fich%chaine,*) corbeille,(liste_pas(i),liste_cell(i),i=1,n)
      ! Vérification que les rapports sont entiers (d'un pas à un autre)
      fr = 1
      do i=2,n
        f1=liste_pas(i)  /liste_pas(i-1)
        f2=liste_pas(i-1)/liste_pas(i)
	l1=nint(f1)
	l2=nint(f2)
        if ((liste_pas(i)>=liste_pas(i-1)).and. (abs(l1*1.0-f1)>0.01).or. &
	    (liste_pas(i)<=liste_pas(i-1)).and. (abs(l2*1.0-f2)>0.01)) then
          print '("ERREUR, Facteur de rafinement non entier entre les pas spatiaux n°",I4," et n°",I4," !!!")',i-1,i
          call lecture_stop(t_fich)
	end if
	if (l1>=1.and. (abs(l1*1.0-f1)<0.01)) then 
	  fr(i) = -l1 ! augmentation à droite du pas de maillage
	else
	  fr(i) = +l2
	end if
      end do
      if (premiermot(1:l)==BV_MAILLAGE_NU_X) then
        i=1
      else if (premiermot(1:l)==BV_MAILLAGE_NU_Y) then
        i=2
      else
        i=3
      end if
      t_vol%mnu(i)%nb_pas=n
      t_vol%mnu(i)%liste_pas=>liste_pas
      t_vol%mnu(i)%liste_cell=>liste_cell
      t_vol%mnu(i)%fr=>fr
    !----------------------------------------------------------------
    case(BV_EPSILON_R)
      status = ctrl_nb_mot_chaine(t_fich,t_fich%chaine,2)
      read(t_fich%chaine,*) corbeille, t_vol%epsilon_r
    case(BV_MU_R)
      status = ctrl_nb_mot_chaine(t_fich,t_fich%chaine,2)
      read(t_fich%chaine,*) corbeille, t_vol%mu_r
    case(BV_SIGMA_E)
      status = ctrl_nb_mot_chaine(t_fich,t_fich%chaine,2)
      read(t_fich%chaine,*) corbeille, t_vol%sigma_e
    case(BV_SIGMA_H)
      status = ctrl_nb_mot_chaine(t_fich,t_fich%chaine,2)
      read(t_fich%chaine,*) corbeille, t_vol%sigma_h
    case('non_moyennage_permittivite') 
      status = ctrl_nb_mot_chaine(t_fich,t_fich%chaine,1)
      t_vol%flag_nonmoy_eps = .true.
    case('non_moyennage_permeabilite') 
      status = ctrl_nb_mot_chaine(t_fich,t_fich%chaine,1)
      t_vol%flag_nonmoy_mu = .true.
    case('non_moyennage_conductivite_electrique') 
      status = ctrl_nb_mot_chaine(t_fich,t_fich%chaine,1)
      t_vol%flag_nonmoy_sigmae = .true.
    case('non_moyennage_conductivite_magnetique') 
      status = ctrl_nb_mot_chaine(t_fich,t_fich%chaine,1)
      t_vol%flag_nonmoy_sigmah = .true.
    case default
      if (t_fich%nomfinbloc == premiermot) exit ! fin du bloc
      print *,'Mot clé non reconnu dans le bloc volume_de_calcul : ', TRIM(premiermot)
      call lecture_stop(t_fich)
  end select 
enddo		

if (.not.flaglxyz) then
  t_vol%Lxyz(1) = t_vol%nx * t_vol%dxyz(1)
  t_vol%Lxyz(2) = t_vol%ny * t_vol%dxyz(2)
  t_vol%Lxyz(3) = t_vol%nz * t_vol%dxyz(3)
end if

!if (any(t_vol%origine/t_vol%dxyz>nxyz+1).or.any(t_vol%origine/t_vol%dxyz<-1)) then
!  print '("ERREUR, Origine réelle mal défini !!!")'
!  call lecture_stop(t_fich)
!end if

if (flag_duree_totale) then
  t_vol%type_duree=type_duree
  if (type_duree==VOL_DUREE_AUTO) then
    t_vol%tlimit = t_vol%dt
  end if
else
  t_vol%type_duree = VOL_DUREE_AUTO
end if
chduree(VOL_DUREE_MANUEL)='Durée totale manuelle'
chduree(VOL_DUREE_AUTO)  ='Durée totale auto'
chduree(VOL_DUREE_SEMIAUTO)  ='Durée totale semi-auto'
if (rang==RANG_MAITRE_AFF) print '(T10,A)',trim(chduree(t_vol%type_duree))

! initialisation volume: information sur le volume global seulement, calcul dt et init
!  maillage non uniforme
call volume_preinit(t_vol)


return

999 print '(/,t10,"Erreur d''extraction des données de la ligne !!!")'
    call lecture_stop(t_fich)

end Subroutine volume_lecture


! ==========================================================
! == Subroutine volume_lecture                            ==
! == Ecriture dans des données pour TRIDIMO               ==
!===========================================================	
subroutine volume_ecriture(f, v)

! === Déclaration des arguments ===
type(st_fichier), intent(inout)       :: f
type(st_volume ), intent(inout)       :: v

! ===  Déclaration des variables  ===
character (len=LEN_CHAINE)  :: premiermot
integer  :: status
character (len=2)  :: ch1(3),ch2(2)
character (len=4)  :: ch3(2)

! ===   Début des traitements   ===
ch1=(/'1D','2D','3D'/);
ch2=(/'TM','TE'/);
ch3=(/'EZHY','EYHZ'/);

!write(f%unite,'(T3,A,3x,EN13.4E2)')  'fmax',v%fmax
!write(f%unite,'(T3,A,3x,EN13.4E2)')  'vmin',v%vmin
!if (v%type_volume == VOL_3D) then
!  write(f%unite,'(T3,A,3x,A)')  'espace',trim(ch1(v%type_volume))
!else if (v%type_volume == VOL_2D) then
!  write(f%unite,'(T3,3(A,3x))')  'espace',trim(ch1(v%type_volume)), &
!                                          trim(ch2(v%type_polar))
!else
!  write(f%unite,'(T3,3(A,3x))')  'espace',trim(ch1(v%type_volume)), &
!                                          trim(ch3(v%type_polar))
!end if

!write(f%unite,'(T3,A,3x,3EN15.6E2)')  BV_LXLYLZ,v%lxyz(1:3)
write(f%unite,'(T3,A,3x,3I5)')  BV_NXNYNZ,v%nx,v%ny,v%nz
write(f%unite,'(T3,A,3x,3EN15.6E2)')  BV_DXDYDZ,v%dxyz(1:3)
if (v%cdtmax /=CDTMAX_DEFAUT) &
  write(f%unite,'(T3,A,3x,F7.4)')  BV_CDTMAX,v%cdtmax

write(f%unite,'(T3,A,3x,EN13.4E2)')  'duree_totale',v%tlimit

if (any(v%origine_cell/=-1)) &
  write(f%unite,'(T3,A,3x,3I7)')  BV_ORIGINE_CELL,v%origine_cell(1:3) 

if (any(v%origine/=0.0)) &
  write(f%unite,'(T3,A,3x,3EN15.6E2)')  BV_ORIGINE_REELLE,v%origine(1:3) 

if (v%flag_coordreel) &
  write(f%unite,'(T3,A)') ID_COORD_REEL

if (v%epsilon_r>1.0) & 
  write(f%unite,'(T3,A,3x,F7.3)')  BV_EPSILON_R,v%epsilon_r
  
if (v%mu_r>1.0) &
  write(f%unite,'(T3,A,3x,F7.3)')  BV_MU_R,v%mu_r
  
if (v%sigma_e>0.0) & 
  write(f%unite,'(T3,A,3x,EN13.4E2)')  BV_SIGMA_E,v%sigma_e

if (v%sigma_h>0.0) &
  write(f%unite,'(T3,A,3x,EN13.4E2)')  BV_SIGMA_H,v%sigma_h


if (v%flag_nonmoy_eps) write(f%unite,'(T3,A)') 'non_moyennage_permittivite'
if (v%flag_nonmoy_mu) write(f%unite,'(T3,A)') 'non_moyennage_permeabilite'
if (v%flag_nonmoy_sigmae) write(f%unite,'(T3,A)') 'non_moyennage_conductivite_electrique'
if (v%flag_nonmoy_sigmah) write(f%unite,'(T3,A)') 'non_moyennage_conductivite_magnetique'

end Subroutine volume_ecriture


! ==========================================================
! == Subroutine critere_stabilite_yee_3d                  ==
!===========================================================	
real function critere_stabilite_yee_3d(v) result(dt)
type(st_volume), intent(inout) :: v

dt = v%cdtmax/(CVIDE*SQRT(1./v%celx**2 + &
			  1./v%cely**2 + &
			  1./v%celz**2))
end function critere_stabilite_yee_3d

! ==========================================================
! == Subroutine volume_preinit                            ==
! == Initialisation des variables, lorsque                ==
! == qu'il n'y a pas de lecture du fichier d'entrée       ==
!===========================================================	
subroutine volume_preinit(v)

type(st_volume), intent(inout) :: v

if (v%flag_maillage_nu) call volume_NU_init(v)
v%celx = v%dxyz(1);
v%cely = v%dxyz(2);
v%celz = v%dxyz(3);
v%nx=v%nxyz(1);
v%ny=v%nxyz(2);
v%nz=v%nxyz(3);

! Les bornes du volumes
v%ivold = 1
v%ivolf = v%nx
v%jvold = 1
v%jvolf = v%ny
v%kvold = 1
v%kvolf = v%nz
v%borne_volume(XMIN) = v%ivold ; v%borne_volume(XMAX) = v%ivolf
v%borne_volume(YMIN) = v%jvold ; v%borne_volume(YMAX) = v%jvolf
v%borne_volume(ZMIN) = v%kvold ; v%borne_volume(ZMAX) = v%kvolf

v%dt = critere_stabilite_yee_3d(v)

v%ntinc = nint(v%tlimit / v%dt)
v%epsilon = v%epsilon_r*EPSILON_0
v%mu = v%mu_r*MU_0
if (rang==RANG_MAITRE_AFF) then
  Print '(T10," DT=",ES16.7E2," ; Durée et Nbre d''itérations =",EN13.4,I7)',v%dt,v%tlimit,v%ntinc
  end if

end subroutine volume_preinit

! ==========================================================
! == Subroutine volume_init                     ==
! == Initialisation des variables, lorsque                ==
! == qu'il n'y a pas de lecture du fichier d'entrée       ==
!===========================================================	
subroutine volume_init(v)

type(st_volume), intent(inout) :: v

v%nx=v%nxyz(1);
v%ny=v%nxyz(2);
v%nz=v%nxyz(3);

! Les bornes du volumes
v%ivold = 1
v%ivolf = v%nx
v%jvold = 1
v%jvolf = v%ny
v%kvold = 1
v%kvolf = v%nz
v%borne_volume(XMIN) = v%ivold ; v%borne_volume(XMAX) = v%ivolf
v%borne_volume(YMIN) = v%jvold ; v%borne_volume(YMAX) = v%jvolf
v%borne_volume(ZMIN) = v%kvold ; v%borne_volume(ZMAX) = v%kvolf

end subroutine volume_init

	
! ==========================================================
! == Subroutine volume_precalcul                          ==
! == Calculs avant les itérations temporelles             ==
! ==========================================================
subroutine volume_precalcul(v)

type(st_volume), intent(inout) :: v
				
! les bornes du volume sont connues (tient compte des PML)
call volume_maillage(v)

end subroutine volume_precalcul

! ==========================================================
! == Subroutine volume_pasE_local1                         
! == Retourne le pas spatial lié au champ électrique à 
! == une position donnée et pour une direction donnée  
! ==========================================================
function volume_pasE_local1(v,i,dir) result(pas)

type(st_volume), intent(in) :: v
integer, intent(in)  :: i, dir
real  :: pas
pas = v%dxyz(dir)
if (v%flag_maillage_nu) then
    pas = v%mnu(dir)%pas_e(i)
end if

end function volume_pasE_local1

! ==========================================================
! == Subroutine volume_pasE_local_ijk                     ==
! == Retourne le pas spatial lié au champ électrique à 
! == une position donnée et pour une direction donnée  
! ==========================================================
function volume_pasE_local_ijk(v,i,j,k,dir) result(pas)

type(st_volume), intent(in) :: v
integer, intent(in)  :: i, j, k, dir				
real  :: pas
pas = v%dxyz(dir)
if (v%flag_maillage_nu) then
  select case (dir)
  case (1)
    pas = v%mnu(dir)%pas_e(i)
  case (2)
    pas = v%mnu(dir)%pas_e(j)
  case (3)
    pas = v%mnu(dir)%pas_e(k)
  end select
end if
end function volume_pasE_local_ijk

! ==========================================================
! == Subroutine volume_pasH_local1                        ==
! == Retourne le pas spatial lié au champ magnétique à 
! == une position donnée et pour une direction donnée  
! ==========================================================
function volume_pasH_local1(v,i,dir) result(pas)

type(st_volume), intent(in) :: v
integer, intent(in)  :: i, dir				
real  :: pas
pas = v%dxyz(dir)
if (v%flag_maillage_nu) then
  pas = v%mnu(dir)%pas_h(i)
end if

end function volume_pasH_local1

! ==========================================================
! == Subroutine volume_pasH_local_ijk                     ==
! == Retourne le pas spatial lié au champ magnétique à 
! == une position donnée et pour une direction donnée  
! ==========================================================
function volume_pasH_local_ijk(v,i,j,k,dir) result(pas)

type(st_volume), intent(in) :: v
integer, intent(in)  :: i, j, k, dir				
real  :: pas
pas = v%dxyz(dir)
if (v%flag_maillage_nu) then
  select case (dir)
  case (1)
    pas = v%mnu(dir)%pas_h(i)
  case (2)
    pas = v%mnu(dir)%pas_h(j)
  case (3)
    pas = v%mnu(dir)%pas_h(k)
  end select
end if

end function volume_pasH_local_ijk

! ==========================================================
! == Subroutine volume_surfaceE_local                     ==
! == Retourne la surface d'une cellule centrée sur une 
! == composante de E à une position donnée et pour un plan 
! == Surface transverce à la composante de E
! ==========================================================
function volume_surfaceE_local(v,i,j,k,plan) result(surf)

type(st_volume), intent(in) :: v
integer, intent(in)  :: i, j, k, plan				
real  :: surf


surf = 0.0
if (plan == PLAN_XOY) then
  surf = v%celx * v%cely
  if (v%flag_maillage_nu) then
    surf = v%mnu(DIR_X)%pas_e(i)*v%mnu(DIR_Y)%pas_e(j)
  end if
else if (plan == PLAN_YOZ) then 
  surf = v%cely * v%celz
  if (v%flag_maillage_nu) then
    surf = v%mnu(DIR_Y)%pas_e(j)*v%mnu(DIR_Z)%pas_e(k)
  end if
else if (plan == PLAN_XOZ) then
  surf = v%celx * v%celz
  if (v%flag_maillage_nu) then
    surf = v%mnu(DIR_X)%pas_e(i)*v%mnu(DIR_Z)%pas_e(k)
  end if
end if

end function volume_surfaceE_local
! ==========================================================
! == Subroutine volume_surfaceH_local                     ==
! == Retourne la surface d'une cellule centrée sur une 
! == composante de H à une position donnée et pour un plan  
! == Surface transverce à la composante de H
! ==========================================================
function volume_surfaceH_local(v,i,j,k,plan) result(surf)

type(st_volume), intent(in) :: v
integer, intent(in)  :: i, j, k, plan				
real  :: surf

surf = 0.0
if (plan == PLAN_XOY) then
  surf = v%celx * v%cely
  if (v%flag_maillage_nu) then
    surf = v%mnu(DIR_X)%pas_e(i)*v%mnu(DIR_Y)%pas_e(j)
  end if
else if (plan == PLAN_YOZ) then 
  surf = v%cely * v%celz
  if (v%flag_maillage_nu) then
    surf = v%mnu(DIR_Y)%pas_e(j)*v%mnu(DIR_Z)%pas_e(k)
  end if
else if (plan == PLAN_XOZ) then
  surf = v%celx * v%celz
  if (v%flag_maillage_nu) then
    surf = v%mnu(DIR_X)%pas_e(i)*v%mnu(DIR_Z)%pas_e(k)
  end if
end if

end function volume_surfaceH_local

! ==========================================================	
! == Subroutine volume_calcul                             ==
! == Calcul pendant les itérations temporelles            ==
! ==========================================================
subroutine volume_calcul(v)

implicit none
				
type(st_volume), intent(inout) :: v

end subroutine volume_calcul


! ==========================================================	
! == Subroutine volume_affichage                          ==
! == Affichage de résultats dans le terminal              ==
! ==========================================================
subroutine volume_affichage(v)

type(st_volume), intent(inout) :: v

print '(/,t3,"Rang=",I5," ; Limites du volume de calcul:")'
print '(t10,"Rang=",I5," ; Selon X, de ivold=", i5, "; ivolf=", i5)', rang,v%ivold, v%ivolf
print '(t10,"Rang=",I5," ; Selon Y, de jvold=", i5, "; jvolf=", i5)', rang,v%jvold, v%jvolf
print '(t10,"Rang=",I5," ; Selon Z, de kvold=", i5, "; kvolf=", i5)', rang,v%kvold, v%kvolf
if (rang==RANG_MAITRE_AFF) then
  print '(t10,"Pas temporel, dt = ",EN16.7e2, " s")', v%dt
  if (v%flag_schema_expon) print '(t10,"Schéma numérique de type exponentiel pour les pertes.")'
end if
end subroutine volume_affichage



! ==========================================================
! == Subroutine volume_maillage                           ==
! == Calcul des tableaux x, y, z, xo, yo, zo              ==
! ==========================================================
subroutine volume_maillage(v)

type(st_volume), intent(inout) :: v

allocate(v%xo(v%ivold-1:v%ivolf+1)); v%xo = 0
allocate(v%x (v%ivold-1:v%ivolf+1)); v%x  = 0
call volume_maillage_un_axe(v%ivold-1,v%ivolf+1,v%dxyz(1),v%xo,v%x)

allocate(v%yo(v%jvold-1:v%jvolf+1)); v%yo = 0
allocate(v%y (v%jvold-1:v%jvolf+1)); v%y  = 0
call volume_maillage_un_axe(v%jvold-1,v%jvolf+1,v%dxyz(2),v%yo,v%y)

allocate(v%zo(v%kvold-1:v%kvolf+1)); v%zo = 0
allocate(v%z (v%kvold-1:v%kvolf+1)); v%z  = 0
call volume_maillage_un_axe(v%kvold-1,v%kvolf+1,v%dxyz(3),v%zo,v%z)

end subroutine volume_maillage

! ==========================================================
! == Subroutine volume_maillage_un_axe                    ==
! == Calcul le maillage selon une seule dimension l       ==
! ==========================================================
subroutine volume_maillage_un_axe(deb,fin,pas,lo,l)

integer, intent (in) :: deb,fin
real, intent (in) :: pas
real, dimension (deb:fin), intent (out) :: lo,l

integer :: i

do i=deb,fin
 lo(i) = (i*1.0-1.0)*pas
 l(i)  = (i*1.0-0.5)*pas
end do

end subroutine volume_maillage_un_axe


! ==========================================================
! == Subroutine volume_maillage_NU                        ==
! == Calcul des tableaux x, y, z, xo, yo, zo              ==
! ==========================================================
subroutine volume_NU_init(v)

! === Déclaration des arguments ===
type(st_volume), intent(inout) :: v

! === Déclaration des variables locales ===
integer :: i

! === DÉBUT DES TRAITEMENTS ===
if (rang==RANG_MAITRE_AFF) &
  print '(T5,"--> MAILLAGE NON UNIFORME :")'
do i=1,3 ! balayage sur les trois direction x,y,z
  if (v%mnu(i)%nb_pas>0) then 
    ! Evaluation du nombre de cellules dans chaque direction
    v%nxyz(i) = sum(v%mnu(i)%liste_cell)
    v%dxyz(i) = minval(v%mnu(i)%liste_pas)
  else
    allocate(v%mnu(i)%liste_cell(1))    
    allocate(v%mnu(i)%liste_pas(1))
    allocate(v%mnu(i)%fr(1))
    v%mnu(i)%liste_cell(1) = v%nxyz(i)    
    v%mnu(i)%liste_pas(1)  = v%dxyz(i)    
    v%mnu(i)%fr(1) = 1
    v%mnu(i)%nb_pas = 1
  end if
  v%mnu(i)%dmin = v%dxyz(i)
  v%mnu(i)%ncell= v%nxyz(i)
end do
if (rang==RANG_MAITRE_AFF) THEN
  print '(T10,"Pas spatiaux min =",3ES15.6)',v%dxyz
  print '(T10,"Nbre de cellules =",3I7,/)',v%nxyz

  print '(T10,"Maillage X ; liste des pas =",15ES10.2)',v%mnu(1)%liste_pas(:)
  print '(T10,"Maillage X ; liste cell=",30I4)',v%mnu(1)%liste_cell(:) 
  print '(T10,"Maillage X ; facteur de rafinement=",30I4)',v%mnu(1)%fr(:) 
  print '(T10,"Maillage Y ; liste des pas =",15ES10.2)',v%mnu(2)%liste_pas(:)
  print '(T10,"Maillage Y ; liste cell=",30I4)',v%mnu(2)%liste_cell(:) 
  print '(T10,"Maillage Y ; facteur de rafinement=",30I4)',v%mnu(2)%fr(:) 
  print '(T10,"Maillage Z ; liste des pas =",15ES10.2)',v%mnu(3)%liste_pas(:)
  print '(T10,"Maillage Z ; liste cell=",15I4)',v%mnu(3)%liste_cell(:) 
  print '(T10,"Maillage Z ; facteur de rafinement=",30I4)',v%mnu(3)%fr(:) 
END IF
end subroutine volume_NU_init


! ==========================================================
! == Subroutine volume_maillage_NU_init                   ==
! ==========================================================
subroutine volume_maillage_NU_init(v)
! === Déclaration des arguments ===
type(st_volume), intent(inout)       :: v

call maillage_NU_init(v%mnu(1),v%ivold,v%ivolf+1)
call maillage_NU_init(v%mnu(2),v%jvold,v%jvolf+1)
call maillage_NU_init(v%mnu(3),v%kvold,v%kvolf+1)

end subroutine volume_maillage_NU_init
! ==========================================================
! == Subroutine maillage_NU_init                          ==
! == Calcul des tableaux x, y, z, xo, yo, zo              ==
! == Les bornes du volume, y compris incluant les frontières
! == de type PML sont déjà calculées
! ==========================================================

subroutine maillage_NU_init(m,imin,imax)
! === Déclaration des arguments ===
type(st_maillage_nu), intent(inout)       :: m
integer, intent(in)  :: imin,imax
! === Déclaration des variables locales ===
integer :: n1,n2,i

! === DÉBUT DES TRAITEMENTS ===
!print '("imin,imax=",2I5)',imin,imax
allocate(m%pas_e(imin:imax))
allocate(m%pas_h(imin:imax))
allocate(m%c_e(imin:imax)) ! coordonnées pour les e
allocate(m%c_h(imin:imax)) ! coordonnées pour les h

! Initialisation des pas en dehors du volume utile
m%pas_e(imin:1) = m%liste_pas(1)
m%pas_h(imin:1) = m%liste_pas(1)
m%pas_e(m%ncell:imax) = m%liste_pas(m%nb_pas)
m%pas_h(m%ncell:imax) = m%liste_pas(m%nb_pas)
!print '("imin,m%ncell,imax",3I5)',imin,m%ncell,imax

n1=1
n2=1
do i=1,m%nb_pas
  n2=n1+m%liste_cell(i)-1
  m%pas_h(n1:n2)=m%liste_pas(i)
!  print '("n1,n2=",2I5,"m%liste_pas(i)=",ES15.5)',n1,n2,m%liste_pas(i)
  n1 = n2+1  
end do

m%c_e(imin) = (imin-1)*m%liste_pas(1)
m%c_h(imin) = m%c_e(imin)+0.5*m%liste_pas(1)

do i=imin+1,imax
  m%pas_e(i)= 0.5*(m%pas_h(i-1)+m%pas_h(i))
  m%c_e(i)  = m%c_e(i-1)+m%pas_h(i-1)
  m%c_h(i)  = m%c_e(i)+0.5*m%pas_h(i)
end do

if (rang==RANG_MAITRE_AFF) THEN
  !print '(T10,"PAS H=",10ES10.2)', m%pas_h
  !print '(T10,"PAS E=",10ES10.2)', m%pas_e
  !print '(T10,"Coordonnées H=",10ES10.2)', m%c_h
  !print '(T10,"Coordonnées E=",10ES10.2)', m%c_e
  n1=1
  do i=1,m%nb_pas+1
    print '(T10,"Interface =",I5," ;PAS E=",ES12.4," ; PAS H=",ES12.4," ; Coord E=",ES12.4," ; Coord H=",ES12.4)', n1,&
	  m%pas_e(n1),m%pas_h(n1),m%c_e(n1),m%c_h(n1)
    if (i==m%nb_pas+1) exit
    n1=n1+m%liste_cell(i)
  end do
  
end if

end subroutine maillage_NU_init
! ==========================================================
! == Fin du module                                        ==
! ==========================================================
end module mfdtd_volume	



