!/////////////////////////////////////////////////////////////////////////////
!// Name:        tridimo.f90
!// Purpose:     Programme principal de TRIDIMO
!      Simulateur FDTD
!// Author:      Christophe Guiffaut
!// Modified by:  Christophe Guiffaut
!// Created  :     Novembre 2001
!// Lieu     : XLIM - Faculté des sciences - 123 Av. Albert Thomas - Limoges
!// Copyright:   (c) CNRS, Université de Limoges
!// Licence  : CECILL-C - Octobre 2006
!/////////////////////////////////////////////////////////////////////////////

!===========================================================
!== Programme principal : FDTD3D                          ==
!===========================================================
program  tridimo

use m_constantes, only : LEN_CHAINE

use mfdtd_fdtd3d
			     
use mfdtd_frontiere, only : frontiere_calcul_e, &
                            frontiere_calcul_h
			     
use mfdtd_champ,     only : champ_calcul_e, &
                            champ_calcul_h

use mfdtd_temps_cpu,  only :  st_temps_cpu, &
                              temps_cpu_initdefaut, &
                              temps_cpu_calcul, &
                              temps_cpu_calcul_un_param, &
                              temps_cpu_affichage


implicit none

type(st_fdtd3d), pointer :: t_fdtd3d ! Structure principale

integer  :: iter ! n° de l'itération temporel
integer  :: i,saut_aff
real     :: t,taviter ! variable pour le temps cpu

character(LEN=LEN_CHAINE) :: nom_fichier_entree
! nombre_arguments : Nb arguments sur la commande de lancement de TRIDIMO
integer  :: nombre_arguments

integer, external :: iargc
external :: getarg


! ========== DEBUT DES TRAITEMENTS ===========

!==================== OUVERTURE DU FICHIER D'ENTRÉE ================



!==============================================================================

print '("=====================================================")'
print '("== LANCEMENT OPEN-TEMSI-FD, version Décembre 2008  ==")'
#ifdef FLAG_OPENMP
print '("==                VERSION OPEN-MP                  ==")'
#endif
print '("=====================================================")'

allocate(t_fdtd3d)

! -- Recupere le nombre de d'arguments sur la ligne de commande --
nombre_arguments = iargc()
if (nombre_arguments/=1) then
  print *, 'Etourdi ! faudrait  me donner un fichier de données !!!'
  print *, 'Erreur ligne de commande : usage : tridimo nom_fichier.'
  stop
endif

! -- Recupere le nom du fichier d'entree --
call getarg(nombre_arguments,nom_fichier_entree)
print '("Nbre d''arguments",I4)',nombre_arguments
print '("Nom du fichier d''entrée =",A)',trim(nom_fichier_entree)

! -- Affecte le nom du fichier dans la structure fichier --
t_fdtd3d%t_fichier%nom_fichier = nom_fichier_entree


!--- Initialisation des temps de calcul ---
call temps_cpu_calcul(t)
call temps_cpu_initdefaut(t_fdtd3d%t_temps_cpu)


print '(//,"===  Initialisation par défaut ...  ===")'
call fdtd3d_initdefaut(t_fdtd3d)

print '(//,"===  Lecture du fichier d''entree...  ===")'
call fdtd3d_lecture(t_fdtd3d)

print '(//,"===  Initialisation des variables et tableaux...  ===")'
call fdtd3d_init(t_fdtd3d)

print '(//,"===  Pre-calcul...  ===")'
call fdtd3d_precalcul(t_fdtd3d)

print '(//,"=== Affichage des donnees ===")'
call fdtd3d_affichage(t_fdtd3d)

call temps_cpu_calcul_un_param(t,t_fdtd3d%t_temps_cpu%t_pre_calcul)

print '(//,"===  Calcul...  ===")'

! =================== DÉBUT DES ITÉRATIONS TEMPORELLES ========================
iter = 1;
write(6, '("Iteration n°: ", I8, " /", I8)') 0, t_fdtd3d%t_volume%ntinc
taviter = t;

saut_aff = 10
if (t_fdtd3d%t_volume%ntinc>=10000) saut_aff = 10*int(t_fdtd3d%t_volume%ntinc/10000)
do iter=1,t_fdtd3d%t_volume%ntinc
  ! -- Calcul de h dans le volume en entier --
  call temps_cpu_calcul(t)
  call champ_calcul_h(t_fdtd3d%t_champ)
  call temps_cpu_calcul_un_param(t,t_fdtd3d%t_temps_cpu%t_calcul_h)

  ! -- Insertion des excitation et modifications sur H
  call fdtd3d_modif_excite_H(t_fdtd3d,iter)
  call temps_cpu_calcul_un_param(t,t_fdtd3d%t_temps_cpu%t_modif_excite_h)

  ! -- Calcul de h au frontiere du volume --
  call frontiere_calcul_h(t_fdtd3d%t_frontiere,t_fdtd3d%t_champ)
  call temps_cpu_calcul_un_param(t,t_fdtd3d%t_temps_cpu%t_frontiere_h)


  ! -- Calcul de e dans le volume en entier --
  call champ_calcul_e(t_fdtd3d%t_champ)
  call temps_cpu_calcul_un_param(t,t_fdtd3d%t_temps_cpu%t_calcul_e)


  ! -- Insertion des excitation et modifications sur E
  call fdtd3d_modif_excite_E(t_fdtd3d,iter)
  call temps_cpu_calcul_un_param(t,t_fdtd3d%t_temps_cpu%t_modif_excite_e)

  ! -- Calcul de e au frontiere du volume --
  call frontiere_calcul_e(t_fdtd3d%t_frontiere,t_fdtd3d%t_champ)
  call temps_cpu_calcul_un_param(t,t_fdtd3d%t_temps_cpu%t_frontiere_e)


  ! -- Calcul annexe et stockage --
  call fdtd3d_calcul_annexe(t_fdtd3d,iter)
  call temps_cpu_calcul_un_param(t,t_fdtd3d%t_temps_cpu%t_calcul_annexe)

  ! affiche n° itération tous les 10
  if (MOD(iter,saut_aff) == 0) then
    write(6, '("Iteration n°: ", I8, " /", I8," ; Temps écoulé=",F10.2,"s /",F12.2,"s")') &
                iter, t_fdtd3d%t_volume%ntinc,t-taviter,(t-taviter)*t_fdtd3d%t_volume%ntinc/iter
!    write(6, '(a6)',advance='no') REPEAT(ACHAR(8),6)
!    write(6, '(TL6)',advance='no')

  end if

enddo

!===================== FIN DES ITÉRATIONS TEMPORELLES =========================

print '(//,"===  Post-calcul...  ===")'

call temps_cpu_calcul(t)
call fdtd3d_postcalcul(t_fdtd3d)

print '(//,"===  Impression... ===")'
call fdtd3d_impression(t_fdtd3d)

print '(//,"===  Fermeture des fichiers de sortie... ===")'
call fdtd3d_fermeture_fichier(t_fdtd3d)

call temps_cpu_calcul_un_param(t,t_fdtd3d%t_temps_cpu%t_post_calcul)

! Temps final
call temps_cpu_calcul(t_fdtd3d%t_temps_cpu%t_fin)

print '(//,"===  Les temps de calcul  ===")'
call fdtd3d_temps_cpu_affichage(t_fdtd3d)

! -- Deallocation de la memoire et fermeture des fichers --
print '(//,"===  Libère la mémoire... ===")'
call fdtd3d_deallocate(t_fdtd3d)
deallocate(t_fdtd3d)

print '( "C''était le simulateur OPEN-TEMSI-FD du laboratoire XLIM, projet OSA/CEM")'
print '( "Logiciel sous Licence CECILL-C, CNRS UMR 6172 - Université de Limoges !")'

#ifdef FLAG_OPENMP
print '("==                VERSION OPEN-MP                  ==")'
#endif


end program tridimo	



